/* A0Event.m
 *
 *
 * 19970211 BS creation
 */

#import "A0Event.h"
#import "A0EventInspector.h"
#import "../General.bproj/NSUserDefaults+Colors.h"
#import "../General.bproj/UniqueString.h"
#import "../General.bproj/Protocols.h"
#import "../General.bproj/PajeType.h"

static NSMutableDictionary *colorDict;

UniqueString *A0EventEntityType;

#define COLORDEFAULTKEY [[[self entityType] name] stringByAppendingString:@" Colors"]

@implementation A0Event
+ (void)initialize
{
    if (!A0EventEntityType)
        A0EventEntityType = [[PajeEventType typeWithName:@"Event" containerType:nil] retain];
    if (![self colorDict]) {
        id dict = nil;
#ifndef GNUSTEP
        dict = [[NSUserDefaults standardUserDefaults] unarchiveObjectForKey:COLORDEFAULTKEY];
#endif
        if (!dict)
            dict = [self defaultColorDict];
        [self setColorDict:dict];
    }
}

+ (NSString *)name
{
    return A0EventEntityType;
}

+ (id)entityType
{
    return A0EventEntityType;
}

+ (NSMutableDictionary *)defaultColorDict
{
    return [NSMutableDictionary dictionary];
}

+ (NSMutableDictionary *)colorDict
{
    return colorDict;
}

+ (void)setColorDict:(id)dict
{
    id tmp = [dict retain];
    [colorDict release];
    colorDict = tmp;
}

+ (NSArray *)allNames
{
    return [[self colorDict] allKeys];
}

+ (NSColor *)colorForName:(NSString *)ev_name
{
    return [[self colorDict] objectForKey:ev_name];
}

+ (void)setColor:(NSColor *)color forName:(NSString *)ev_name
{
    NSColor *orig_color = [self colorForName:ev_name];

    if (!orig_color || ![color isEqual:orig_color]) {
        [[self colorDict] setObject:color forKey:ev_name];
        [[NSUserDefaults standardUserDefaults] setArchivedObject:[self colorDict] forKey:COLORDEFAULTKEY];
    }
}

+ (id)eventWithObjects:(NSArray *)objects forKeys:(NSArray *)keys
{
    return [[[self alloc] initWithObjects:objects forKeys:keys] autorelease];
}

- (id)initWithObjects:(NSArray *)objects forKeys:(NSArray *)keys
{
    self = [super init];
    fieldDict = [[NSMutableDictionary alloc] initWithObjects:objects
                                                     forKeys:keys];
    name = [fieldDict objectForKey:@"Name"];
    node = [fieldDict objectForKey:@"Node"];
    thread = [fieldDict objectForKey:@"Thread"];
    time = [fieldDict objectForKey:@"Time"];
    return self;
}

- (id)mutableCopyWithZone:(NSZone *)zone
{
    A0Event *copy = [[[self class] allocWithZone:zone] init];
    copy->fieldDict = [fieldDict mutableCopyWithZone:zone];
    [copy setObject:name forKey:@"Name"];
    return copy;
}
- (id)initWithName:(NSString *)n
{
    self = [super init];
    fieldDict = [[NSMutableDictionary alloc] initWithObjectsAndKeys:n, @"Name", nil];
    name = n;
    return self;
}

- (void)dealloc
{
/*
    [name release];
    [node release];
    [thread release];
    [time release];
*/
    [fieldDict release];
    [super dealloc];
}

//
// Methods for being an NSMutableDictionary
//

- (unsigned)count
{
    return [fieldDict count];
}

- (id)objectForKey:(id)aKey
{
    if ([aKey isEqualToString:@"Name"])
        return [self name];
    else if ([aKey isEqualToString:@"Node"])
        return [self node];
    else if ([aKey isEqualToString:@"Thread"])
        return [self thread];
    else if ([aKey isEqualToString:@"Time"])
        return [self time];
    else if ([aKey isEqualToString:@"Duration"])
        return [self duration];

    return [fieldDict objectForKey:aKey];
}

- (NSEnumerator *)keyEnumerator
{
    return [fieldDict keyEnumerator];
}

- (NSArray *)allKeys
{
    return [fieldDict allKeys];
}

- (void)setObject:(id)anObject forKey:(id)aKey
{
    [fieldDict setObject:anObject forKey:aKey];
    name = [fieldDict objectForKey:@"Name"];
    node = [fieldDict objectForKey:@"Node"];
    thread = [fieldDict objectForKey:@"Thread"];
    time = [fieldDict objectForKey:@"Time"];
}

- (void)removeObjectForKey:(id)theKey
{
    [fieldDict removeObjectForKey:theKey];
}


//
// PajeEvent protocol
//
- (NSColor *)color
{
    NSColor *c = [fieldDict objectForKey:@"Color"];
    if (c) return c;
    c = [[self class] colorForName:[self name]];
    if (c) return c;
    c = [NSColor yellowColor];
    [[self class] setColor:c forName:[self name]];
    return c;
}

- (void)setColor:(NSColor *)c
{
    [[self class] setColor:c forName:[self name]];
}

- (void)takeColorFrom:sender
{
    [self setColor:[sender color]];
}

- (void)inspect
{
    Class inspectorClass = nil;
    Class class = [self class];

    while (!inspectorClass && !(class == [class superclass])) {
        NSString *inspName = [NSStringFromClass(class) stringByAppendingString:@"Inspector"];
        inspectorClass = NSClassFromString(inspName);
        class = [class superclass];
    }
    if (inspectorClass)
        [[inspectorClass inspector] inspect:self];
}

- (void)inspect:(id)sender
{
    [self inspect];
}

- (id)entityType
{
    return A0EventEntityType;
}

- (NSString *)printForDebugger
{
    return [fieldDict description];
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"%@ in thread %@ at time %@",
        [self name], [self thread], [[self time] description]];
}

int CTR=0;

- (id)name            {CTR++;return name;}
- (id)node            {CTR++;return node;}
- (id)sourceNode      {return [self node];}
- (id)destNode        {return [self node];}
- (id)thread          {CTR++;return thread;}
- (id)sourceThread    {return [self thread];}
- (id)destThread      {return [self thread];}
- (NSDate *)time      {CTR++;return time;}
/*
- (id)name            {CTR++;return [fieldDict objectForKey:@"Name"];}
- (id)node            {CTR++;return [fieldDict objectForKey:@"Node"];}
- (id)thread          {CTR++;return [fieldDict objectForKey:@"Thread"];}
- (NSDate *)time      {CTR++;return [fieldDict objectForKey:@"Time"];}
*/
- (id)service         {return [thread serviceId];}
- (id)sourceService   {return [self service];}
- (id)destService     {return [self service];}
- (NSDate *)startTime {return [fieldDict objectForKey:@"Time"];}
- (NSDate *)endTime   {return [fieldDict objectForKey:@"Time"];}
- (NSDate *)firstTime {return [fieldDict objectForKey:@"Time"];}
- (NSDate *)lastTime  {return [fieldDict objectForKey:@"Time"];}
- (id)duration
{
    return [NSString stringWithFormat:@"%.6f",
            [[self lastTime] timeIntervalSinceDate:[self firstTime]]];
}
- (id)serialNo        {return [fieldDict objectForKey:@"SerialNo"];}
- (NSArray *)relatedEntities{return [fieldDict objectForKey:@"RelatedEntities"];}

- (void)setName:(id)o   { [self setObject:o forKey:@"Name"]; }
- (void)setNode:(id)o   { [self setObject:o forKey:@"Node"]; }
- (void)setThread:(id)o { [self setObject:o forKey:@"Thread"]; }
- (void)setTime:(id)o   { [self setObject:o forKey:@"Time"]; }

- (void)setRelatedEntities:(NSArray *)relatedEntities
{
    [self setObject:relatedEntities forKey:@"RelatedEntities"];
}

- (void)setRelatedEntity:(id)entity
{
    [self setObject:[NSMutableArray arrayWithObject:entity]
             forKey:@"RelatedEntities"];
}

- (void)addRelatedEntity:(id)entity
{
    NSMutableArray *related = [self relatedEntities];

    if (related)
        [related addObject:entity];
    else
        [self setRelatedEntity:entity];
}

- (void)removeRelatedEntity:(id)entity
{
    NSMutableArray *related = [self relatedEntities];

    if (related)
        [related removeObject:entity];
}

- (id)container {
    return [self thread];
//    return [NSArray arrayWithObjects:[self node], [NSNumber numberWithInt:[[self thread] intValue]], nil];
}
+ (id)hierarchy {
    static id hier = nil;
    if (!hier)
        hier = [[NSArray alloc] initWithObjects:@"Node", @"Thread", nil];
    return hier;
}
- (id)hierarchy { return [[self class] hierarchy]; }

- (int)imbricationLevel { return 0; }

+ (PajeDrawingType)drawingType { return PajeEventDrawingType; }
- (PajeDrawingType)drawingType { return PajeEventDrawingType; }

- (NSArray *)fieldNames
{
    return [self allKeys];
}

- (id)valueOfFieldNamed:(NSString *)fieldName
{
    return [self objectForKey:fieldName];
}


// NSCoding Protocol
- (void)encodeWithCoder:(NSCoder *)coder
{
//    [super encodeWithCoder:coder];
/*
    [coder encodeObject:name];
    [coder encodeObject:node];
    [coder encodeObject:thread];
    [coder encodeObject:time];
*/
    [coder encodeObject:fieldDict];
}

- (id)initWithCoder:(NSCoder *)coder
{
//    self = [super initWithCoder:coder];

/*
    name = [[coder decodeObject] retain];
    node = [[coder decodeObject] retain];
    thread = [[coder decodeObject] retain];
    time = [[coder decodeObject] retain];
*/
    fieldDict = [[coder decodeObject] retain];
    name = [fieldDict objectForKey:@"Name"];
    node = [fieldDict objectForKey:@"Node"];
    thread = [fieldDict objectForKey:@"Thread"];
    time = [fieldDict objectForKey:@"Time"];

    return self;
}

@end