#import "A0EventInspector.h"
#import "../General.bproj/SourceCodeReference.h"
#import "../A0bSimul.bproj/SourceTextController.h"
#import "../General.bproj/PajeType.h"

NSPoint lastWindowPosition;

@implementation A0EventInspector

static NSMutableArray *allInstances;

+ (A0EventInspector *)inspector
{
    NSEnumerator *instEnum;
    A0EventInspector *inspector;
    if (!allInstances)
        allInstances = [[NSMutableArray alloc] init];
    instEnum = [allInstances objectEnumerator];
    while (inspector = [instEnum nextObject]) {
        if ([inspector isReusable]) {
            inspector->isa = self;
            return inspector;
        }
    }
    inspector = [[self alloc] init];
    [allInstances addObject:inspector];
    return inspector;
}

- (void)windowWillClose:(NSNotification *)notification
{
    if ([notification object] == window) {
        [allInstances removeObjectIdenticalTo:self];
        [self autorelease];
    }
}

- (id)init
{
    Class class = [self class];
    BOOL loaded = NO;
    NSDictionary *nameTable = [NSDictionary dictionaryWithObjectsAndKeys:
                                  self, @"NSOwner", nil];

    while (!loaded && (class != [class superclass])) {
        NSBundle *bundle = [NSBundle bundleForClass:class];
#ifdef GNUSTEP
        NSString *filename = @"A0EventInspector";
        loaded = [bundle loadNibFile:filename
	    	   externalNameTable:nameTable
		            withZone:[self zone]];
#else
        NSString *filename = [bundle pathForResource:@"A0EventInspector" ofType:@"nib"];
        if (filename)
            loaded = [NSBundle loadNibFile:filename
                         externalNameTable:nameTable
                                  withZone:[self zone]];
#endif
        class = [class superclass];
    }
    if (!loaded) {
        NSRunAlertPanel(@"Can't find file",
                        @"Interface description file '%@' not found",
                        nil, nil, nil, @"A0EventInspector.nib");
        [self release];
        return nil;
    }

    top = NSMaxY([fieldBox frame]);
    bottom = NSMinY([relatedEntitiesBox frame]);

    [fieldBox retain];
    [fieldBox removeFromSuperview];
    [fileBox retain];
    [fileBox removeFromSuperview];
    [relatedEntitiesBox retain];
    [relatedEntitiesBox removeFromSuperview];

    return self;
}

- (void)dealloc
{
    if (inspectedEntity) [inspectedEntity release];
    [nonDisplayedFields release];
    [fieldBox release];
    [fileBox release];
    [relatedEntitiesBox release];
    [super dealloc];
}

- (void)reset
{
    [nameField retain];
    [colorField retain];
    [reuseButton retain];
    [[[[[window contentView] subviews] mutableCopy] autorelease] makeObjectsPerformSelector:@selector(removeFromSuperview)];
    [[window contentView] addSubview:nameField];
    [[window contentView] addSubview:colorField];
    [[window contentView] addSubview:reuseButton];
    [nameField release];
    [colorField release];
    [reuseButton release];

    top = NSMaxY([fieldBox frame]);
    bottom = NSMinY([relatedEntitiesBox frame]);
}

- (void)addSubview:(NSView *)view atBottom:(BOOL)atBottom
{
    NSRect frame;
    float ypos;

    if (!view)
        return;
    frame = [view frame];
    if (atBottom) {
        ypos = bottom;
        bottom += (NSHeight(frame) + 2);
    } else {
        ypos = top - NSHeight(frame);
        top = ypos - 2;
    }
    frame.origin.y = ypos;
    [view setFrame:frame];
    [[window contentView] addSubview:view];
}

- (void)addLastSubview:(NSView *)view
{
    NSRect frame;

    if (!view)
        return;

    frame = [view frame];
    frame.origin.y = bottom;
    frame.size.height = top - bottom;
    [view setFrame:frame];
    [[window contentView] addSubview:view];
}

- (NSBox *)boxWithTitle:(NSString *)boxTitle
            fieldTitles:(NSArray *)titles
            fieldValues:(NSArray *)values
{
    // I don't know a better way of copying a box...
    NSBox *box = /*[*/[NSUnarchiver unarchiveObjectWithData:[NSArchiver archivedDataWithRootObject:fieldBox]]/* autorelease]*/;
    NSForm *form = [[[box contentView] subviews] objectAtIndex:0];
    int index;

//#ifdef GNUSTEP
//return nil;
//#endif
    NSAssert([titles count] == [values count],
             @"count of fields and values do not match");
    
    [box setTitle:boxTitle];
    
    // Field Matrix
    while ([form cellAtIndex:0])
        [form removeEntryAtIndex:0];

    for (index = 0; index < [titles count]; index++) {
        NSString *fieldTitle = [titles objectAtIndex:index];
        NSString *fieldValue = [values objectAtIndex:index];
        NSFormCell *fieldCell;
        
//            [nonDisplayedFields removeObject:fieldName];
        fieldCell = [form addEntry:[fieldTitle stringByAppendingString:@":"]];
        [fieldCell setStringValue:[fieldValue description]];
        [fieldCell setBordered:NO];
        [fieldCell setEditable:NO];
        [fieldCell setTitleFont:[[NSFontManager sharedFontManager] convertFont:[fieldCell titleFont] toHaveTrait:NSBoldFontMask]];
    }
    [form sizeToCells];
    [box sizeToFit];
    return box;
}

- (NSBox *)boxWithTitle:(NSString *)boxTitle
            fieldTitles:(NSArray *)titles
             fieldNames:(NSArray *)names
{
    NSString *fieldName;
    NSEnumerator *fieldNameEnum = [names objectEnumerator];
    NSMutableArray *fieldValues = [NSMutableArray array];

    while (fieldName = [fieldNameEnum nextObject]) {
        id fieldValue = [inspectedEntity objectForKey:fieldName];
        if (!fieldValue) fieldValue = @"";
        [fieldValues addObject:[fieldValue description]];
        [nonDisplayedFields removeObject:fieldName];
   }
    return [self boxWithTitle:boxTitle
                  fieldTitles:titles
                  fieldValues:fieldValues];
}

- (void)addLocalFields
{
    NSBox *box;
    
    // thread id
    box = [self boxWithTitle:@"Thread Identification"
                 fieldTitles:[NSArray arrayWithObjects:@"Node", @"Thread", @"Service", @"Time", nil]
                 fieldValues:[NSArray arrayWithObjects:[inspectedEntity node], [inspectedEntity thread], [inspectedEntity service], [inspectedEntity time], nil]];
    [self addSubview:box atBottom:NO];
}

- (BOOL)isReusable
{
    return [reuseButton state];
}

- (void)setReusable:(BOOL)reuse
{
    [reuseButton setState:reuse];
}

- (void)inspect:(A0Event *)entity
{
    NSEnumerator *relatedEnum;
    SourceCodeReference *sourceRef;

    if (inspectedEntity) [inspectedEntity release];
    inspectedEntity = [entity retain];

    nonDisplayedFields = [[NSMutableSet alloc]
                          initWithArray:[entity allKeys]];

    [self reset];
    
    // global fields
    [nameField setStringValue:[inspectedEntity name]];
    [colorField setColor:[inspectedEntity color]];
    [colorField setAction:@selector(takeColorFrom:)];
    [colorField setTarget:inspectedEntity];
    [window setTitle:[[inspectedEntity entityType] name]];

    [nonDisplayedFields removeObject:@"Name"];
    [nonDisplayedFields removeObject:@"Color"];

    [self addLocalFields];
    
    // source code reference
    sourceRef = [inspectedEntity objectForKey:@"FileReference"];
    if (sourceRef) {
        NSString *filename = [sourceRef filename];
        int lineNumber = [sourceRef lineNumber];
        [filenameField setStringValue:[filename description]];
        [lineNumberField setIntValue:lineNumber];
        [self addSubview:fileBox atBottom:YES];
        [nonDisplayedFields removeObject:@"FileReference"];
    }

    [nonDisplayedFields removeObject:@"RelatedEntities"];

    // other fields
    if ([nonDisplayedFields count] > 0) {
        NSBox *box;
        NSArray *fieldTitles = [nonDisplayedFields allObjects];

        box = [self boxWithTitle:@"Other values"
                     fieldTitles:fieldTitles
                     fieldNames:fieldTitles];
        [self addSubview:box atBottom:NO];
    }

    // Related entities
    relatedEnum = [[inspectedEntity relatedEntities] objectEnumerator];
    if (relatedEnum) {
        int rowCount, colCount;
        id related;
        [relatedEntitiesMatrix getNumberOfRows:&rowCount columns:&colCount];
        while (rowCount--) {
            [relatedEntitiesMatrix removeRow:rowCount];
        }
        while (related = [relatedEnum nextObject]) {
            NSButtonCell *cell;
            [relatedEntitiesMatrix addRow];
            cell = [relatedEntitiesMatrix cellAtRow:++rowCount column:0];
            [cell setTitle:[related name]];
            [cell setTarget:related];
            [cell setAction:@selector(inspect:)];
        }
        [relatedEntitiesMatrix sizeToCells];
        [relatedEntitiesBox sizeToFit];
        [self addLastSubview:relatedEntitiesBox];
    }
    
//    lastWindowPosition = [window cascadeTopLeftFromPoint:lastWindowPosition];
//    [window display];
    [window orderFront:self];
}

- (void)showSource:(id)sender
{
    SourceCodeReference *FileReference = [inspectedEntity objectForKey:@"FileReference"];
    NSString *filename;
    NSArray *sourcePaths = [[NSUserDefaults standardUserDefaults] arrayForKey:@"PajeSourcePaths"];

    if (!FileReference) {
        NSBeep();
        return;
    }

    filename = [FileReference filename];
    if (sourcePaths && ([sourcePaths count] > 0)) {
        NSEnumerator *sourcePathEnum = [sourcePaths objectEnumerator];
        NSString *fullPath;

        while (fullPath = [sourcePathEnum nextObject]) {
            fullPath = [fullPath stringByAppendingPathComponent:filename];
            if ([[NSFileManager defaultManager] isReadableFileAtPath:fullPath]) {
//                [[NSWorkspace sharedWorkspace] openFile:fullPath];
                [[NSClassFromString(@"SourceTextController") controllerForFilename:fullPath] selectLineNumber:[FileReference lineNumber]];
                return;
            }
        }
    }
    NSRunAlertPanel(@"File Not Found",
                    @"File %@ has not been found.\nVerify the value of the defaults variable \"PajeSourcePaths\" in the Info/Preferences... panel.\nCurrently, it is set to %@",
                    @"Ok", nil, nil, filename, sourcePaths);
}
@end
