
#import "A0Comm.h"
#import "A0bSimul.h"
#import "../General.bproj/UniqueString.h"
#import "../General.bproj/Macros.h"

static NSMutableDictionary *colorDict;

// entity types
PajeEntityType *A0CommEntityType;

// Possible entity names
NSString *A0CommunicationLinkName;
NSString *A0SyncCommunicationLinkName;
NSString *A0AsyncCommunicationLinkName;

@interface A0SyncComm : A0Comm
@end
@implementation A0SyncComm
- (NSString *)name { return A0SyncCommunicationLinkName; }
@end

@interface A0AsyncComm : A0Comm
@end
@implementation A0AsyncComm
- (NSString *)name { return A0AsyncCommunicationLinkName; }
@end


@implementation A0Comm
+ (void)initialize
{
    if (self == [A0Comm class]) {
        // entity type
        A0CommEntityType   = [[PajeLinkType alloc] initWithName:@"Communication" containerType:[A0bSimul executionEntityType]];

        // entity names
        A0CommunicationLinkName         = [UniqueString stringWithString:@"Communication"];
        A0SyncCommunicationLinkName     = [UniqueString stringWithString:@"Synchronous Communication"];
        A0AsyncCommunicationLinkName    = [UniqueString stringWithString:@"Asynchronous Communication"];
    }
    // A0Entity sets the colors...
    [super initialize];
}

+ (A0Comm *)asynchronousCommunication
{
    return [[[A0AsyncComm alloc] initWithName:A0AsyncCommunicationLinkName] autorelease];
}

+ (A0Comm *)synchronousCommunication
{
    return [[[A0SyncComm alloc] initWithName:A0SyncCommunicationLinkName] autorelease];
}

+ (A0Comm *)communication
{
    return [[[self alloc] initWithName:A0CommunicationLinkName] autorelease];
}

- (id)initWithName:(NSString *)n
{
    self = [super initWithType:A0CommEntityType name:n container:nil];
    if (!self) return nil;
    sendEvent = sendCompletionEvent = recvEvent = recvCompletionEvent = nil;
    return self;
}

- (NSString *)name { return A0CommunicationLinkName; }

- (void)dealloc
{
    [sendEvent release];
    [sendCompletionEvent release];
    [recvEvent release];
    [recvCompletionEvent release];
    [super dealloc];
}

+ (PajeEntityType *)entityType       { return A0CommEntityType; }

// support for colors
+ (NSMutableDictionary *)colorDict
{
    if (!colorDict) colorDict = [[NSMutableDictionary alloc] init];
    return colorDict;
}

- (void)setSendEvent:(id<PajeEvent>)e
{
    Assign(sendEvent, e);
    container = [[[[e container] container] container] retain];
}
- (void)setSendCompletionEvent:(id<PajeEvent>)e
{
    Assign(sendCompletionEvent, e);
}
- (void)setRecvEvent:(id<PajeEvent>)e
{
    Assign(recvEvent, e);
}
- (void)setRecvCompletionEvent:(id<PajeEvent>)e
{
    Assign(recvCompletionEvent, e);
}

- (id)port
{
    if (sendEvent) return [sendEvent objectForKey:@"Port"];
    return [recvEvent objectForKey:@"Port"];
}
- (id)tag
{
    if (sendEvent) return [sendEvent objectForKey:@"Tag"];
    return [recvEvent objectForKey:@"Tag"];
}
- (id)size
{
    return [recvCompletionEvent objectForKey:@"Size"];
}

- (NSDate *)startTime { return [sendEvent time]; }
- (id)sourceNode    { return [[sendEvent thread] node]; }
- (id)sourceNodeId  { return [[[sendEvent thread] node] nodeId]; }
- (id)sourceThread  { return [sendEvent thread]; }
- (id)sourceService { return [sendEvent service]; }

- (NSDate *)endTime { return [recvCompletionEvent time]; }
- (id)destNode    { return [[recvEvent thread] node]; }
- (id)destNodeId  { return [[[recvEvent thread] node] nodeId]; }
- (id)destThread  { return [recvEvent thread]; }
- (id)destService { return [recvEvent service]; }

- (id)sourceEntityType { return [A0ThreadState entityType]; }
- (id)sourceContainer { return [sendEvent container]; }
- (id)destEntityType { return [A0ThreadState entityType]; }
- (id)destContainer { return [recvEvent container]; }

- (NSArray *)relatedEntities
{
    id array[4];
    int i = 0;
    NSArray *relatedArray;
    
    if (sendEvent) array[i++] = sendEvent;
    if (sendCompletionEvent) array[i++] = sendCompletionEvent;
    if (recvEvent) array[i++] = recvEvent;
    if (recvCompletionEvent) array[i++] = recvCompletionEvent;
    relatedArray = [[[NSArray alloc] initWithObjects:array count:i] autorelease];

    return [[super relatedEntities] arrayByAddingObjectsFromArray:relatedArray];
}
@end
