
#import "A0Entity.h"
#import "../General.bproj/NSUserDefaults+Colors.h"

#define COLORDEFAULTKEY [[[self entityType] name] stringByAppendingString:@" Colors"]
@implementation A0Entity

+ (NSMutableDictionary *)colorDict { return nil; }

+ (void)initialize
{
    NSMutableDictionary *colorDict = [self colorDict];
    if (colorDict && [colorDict count] == 0) {
        id dict = [[NSUserDefaults standardUserDefaults] unarchiveObjectForKey:COLORDEFAULTKEY];
        if (dict)
            [colorDict addEntriesFromDictionary:dict];
    }
}

- (void)dealloc
{
    if (relatedEntities) {
        [relatedEntities release];
        relatedEntities = nil;
    }
    [super dealloc];
}

// PajeTiming protocol
- (NSDate *)time      { [self _subclassResponsibility:_cmd]; return nil; }
- (NSDate *)startTime { return [self time]; }
- (NSDate *)endTime   { return [self time]; }
- (NSDate *)firstTime { return [self startTime]; }
- (NSDate *)lastTime  { return [self endTime]; }
- (id)duration
{
    return [NSString stringWithFormat:@"%.6f",
            [[self lastTime] timeIntervalSinceDate:[self firstTime]]];
}
- (id)xduration {
    return [NSNumber numberWithDouble:[[self endTime] timeIntervalSinceDate:[self startTime]]];
}

// PajeEntity protocol

+ (NSArray *)allNames { return [[self colorDict] allKeys]; }

+ (NSColor *)colorForName:(NSString *)n
{
    return [[self colorDict] objectForKey:n];
}

+ (void)setColor:(NSColor*)color forName:(NSString *)n
{
    NSColor *orig_color = [self colorForName:n];

    if (!orig_color || ![color isEqual:orig_color]) {
        [[self colorDict] setObject:color forKey:n];
        [[NSUserDefaults standardUserDefaults]
            setArchivedObject:[self colorDict] forKey:COLORDEFAULTKEY];
    }
}

- (NSColor *)color
{
    NSColor *c = [[self class] colorForName:[self name]];
    if (!c) {
        c = [NSColor yellowColor];
        [[self class] setColor:c forName:[self name]];
    }
    return c;
}

- (void)setColor:(NSColor *)c
{
    [[self class] setColor:c forName:[self name]];
}

- (void)takeColorFrom:(id)sender
{
    [self setColor:[sender color]];
}


- (NSArray *)relatedEntities
{
    if (relatedEntities)
        return relatedEntities;
    return [NSArray array];
}

- (void)addRelatedEntity:(id)entity
{
    if (relatedEntities)
        [relatedEntities addObject:entity];
    else
        relatedEntities = [[NSMutableArray arrayWithObject:entity] retain];
}

- (void)removeRelatedEntity:(id)entity
{
    if (relatedEntities) {
        [relatedEntities removeObject:entity];
        if ([relatedEntities count]) {
            [relatedEntities release];
            relatedEntities = nil;
        }
    }
}

//+ (NSString *)name { return [[self entityType] name]; }

- (id)node         { [self _subclassResponsibility:_cmd]; return nil; }
- (id)thread       { [self _subclassResponsibility:_cmd]; return nil; }
- (id)service      { [self _subclassResponsibility:_cmd]; return nil; }

- (id)sourceNode      { return [self node]; }
- (id)sourceThread    { return [self thread]; }
- (id)sourceService   { return [self service]; }
- (id)destNode        { return [self node]; }
- (id)destThread      { return [self thread]; }
- (id)destService     { return [self service]; }

- (void)inspect
{
    Class inspectorClass = nil;
    Class class = [self class];

    while (!inspectorClass && !(class == [class superclass])) {
        NSString *inspName = [NSStringFromClass(class) stringByAppendingString:@"Inspector"];
        inspectorClass = NSClassFromString(inspName);
        class = [class superclass];
    }
    if (inspectorClass)
//        [[[inspectorClass alloc] init] inspect:self];
        [[inspectorClass inspector] inspect:self];
}

- (void)inspect:(id)sender { [self inspect]; }

- (NSArray *)allKeys
{
    return [NSArray array];
}

- (id)objectForKey:(NSString *)key
{
    if ([key isEqualToString:@"Name"]) return [self name];
    if ([key isEqualToString:@"Node"]) return [self node];
    if ([key isEqualToString:@"Thread"]) return [self thread];
    if ([key isEqualToString:@"Time"]) return [self time];
    if ([key isEqualToString:@"SourceNode"]) return [self sourceNode];
    if ([key isEqualToString:@"DestNode"]) return [self destNode];
    if ([key isEqualToString:@"SourceThread"]) return [self sourceThread];
    if ([key isEqualToString:@"DestThread"]) return [self destThread];
    if ([key isEqualToString:@"StartTime"]) return [self startTime];
    if ([key isEqualToString:@"EndTime"]) return [self endTime];
    if ([key isEqualToString:@"RelatedEntities"]) return [self relatedEntities];
    if ([key isEqualToString:@"Size"]) return [self size];
    if ([key isEqualToString:@"Service"]) return [self service];
    if ([key isEqualToString:@"ThreadService"]) return [self service];
    if ([key isEqualToString:@"SourceService"]) return [self sourceService];
    if ([key isEqualToString:@"DestService"]) return [self destService];
    if ([key isEqualToString:@"SerialNo"]) return [self serialNo];
    if ([key isEqualToString:@"Duration"]) return [self duration];
    return nil;
}
- (id)hierarchy { return [[self class] hierarchy]; }
- (PajeDrawingType)drawingType { [self _subclassResponsibility:_cmd]; return 0; }

// NSCoding Protocol
- (void)encodeWithCoder:(NSCoder *)coder
{
//    [super encodeWithCoder:coder];
}

- (id)initWithCoder:(NSCoder *)coder
{
    return [super init];//WithCoder:coder];
}
@end
