
#import "A0Link.h"
#import "A0bSimul.h"
#import "../General.bproj/UniqueString.h"
#import "../General.bproj/PajeType.h"

static NSMutableDictionary *colorDict;

// entity types
PajeEntityType *A0LinkEntityType;

// Possible entity names
NSString *A0NewMutexLinkName;
NSString *A0UnlockMutexLinkName;
NSString *A0NewSemaphoreLinkName;
NSString *A0VSemaphoreLinkName;
NSString *A0NewSlaveLinkName;
NSString *A0JoinLinkName;
NSString *A0NewPipeLinkName;
NSString *A0StartRemoteThreadLinkName;
NSString *A0StartRemoteUrgentLinkName;


// Links that start in an event
@interface A0LinkFromEvent : A0Link
{
    A0Event *sourceEvent;
}
- (id)initWithType:(PajeEntityType *)t
              name:(NSString *)n
         container:(PajeContainer *)cont
             event:(id <PajeEvent>)e;
@end
@implementation A0LinkFromEvent

- (id)initWithType:(PajeEntityType *)t
              name:(NSString *)n
         container:(PajeContainer *)cont
             event:(id <PajeEvent>)e
{
    self = [super initWithType:t name:n container:cont];
    if (self != nil) {
        sourceEvent = [e retain];
    }
    return self;
}

- (void)dealloc
{
    [sourceEvent release];
    [super dealloc];
}

- (NSArray *)relatedEntities
{
    return [[super relatedEntities] arrayByAddingObject:sourceEvent];
}

- (id)sourceNode { return [[sourceEvent thread] node]; }
- (id)sourceThread { return [sourceEvent thread]; }
- (id)sourceService { return [sourceEvent service]; }
- (NSDate *)startTime { return [sourceEvent time]; }

- (id)sourceEntityType { return [sourceEvent entityType]; }
- (id)sourceContainer { return [sourceEvent container]; }
- (id)container { return [[[sourceEvent container] container] container]; }
+ (id)container { return [A0bSimul class]; }

@end

// Links that are defined by two events
@interface A0TwoEventsLink : A0LinkFromEvent
{
    A0Event *destEvent;
}
- (id)initWithType:(PajeEntityType *)t
              name:(NSString *)n
         container:(PajeContainer *)cont
       sourceEvent:(id <PajeEvent>)e1
         destEvent:(id <PajeEvent>)e2;
@end
@implementation A0TwoEventsLink

- (id)initWithType:(PajeEntityType *)t
              name:(NSString *)n
         container:(PajeContainer *)cont
       sourceEvent:(id <PajeEvent>)e1
         destEvent:(id <PajeEvent>)e2
{
    self = [super initWithType:(PajeEntityType *)t
                          name:(NSString *)n
                     container:(PajeContainer *)cont
                         event:e1];
    if (self != nil) {
        destEvent = [e2 retain];
    }
    return self;
}

- (void)dealloc
{
    [destEvent release];
    [super dealloc];
}

- (NSArray *)relatedEntities
{
    return [[super relatedEntities] arrayByAddingObject:destEvent];
}

- (id)destNode { return [[destEvent thread] node]; }
- (id)destThread { return [destEvent thread]; }
- (id)destService { return [destEvent service]; }
- (NSDate *)endTime { return [destEvent time]; }

- (id)destEntityType { return [destEvent entityType]; }
- (id)destContainer { return [destEvent container]; }
@end

// Links that go from a event to the start of a state
@interface A0EventToStartStateLink : A0LinkFromEvent
{
    A0State *destState;
}
- (id)initWithType:(PajeEntityType *)t
              name:(NSString *)n
         container:(PajeContainer *)cont
       sourceEvent:(id <PajeEvent>)e
         destState:(id <PajeState>)s;
@end
@implementation A0EventToStartStateLink

- (id)initWithType:(PajeEntityType *)t
              name:(NSString *)n
         container:(PajeContainer *)cont
       sourceEvent:(id <PajeEvent>)e
         destState:(id <PajeState>)s
{
    self = [super initWithType:t
                          name:n
                     container:cont
                         event:e];
    if (self != nil) {
        destState = [s retain];
    }
    return self;
}

- (void)dealloc
{
    [destState release];
    [super dealloc];
}

- (NSArray *)relatedEntities
{
    return [[super relatedEntities] arrayByAddingObject:destState];
}

- (id)destNode { return [[destState thread] node]; }
- (id)destThread { return [destState thread]; }
- (id)destService { return [destState service]; }
- (NSDate *)endTime { return [destState startTime]; }

- (id)destEntityType { return [destState entityType]; }
- (id)destContainer { return [destState container]; }
@end


// Links that go from a event to the end of a state
@interface A0EventToEndStateLink : A0EventToStartStateLink
@end
@implementation A0EventToEndStateLink
//- (NSDate *)endTime { return [destState endTime]; }
@end

#if 0
// Link between an a0NewMutex and the first state of the created mutex
@interface A0NewMutexLink : A0EventToStartStateLink
@end
@implementation A0NewMutexLink
- (id)initWithSourceEvent:(id <PajeEvent>)e destState:(id <PajeState>)s
{
    return [super initWithName:[A0NewMutexLinkType name]
                    entityType:A0NewMutexLinkType
                     container:[[s container] container]
                   sourceEvent:e
                     destState:s];
}
@end

// Link between an a0UnlockMutex and the unblocking event of the
// thread that gets the mutex.
@interface A0UnlockMutexLink : A0TwoEventsLink
@end
@implementation A0UnlockMutexLink
- (id)initWithSourceEvent:(id <PajeEvent>)se destEvent:(id <PajeEvent>)de
{
    return [super initWithName:[A0UnlockMutexLinkType name]
                    entityType:A0UnlockMutexLinkType
                     container:[[se container] container]
                   sourceEvent:se
                     destEvent:de];
}
@end

// Link between an a0NewSemaphore and the first state of the created semaphore
@interface A0NewSemaphoreLink : A0EventToStartStateLink
@end
@implementation A0NewSemaphoreLink
- (NSString *)name { return A0NewSemaphoreLinkName; }
@end

// Link between an a0VSemaphore and the unblocking event of the
// thread that gets unblocked.
@interface A0VSemaphoreLink : A0TwoEventsLink
@end
@implementation A0VSemaphoreLink
- (NSString *)name { return A0VSemaphoreLinkName; }
@end

// Link between an a0NewSlave event and the start of the first thread state
@interface A0NewSlaveLink : A0EventToStartStateLink
@end
@implementation A0NewSlaveLink
- (NSString *)name { return A0NewSlaveLinkName; }
@end

// Link between an end thread event and the corresponding join event
@interface A0JoinLink : A0EventToEndStateLink
@end
@implementation A0JoinLink
- (NSString *)name { return A0JoinLinkName; }
@end

// Link between aa synchronous a0NewPipe event and the start of the first 
// state of the thread created to handle this pipe
@interface A0NewPipeLink : A0EventToStartStateLink
@end
@implementation A0NewPipeLink
- (NSString *)name { return A0NewPipeLinkName; }
@end

// Link the creation event of a remote thread and its first event.
@interface A0StartRemoteThreadLink : A0TwoEventsLink
@end
@implementation A0StartRemoteThreadLink
- (NSString *)name { return A0StartRemoteThreadLinkName; }
@end

// Link the creation event of a remote urgent thread and its first event.
@interface A0StartRemoteUrgentLink : A0TwoEventsLink
@end
@implementation A0StartRemoteUrgentLink
- (NSString *)name { return A0StartRemoteUrgentLinkName; }
@end

#endif


@implementation A0Link
+ (void)initialize
{
    // entity types
    if (self == [A0Link class]) {
        A0LinkEntityType   = [[PajeLinkType alloc] initWithName:[UniqueString stringWithString:@"Link"] containerType:[A0bNode entityType]];
        
    // Possible entity names
        A0NewMutexLinkName     = [UniqueString stringWithString:@"Mutex creation"];
        A0UnlockMutexLinkName  = [UniqueString stringWithString:@"Mutex unlock"];
        A0NewSemaphoreLinkName = [UniqueString stringWithString:@"Semaphore creation"];
        A0VSemaphoreLinkName   = [UniqueString stringWithString:@"Semaphore V operation"];
        A0NewSlaveLinkName     = [UniqueString stringWithString:@"Slave thread creation"];
        A0JoinLinkName         = [UniqueString stringWithString:@"Join thread"];
        A0NewPipeLinkName      = [UniqueString stringWithString:@"Sync pipe thread creation"];
        A0StartRemoteThreadLinkName   = [UniqueString stringWithString:@"Remote thread creation"];
        A0StartRemoteUrgentLinkName   = [UniqueString stringWithString:@"Remote urgent thread creation"];
    }
    // A0Entity sets the colors...
    [super initialize];
}

+ (A0Link *)newMutexLinkFromEvent:(id <PajeEvent>)e
                          toState:(id <PajeState>)s
{
    A0EventToStartStateLink *link = [A0EventToStartStateLink alloc];
    return [[link initWithType:A0LinkEntityType
                          name:A0NewMutexLinkName
                     container:[[e container] container]
                   sourceEvent:e
                     destState:s] autorelease];
}

+ (A0Link *)unlockMutexLinkFromEvent:(id <PajeEvent>)e1
                             toEvent:(id <PajeEvent>)e2
{
    A0TwoEventsLink *link = [A0TwoEventsLink alloc];
    return [[link initWithType:A0LinkEntityType
                          name:A0UnlockMutexLinkName
                     container:[[e1 container] container]
                   sourceEvent:e1
                     destEvent:e2] autorelease];
}

+ (A0Link *)newSemaphoreLinkFromEvent:(id <PajeEvent>)e
                              toState:(id <PajeState>)s
{
    A0EventToStartStateLink *link = [A0EventToStartStateLink alloc];
    return [[link initWithType:A0LinkEntityType
                          name:A0NewSemaphoreLinkName
                     container:[[e container] container]
                   sourceEvent:e
                     destState:s] autorelease];
}

+ (A0Link *)VSemaphoreLinkFromEvent:(id <PajeEvent>)e1
                            toEvent:(id <PajeEvent>)e2
{
    A0TwoEventsLink *link = [A0TwoEventsLink alloc];
    return [[link initWithType:A0LinkEntityType
                          name:A0VSemaphoreLinkName
                     container:[[e1 container] container]
                   sourceEvent:e1
                     destEvent:e2] autorelease];
}

+ (A0Link *)newSlaveLinkFromEvent:(id <PajeEvent>)e
                          toState:(id <PajeState>)s
{
    A0EventToStartStateLink *link = [A0EventToStartStateLink alloc];
    return [[link initWithType:A0LinkEntityType
                          name:A0NewSlaveLinkName
                     container:[[e container] container]
                   sourceEvent:e
                     destState:s] autorelease];
}

+ (A0Link *)joinLinkFromEvent:(id <PajeEvent>)e
                      toState:(id <PajeState>)s
{
    A0EventToStartStateLink *link = [A0EventToStartStateLink alloc];
    return [[link initWithType:A0LinkEntityType
                          name:A0JoinLinkName
                     container:[[e container] container]
                   sourceEvent:e
                     destState:s] autorelease];
}

+ (A0Link *)newPipeLinkFromEvent:(id <PajeEvent>)e
                         toState:(id <PajeState>)s
{
    A0EventToStartStateLink *link = [A0EventToStartStateLink alloc];
    return [[link initWithType:A0LinkEntityType
                          name:A0NewPipeLinkName
                     container:[[e container] container]
                   sourceEvent:e
                     destState:s] autorelease];
}

+ (A0Link *)startRemoteThreadLinkFromEvent:(id <PajeEvent>)e1
                                   toEvent:(id <PajeEvent>)e2
{
    A0TwoEventsLink *link = [A0TwoEventsLink alloc];
    return [[link initWithType:A0LinkEntityType
                          name:A0StartRemoteThreadLinkName
                     container:[[e1 container] container]
                   sourceEvent:e1
                     destEvent:e2] autorelease];
}

+ (A0Link *)startRemoteUrgentLinkFromEvent:(id <PajeEvent>)e1
                                   toEvent:(id <PajeEvent>)e2
{
    A0TwoEventsLink *link = [A0TwoEventsLink alloc];
    return [[link initWithType:A0LinkEntityType
                          name:A0StartRemoteUrgentLinkName
                     container:[[e1 container] container]
                   sourceEvent:e1
                     destEvent:e2] autorelease];
}


// support for colors
+ (NSMutableDictionary *)colorDict
{
    if (!colorDict) colorDict = [[NSMutableDictionary alloc] init];
    return colorDict;
}

- (NSDate *)time { return [self startTime]; }
// if clocks are not synchronized, a message can arrive before being sent
- (NSDate *)firstTime { return [[self startTime] earlierDate:[self endTime]]; }
- (NSDate *)lastTime  { return [[self startTime] laterDate:[self endTime]]; }

- (id)node    { return [self sourceNode]; }
- (id)thread  { return [self sourceThread]; }
- (id)service { return [self sourceService]; }

+ (PajeDrawingType)drawingType { return PajeLinkDrawingType; }
- (PajeDrawingType)drawingType { return PajeLinkDrawingType; }

+ (PajeEntityType *)entityType       { return A0LinkEntityType; }
- (PajeEntityType *)entityType       { return A0LinkEntityType; }

- (id)sourceNode    { [self _subclassResponsibility:_cmd]; return nil; }
- (id)sourceThread  { [self _subclassResponsibility:_cmd]; return nil; }
- (id)sourceService { [self _subclassResponsibility:_cmd]; return nil; }
- (id)destNode    { [self _subclassResponsibility:_cmd]; return nil; }
- (id)destThread  { [self _subclassResponsibility:_cmd]; return nil; }
- (id)destService { [self _subclassResponsibility:_cmd]; return nil; }

- (id)sourceContainer  { [self _subclassResponsibility:_cmd]; return nil; }
- (id)sourceEntityType { [self _subclassResponsibility:_cmd]; return nil; }
- (id)destContainer  { [self _subclassResponsibility:_cmd]; return nil; }
- (id)destEntityType { [self _subclassResponsibility:_cmd]; return nil; }

- (NSString *)description
{
    return [NSString stringWithFormat:@"%@ from %@ at %@ to %@ at %@",
        [self name], [self sourceThread], [self startTime],
        [self destThread], [self endTime]];
}
+ (id)hierarchy {
    static id hier = nil;
    if (!hier)
        hier = [[NSArray alloc] init];//WithObject:@"Node"];
    return hier;
}
@end
