/* see A0MutexThreadState.h */

#import "A0MutexThreadState.h"
#import "A0bMutex.h"
#import "A0bThread.h"
#import "UniqueString.h"

static NSMutableDictionary *colorDict;

NSString *MutexThreadStateEntityType;

// Define possible states' names
NSString *PHoldingMutex;
NSString *PWaitingMutex;

@implementation A0MutexThreadState

+ (void)initialize
{
    if (!MutexThreadStateEntityType) {
        MutexThreadStateEntityType = [UniqueString stringWithString:@"Mutex Thread State"];
        PHoldingMutex = [UniqueString stringWithString:@"Holding mutex"];
        PWaitingMutex = [UniqueString stringWithString:@"Waiting mutex"];
    }
}

+ (NSMutableDictionary *)defaultColorDict
{
    return [NSMutableDictionary dictionaryWithObjectsAndKeys:
        [NSColor greenColor],	PHoldingMutex,
        [NSColor redColor],	PWaitingMutex,
        nil];
}

+ (NSMutableDictionary *)colorDict
{
    return colorDict;
}

+ (void)setColorDict:(id)dict
{
    id tmp = [dict retain];
    [colorDict release];
    colorDict = tmp;
}

+ (NSArray *)allNames
{
    return [[self colorDict] allKeys];
}

+ (NSString *)name
{
    return MutexThreadStateEntityType;
}

+ (A0MutexThreadState *)holdingStateWithMutex:(A0bMutex *)m
                                       thread:(A0bThread *)t
{
    return [[[self alloc] initWithName:PHoldingMutex mutex:m thread:t] autorelease];
}

+ (A0MutexThreadState *)waitingStateWithMutex:(A0bMutex *)m
                                       thread:(A0bThread *)t
{
    return [[[self alloc] initWithName:PWaitingMutex mutex:m thread:t] autorelease];
}

- initWithName:(NSString *)n mutex:(A0bMutex *)m thread:(A0bThread *)t
{
    self = [super initWithName:n];
    [self setMutex:m];
    [self setThread:t];
    return self;
}

- (void)setMutex:(A0bMutex *)mutex
{
    [self setObject:mutex forKey:@"Mutex"];
}

- (NSString *)entityType
{
    return MutexThreadStateEntityType;
}
+ (NSString *)entityType
{
    return MutexThreadStateEntityType;
}
- (id)container { return [self thread];/*[NSArray arrayWithObjects:[self node], [self thread], nil];*/ }
+ (id)hierarchy {
    static id hier = nil;
    if (!hier)
        hier = [[NSArray alloc] initWithObjects:@"Node", @"Thread", nil];
    return hier;
}
@end