
#import "A0State.h"

#import "A0bSemaphore.h"
#import "A0bMutex.h"
#import "../General.bproj/UniqueString.h"


static NSMutableDictionary *threadStateColorDict;
static NSMutableDictionary *semaphoreStateColorDict;
static NSMutableDictionary *mutexStateColorDict;
static NSMutableDictionary *mutexThreadStateColorDict;

// entity types
PajeEntityType *A0ThreadStateEntityType;
PajeEntityType *A0MutexStateEntityType;
PajeEntityType *A0MutexThreadStateEntityType;
PajeEntityType *A0SemaphoreStateEntityType;
//NSString *A0UserThreadStateEntityType;
//NSString *A0UserNodeStateEntityType;
//NSString *A0UserGlobalStateEntityType;

// Possible thread state names
NSString *A0RunnableThreadStateName;
NSString *A0BlockedInJoinThreadStateName;
NSString *A0BlockedInSendThreadStateName;
NSString *A0BlockedInReceiveThreadStateName;
NSString *A0BlockedInWaitRequestThreadStateName;
NSString *A0BlockedInLockMutexThreadStateName;
NSString *A0BlockedInWaitConditionThreadStateName;
NSString *A0BlockedInPSemaphoreThreadStateName;

// Possible semaphore state names
NSString *A0UnblockedSemaphoreStateName;
NSString *A0BlockedSemaphoreStateName;

// Possible mutex state names
NSString *A0UnlockedMutexStateName;
NSString *A0LockedMutexStateName;

// Possible mutex thread state names
NSString *A0HoldingMutexThreadStateName;
NSString *A0WaitingMutexThreadStateName;


@implementation A0TwoEventsState
- (id)initWithStartEvent:(id <PajeEvent>)e
{
    self = [super init];
    if (!self) return nil;
    firstEvent = [e retain];
    secondEvent = nil;
    return self;
}

- (void)setEndEvent:(id <PajeEvent>)e
{
    NSAssert(secondEvent == nil, @"end event being set twice");
    secondEvent = [e retain];
}

- (void)dealloc
{
    [firstEvent release];
    [secondEvent release];
    [super dealloc];
}

- (A0Event *)startEvent { return firstEvent; }
- (A0Event *)endEvent   { return secondEvent; }

- (NSArray *)relatedEntities
{
    NSArray *array = [[super relatedEntities] arrayByAddingObject:firstEvent];
    if (secondEvent)
        return [array arrayByAddingObject:secondEvent];
    else
        return array;
}

- (id)node            { return [[firstEvent thread] node]; }
- (id)thread          { return [firstEvent thread]; }
- (id)service         { return [firstEvent service]; }
- (id)container       { return [firstEvent container]; }
- (NSDate *)startTime { return [firstEvent time]; }
- (NSDate *)endTime
{
//    NSDate *time;
    
    if (secondEvent)
        return [secondEvent time];
//    time = [firstEvent objectForKey:@"EndTime"];
//    if (time)
//        return time;
//    return [self startTime];
    return [NSDate distantFuture];
}
@end

// a special state for a new slave, because the first event is in the
// creator thread, not in the created one.
@interface A0RunnableWithNewSlaveThreadState : A0ThreadState
@end
@implementation A0RunnableWithNewSlaveThreadState
- (NSString *)name { return A0RunnableThreadStateName; }
- (id)thread       { return [firstEvent objectForKey:@"OtherThread"]; }
- (id)container {
    return [self thread];/*[NSArray arrayWithObjects:[self node], [NSNumber numberWithInt:[[self thread] intValue]], nil];*/
}
@end

@interface A0RunnableThreadState : A0ThreadState
@end
@implementation A0RunnableThreadState
- (NSString *)name { return A0RunnableThreadStateName; }
@end

@interface A0BlockedInJoinThreadState : A0ThreadState
@end
@implementation A0BlockedInJoinThreadState
- (NSString *)name { return A0BlockedInJoinThreadStateName; }
@end

@interface A0BlockedInSendThreadState : A0ThreadState
@end
@implementation A0BlockedInSendThreadState
- (NSString *)name { return A0BlockedInSendThreadStateName; }
@end

@interface A0BlockedInReceiveThreadState : A0ThreadState
@end
@implementation A0BlockedInReceiveThreadState
- (NSString *)name { return A0BlockedInReceiveThreadStateName; }
@end

@interface A0BlockedInWaitRequestThreadState : A0ThreadState
@end
@implementation A0BlockedInWaitRequestThreadState
- (NSString *)name { return A0BlockedInWaitRequestThreadStateName; }
@end

@interface A0BlockedInLockMutexThreadState : A0ThreadState
@end
@implementation A0BlockedInLockMutexThreadState
- (NSString *)name { return A0BlockedInLockMutexThreadStateName; }
@end

@interface A0BlockedInWaitConditionThreadState : A0ThreadState
@end
@implementation A0BlockedInWaitConditionThreadState
- (NSString *)name { return A0BlockedInWaitConditionThreadStateName; }
@end

@interface A0BlockedInPSemaphoreThreadState : A0ThreadState
@end
@implementation A0BlockedInPSemaphoreThreadState
- (NSString *)name { return A0BlockedInPSemaphoreThreadStateName; }
@end


@implementation A0ThreadState
+ (NSDictionary *)eventNameToClass
{
    static NSDictionary *eventNameToClass = nil;
    if (!eventNameToClass) {
        eventNameToClass = [[NSDictionary alloc] initWithObjectsAndKeys:
            [A0RunnableWithNewSlaveThreadState class], A0NewSlaveEventName,
            [A0BlockedInJoinThreadState class], A0JoinSlaveEventName,
            [A0BlockedInSendThreadState class], A0SendEventName,
            [A0BlockedInSendThreadState class], A0SendBufferEventName,
            [A0BlockedInSendThreadState class], A0FSendEventName,
            [A0BlockedInReceiveThreadState class], A0ReceiveEventName,
            [A0BlockedInReceiveThreadState class], A0ReceiveBufferEventName,
            [A0BlockedInReceiveThreadState class], A0FReceiveEventName,
            [A0BlockedInWaitRequestThreadState class], A0WaitRequestEventName,
            [A0BlockedInWaitRequestThreadState class], A0WaitAnyRequestEventName,
            [A0BlockedInLockMutexThreadState class], A0LockMutexEventName,
            [A0BlockedInWaitConditionThreadState class], A0WaitConditionEventName,
            [A0BlockedInWaitConditionThreadState class], A0TimedWaitConditionEventName,
            [A0BlockedInPSemaphoreThreadState class], A0PSemaphoreEventName,
            nil];
    }
    return eventNameToClass;
}
// support for colors
+ (NSMutableDictionary *)colorDict
{
    if (!threadStateColorDict)
        threadStateColorDict = [[NSMutableDictionary alloc] init];
    return threadStateColorDict;
}
+ (A0ThreadState *)runnableThreadStateFromEvent:(id<PajeEvent>)e
{
    Class class;
    class = [[self eventNameToClass] objectForKey:[e name]];
    if (class)
        return [[[class alloc]
                   initWithStartEvent:e] autorelease];
    else
        return [[[A0RunnableThreadState alloc]
                   initWithStartEvent:e] autorelease];
}
+ (A0ThreadState *)blockedThreadStateFromEvent:(id<PajeEvent>)e
{
    Class class;
    class = [[self eventNameToClass] objectForKey:[e name]];
    NSAssert1(class, @"Unknown blocking event %@", [e name]);
    return [[[class alloc] initWithStartEvent:e] autorelease];
}

+ (PajeEntityType *)entityType { return A0ThreadStateEntityType; }
- (PajeEntityType *)entityType { return A0ThreadStateEntityType; }

+ (id)hierarchy {
    static id hier = nil;
    if (!hier)
        hier = [[NSArray alloc] initWithObjects:@"Node", @"Thread", nil];
    return hier;
}
@end

@interface A0UnblockedSemaphoreState : A0SemaphoreState
@end
@implementation A0UnblockedSemaphoreState
- (NSString *)name { return A0UnblockedSemaphoreStateName; }
@end

@interface A0BlockedSemaphoreState : A0SemaphoreState
@end
@implementation A0BlockedSemaphoreState
- (NSString *)name { return A0BlockedSemaphoreStateName; }
@end

@implementation A0SemaphoreState
- (id)initWithFromEvent:(id<PajeEvent>)e
              semaphore:(A0bSemaphore *)sema
                  value:(int)v
{
    self = [super initWithStartEvent:e];
    if (self) {
        semaphore = [sema retain];
        value = v;
    }
    return self;
}
- (void)dealloc
{
    [semaphore release];
    [super dealloc];
}
+ (A0SemaphoreState *)blockedSemaphoreStateFromEvent:(id<PajeEvent>)e
                                           semaphore:(A0bSemaphore *)sema
                                               value:(int)v
{
    return [[[A0BlockedSemaphoreState alloc] initWithFromEvent:e
                                                     semaphore:sema
                                                         value:v] autorelease];
}
+ (A0SemaphoreState *)unblockedSemaphoreStateFromEvent:(id<PajeEvent>)e
                                             semaphore:(A0bSemaphore *)sema
                                                 value:(int)v
{
    return [[[A0UnblockedSemaphoreState alloc] initWithFromEvent:e
                                                       semaphore:sema
                                                           value:v] autorelease];
}
+ (NSMutableDictionary *)colorDict
{
    if (!semaphoreStateColorDict)
        semaphoreStateColorDict = [[NSMutableDictionary alloc] init];
    return semaphoreStateColorDict;
}
+ (PajeEntityType *)entityType { return A0SemaphoreStateEntityType; }
- (PajeEntityType *)entityType { return A0SemaphoreStateEntityType; }

+ (id)hierarchy {
    static id hier = nil;
    if (!hier)
        hier = [[NSArray alloc] initWithObjects:@"Node", @"Semaphore", nil];
    return hier;
}
- (id)container {
    return semaphore;/*[NSArray arrayWithObjects:[firstEvent node], semaphore, nil];*/
}
@end

@interface A0UnlockedMutexState : A0MutexState
@end
@implementation A0UnlockedMutexState
- (NSString *)name { return A0UnlockedMutexStateName; }
@end

@interface A0LockedMutexState : A0MutexState
@end
@implementation A0LockedMutexState
- (NSString *)name { return A0LockedMutexStateName; }
@end

@implementation A0MutexState
- (id)initWithFromEvent:(id<PajeEvent>)e
                  mutex:(A0bMutex *)m
{
    self = [super initWithStartEvent:e];
    if (self) {
        mutex = [m retain];
    }
    return self;
}
- (void)dealloc
{
    [mutex release];
    [super dealloc];
}
+ (A0MutexState *)lockedMutexStateFromEvent:(id<PajeEvent>)e
                                      mutex:(A0bMutex *)m
{
    return [[[A0LockedMutexState alloc] initWithFromEvent:e
                                                    mutex:m] autorelease];
}
+ (A0MutexState *)unlockedMutexStateFromEvent:(id<PajeEvent>)e
                                        mutex:(A0bMutex *)m
{
    return [[[A0UnlockedMutexState alloc] initWithFromEvent:e
                                                      mutex:m] autorelease];
}
+ (NSMutableDictionary *)colorDict
{
    if (!mutexStateColorDict)
        mutexStateColorDict = [[NSMutableDictionary alloc] init];
    return mutexStateColorDict;
}
+ (PajeEntityType *)entityType { return A0MutexStateEntityType; }
- (PajeEntityType *)entityType { return A0MutexStateEntityType; }

+ (id)hierarchy {
    static id hier = nil;
    if (!hier)
        hier = [[NSArray alloc] initWithObjects:@"Node", @"Mutex", nil];
    return hier;
}
- (id)container {
    return mutex;
}
- (A0bMutex *)mutex {
    return mutex;
}
@end


@interface A0WaitingMutexThreadState : A0MutexThreadState
@end
@implementation A0WaitingMutexThreadState
- (NSString *)name { return A0WaitingMutexThreadStateName; }
@end

@interface A0HoldingMutexThreadState : A0MutexThreadState
@end
@implementation A0HoldingMutexThreadState
- (NSString *)name { return A0HoldingMutexThreadStateName; }
@end

@implementation A0MutexThreadState
- (id)initWithFromEvent:(id<PajeEvent>)e
                  mutex:(A0bMutex *)m
{
    self = [super initWithStartEvent:e];
    if (self) {
        mutex = [m retain];
    }
    return self;
}
- (void)dealloc
{
    [mutex release];
    [super dealloc];
}
+ (A0MutexThreadState *)holdingMutexThreadStateFromEvent:(id<PajeEvent>)e
                                                   mutex:(A0bMutex *)m
{
    return [[[A0HoldingMutexThreadState alloc] initWithFromEvent:e
                                                           mutex:m] autorelease];
}
+ (A0MutexThreadState *)waitingMutexThreadStateFromEvent:(id<PajeEvent>)e
                                                   mutex:(A0bMutex *)m
{
    return [[[A0WaitingMutexThreadState alloc] initWithFromEvent:e
                                                           mutex:m] autorelease];
}
+ (NSMutableDictionary *)colorDict
{
    if (!mutexThreadStateColorDict)
        mutexThreadStateColorDict = [[NSMutableDictionary alloc] init];
    return mutexThreadStateColorDict;
}
+ (PajeEntityType *)entityType { return A0MutexThreadStateEntityType; }
- (PajeEntityType *)entityType { return A0MutexThreadStateEntityType; }

+ (id)hierarchy {
    static id hier = nil;
    if (!hier)
        hier = [[NSArray alloc] initWithObjects:@"Node", @"Thread", nil];
    return hier;
}
- (id)container {
    return [firstEvent container];
}
- (A0bMutex *)mutex {
    return mutex;
}
@end


@implementation A0State
+ (void)initialize
{
    if (self == [A0State class]) {
        // entity types
        A0ThreadStateEntityType      = [[PajeStateType alloc] initWithName:[UniqueString stringWithString:@"Thread State"] containerType:[A0bThread entityType]];
        A0MutexStateEntityType       = [[PajeStateType alloc] initWithName:[UniqueString stringWithString:@"Mutex State"] containerType:[A0bMutex entityType]];
        A0MutexThreadStateEntityType = [[PajeStateType alloc] initWithName:[UniqueString stringWithString:@"Mutex Thread State"] containerType:[A0bThread entityType]];
        A0SemaphoreStateEntityType   = [[PajeStateType alloc] initWithName:[UniqueString stringWithString:@"Semaphore State"] containerType:[A0bSemaphore entityType]];
        /*
        A0UserThreadStateEntityType  = [[PajeEntityType alloc] initWithName:[UniqueString stringWithString:@"User Thread State"] containerType:[A0bThread entityType]];
        A0UserNodeStateEntityType    = [[PajeEntityType alloc] initWithName:[UniqueString stringWithString:@"User Node State"] containerType:[A0bNode entityType]];
         A0UserGlobalStateEntityType  = [[PajeEntityType alloc] initWithName:[UniqueString stringWithString:@"User Global State"] containerType:[A0bSimul executionEntityType]];
         */
        
        // Possible thread state names
        A0RunnableThreadStateName
            = [UniqueString stringWithString:@"Thread runnable"];
        A0BlockedInJoinThreadStateName
            = [UniqueString stringWithString:@"Thread blocked in a0Join"];
        A0BlockedInSendThreadStateName
            = [UniqueString stringWithString:@"Thread blocked in a0Send"];
        A0BlockedInReceiveThreadStateName
            = [UniqueString stringWithString:@"Thread blocked in a0Receive"];
        A0BlockedInWaitRequestThreadStateName
            = [UniqueString stringWithString:@"Thread blocked in a0WaitRequest"];
        A0BlockedInLockMutexThreadStateName
            = [UniqueString stringWithString:@"Thread blocked in a0LockMutex"];
        A0BlockedInWaitConditionThreadStateName
            = [UniqueString stringWithString:@"Thread blocked in a0WaitCondition"];
        A0BlockedInPSemaphoreThreadStateName
            = [UniqueString stringWithString:@"Thread blocked in a0PSemaphore"];
        
        // Possible semaphore state names
        A0UnblockedSemaphoreStateName = [UniqueString stringWithString:@"Semaphore unblocked"];
        A0BlockedSemaphoreStateName   = [UniqueString stringWithString:@"Semaphore blocked"];
        
        // Possible mutex state names
        A0UnlockedMutexStateName      = [UniqueString stringWithString:@"Mutex unlocked"];
        A0LockedMutexStateName        = [UniqueString stringWithString:@"Mutex locked"];
        
        // Possible mutex thread state names
        A0HoldingMutexThreadStateName = [UniqueString stringWithString:@"Holding mutex"];
        A0WaitingMutexThreadStateName = [UniqueString stringWithString:@"Waiting mutex"];
    }
    // A0Entity sets the colors...
    [super initialize];
}

- (NSDate *)startTime    { [self _subclassResponsibility:_cmd]; return nil; }
- (NSDate *)endTime      { [self _subclassResponsibility:_cmd]; return nil; }

+ (PajeDrawingType)drawingType { return PajeStateDrawingType; }
- (PajeDrawingType)drawingType { return PajeStateDrawingType; }

- (NSDate *)time      { return [self startTime]; }
@end
