/* see A0bComm.h */

#import "A0bComm.h"

static NSMutableDictionary *colorDict;

NSString *PCommNewSlave          = @"New Slave";
NSString *PCommNewPipe           = @"New Pipe";
NSString *PCommJoin              = @"Join";
NSString *PCommStartRemoteThread = @"Start Remote Thread";
NSString *PCommStartRemoteUrgent = @"Start Remote Urgent";
NSString *PCommMessage           = @"Message";

@implementation A0bComm

+ (NSString *)name
{
    return @"Communication b";
}

+ (NSMutableDictionary *)defaultColorDict
{
    return [NSMutableDictionary dictionaryWithObjectsAndKeys:
        [NSColor orangeColor], PCommNewSlave,
        [NSColor orangeColor], PCommNewPipe,
        [NSColor orangeColor], PCommJoin,
        [NSColor orangeColor], PCommStartRemoteThread,
        [NSColor brownColor], PCommStartRemoteUrgent,
        [NSColor blueColor], PCommMessage,
        nil];
}

+ (NSMutableDictionary *)colorDict
{
    return colorDict;
}

+ (void)setColorDict:(id)dict
{
    id tmp = [dict retain];
    [colorDict release];
    colorDict = tmp;
}

+ (A0bComm *)commWithName:(NSString *)string
{
    return [[[self alloc] initWithName:string] autorelease];
}

+ (A0bComm *)linkWithName:(NSString *)string
               fromEntity:(id<PajeEntity>)fromEntity
                 toEntity:(id<PajeEntity>)toEntity
{
    A0bComm *link = [[self alloc] initWithName:string];
    [link setSourceNode:[fromEntity node]];
    [link setSourceThread:[fromEntity thread]];
    [link setSourceService:[fromEntity service]];
    [link setStartTime:[fromEntity time]];

    [link setDestNode:[toEntity node]];
    [link setDestThread:[toEntity thread]];
    [link setDestService:[toEntity service]];
    [link setEndTime:[toEntity time]];

    [link addRelatedEntity:fromEntity];
    [link addRelatedEntity:toEntity];
    
    return [link autorelease];
}

- (id)XinitWithName:(NSString *)string
{
    [super init];
    fieldDict = [[NSMutableDictionary dictionaryWithObjectsAndKeys:
        string,                          @"Name",
//        [colorDict objectForKey:name], @"Color",
        nil] retain];
    return self;
}

- (id)sourceNode    {return [fieldDict objectForKey:@"SourceNode"];}
- (id)destNode      {return [fieldDict objectForKey:@"DestNode"];}
- (id)node          {return @"";}

- (id)sourceService {return [fieldDict objectForKey:@"SourceService"];}
- (id)destService   {return [fieldDict objectForKey:@"DestService"];}
- (id)sourceThread  {return [fieldDict objectForKey:@"SourceThread"];}
- (id)destThread    {return [fieldDict objectForKey:@"DestThread"];}

// in case the clocks are not synchronized, a message can arrive before beeing sent
- (NSDate *)firstTime
{
    return [[self startTime] earlierDate:[self endTime]];
}

- (NSDate *)lastTime
{
    return [[self startTime] laterDate:[self endTime]];
}

- (void)setSourceNode:(id)o    { [self setObject:o forKey:@"SourceNode"]; }
- (void)setSourceThread:(id)o  { [self setObject:o forKey:@"SourceThread"]; }
- (void)setSourceService:(id)o { [self setObject:o forKey:@"SourceService"]; }
- (void)setDestNode:(id)o      { [self setObject:o forKey:@"DestNode"]; }
- (void)setDestThread:(id)o    { [self setObject:o forKey:@"DestThread"]; }
- (void)setDestService:(id)o   { [self setObject:o forKey:@"DestService"]; }

- (NSString *)entityType
{
    return PajeComm;
}
+ (NSString *)entityType
{
    return PajeComm;
}
+ (PajeDrawingType)drawingType { return PajeLinkDrawingType; }
- (PajeDrawingType)drawingType { return PajeLinkDrawingType; }
@end