/* see A0bNode.h */

#import "A0bNode.h"
#import "A0bSimul.h"
//#import "../General.bproj/UniqueString.h"
#import "../General.bproj/Macros.h"

PajeContainerType *A0NodeEntityType;

@implementation A0bNode

+ (void)initialize
{
    if (self == [A0bNode class]) {
        A0NodeEntityType = [[PajeContainerType alloc]
                                initWithName:@"Node"
                               containerType:[A0bSimul executionEntityType]];
    }
    [super initialize];
}

+ (PajeContainerType *)entityType
{
    return A0NodeEntityType;
}

+ (A0bNode *)nodeWithId:(id)nId
              simulator:(A0bSimul *)simul
              container:(PajeContainer *)cont
{
    return [[[self alloc] initWithId:nId
                           simulator:simul
                           container:cont] autorelease];
}

- (id)initWithId:(id)nId
       simulator:(A0bSimul *)simul
       container:(PajeContainer *)cont
{
    self = [super initWithType:A0NodeEntityType
                          name:[nId description]
                     container:cont];
    if (self != nil) {
        nodeId = [nId retain];
        simulator = simul;
        threads = [[NSMutableDictionary alloc] init];
        mutexes = [[NSMutableDictionary alloc] init];
        semaphores = [[NSMutableDictionary alloc] init];
        pendingThreadCreations = [[PSortedArray alloc] initWithSelector:@selector(serialNo)];
        pendingSends = [[PSortedArray alloc] initWithSelector:@selector(serialNo)];
        pendingSendCompletions = [[NSMutableDictionary alloc] init];
        pendingSendComms = [[NSMutableArray alloc] init];
        pendingReceives = [[PSortedArray alloc] initWithSelector:@selector(serialNo)];
        pendingReceiveCompletions = [[NSMutableDictionary alloc] init];
        pendingReceiveComms = [[NSMutableArray alloc] init];

        /// HACK for working with the new traces... should review this after these is finished.
        reallyPendingSends = [[NSMutableDictionary alloc] init];
        reallyPendingReceives = [[NSMutableDictionary alloc] init];

        Assign(userEntities, [NSMutableDictionary dictionary]);
    }

    return self;
}

- (void)dealloc
{
    [nodeId release];
    [threads release];
    [mutexes release];
    [semaphores release];
    [pendingThreadCreations release];
    [pendingSends release];
    [pendingSendCompletions release];
    [pendingSendComms release];
    [pendingReceives release];
    [pendingReceiveCompletions release];
    [pendingReceiveComms release];
    [super dealloc];
}

- (id)nodeId
{
    return nodeId;
}

- (void)addThread:(A0bThread *)thread
{
    [threads setObject:thread forKey:[thread name]];
}

- (A0bThread *)threadWithId:(id)threadId
{
    return [threads objectForKey:threadId];
}

- (NSEnumerator *)threadsEnumerator
{
    return [threads objectEnumerator];
}

- (void)removeThread:(A0bThread *)thread
{
    [threads removeObjectForKey:[thread name]];
}


- (void)addMutex:(A0bMutex *)mutex
{
    [mutexes setObject:mutex forKey:[mutex name]];
}

- (A0bMutex *)mutexWithId:(id)mutexId
{
    return [mutexes objectForKey:mutexId];
}

- (NSEnumerator *)mutexesEnumerator
{
    return [mutexes objectEnumerator];
}

- (void)removeMutex:(A0bMutex *)mutex
{
    [mutexes removeObjectForKey:[mutex name]];
}


- (void)addSemaphore:(A0bSemaphore *)semaphore
{
    [semaphores setObject:semaphore forKey:[semaphore name]];
}

- (A0bSemaphore *)semaphoreWithId:(id)semaphoreId
{
    return [semaphores objectForKey:semaphoreId];
}

- (NSEnumerator *)semaphoresEnumerator
{
    return [semaphores objectEnumerator];
}

- (void)removeSemaphore:(A0bSemaphore *)semaphore
{
    [semaphores removeObjectForKey:[semaphore name]];
}





- (PSortedArray *)pendingThreadCreations
{
    return pendingThreadCreations;
}

- (PSortedArray *)pendingSends
{
    return pendingSends;
}
- (NSMutableDictionary *)pendingSendCompletions
{
    return pendingSendCompletions;
}
- (NSMutableArray *)pendingSendComms
{
    return pendingSendComms;
}
- (PSortedArray *)pendingReceives
{
    return pendingReceives;
}
- (NSMutableDictionary *)pendingReceiveCompletions
{
    return pendingReceiveCompletions;
}
- (NSMutableArray *)pendingReceiveComms
{
    return pendingReceiveComms;
}
- (NSMutableDictionary *)reallyPendingSends
{
    return reallyPendingSends;
}
- (NSMutableDictionary *)reallyPendingReceives
{
    return reallyPendingReceives;
}

- (NSDate *)startTime {
    return [simulator startTime];
}

- (NSDate *)endTime {
    return [simulator endTime];
}


// NSObject protocol
- (unsigned int)hash;
{
    return [nodeId hash];
}

- (BOOL)isEqual:(id)anObject
{
    if (anObject == self)
        return YES;
    if (![anObject isKindOfClass:[A0bNode class]])
        return NO;
    return [nodeId isEqual:[anObject nodeId]];
}

// NSCopying Protocol
- (id)copyWithZone:(NSZone *)zone
{
    return [self retain];
}

- (NSComparisonResult)compare:(id)other
{
    return [(NSNumber *)[self nodeId] compare:[other nodeId]];
}

// NSCoding Protocol
- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];
    [coder encodeObject:nodeId];
    [coder encodeObject:threads];
    [coder encodeObject:pendingThreadCreations];
    [coder encodeObject:pendingSends];
    [coder encodeObject:pendingSendCompletions];
    [coder encodeObject:pendingSendComms];
    [coder encodeObject:pendingReceives];
    [coder encodeObject:pendingReceiveCompletions];
    [coder encodeObject:pendingReceiveComms];
}

- (id)initWithCoder:(NSCoder *)coder
{
    self = [super initWithCoder:coder];

    nodeId = [[coder decodeObject] retain];
    threads = [[coder decodeObject] retain];
    pendingThreadCreations = [[coder decodeObject] retain];
    pendingSends = [[coder decodeObject] retain];
    pendingSendCompletions = [[coder decodeObject] retain];
    pendingSendComms = [[coder decodeObject] retain];
    pendingReceives = [[coder decodeObject] retain];
    pendingReceiveCompletions = [[coder decodeObject] retain];
    pendingReceiveComms = [[coder decodeObject] retain];
    
    return self;
}
@end
