/* A0bSemaphore.m created by benhur on Thu 22-May-1997 */

#import "A0bSemaphore.h"
#import "../General.bproj/UniqueString.h"

PajeContainerType *A0SemaphoreEntityType;

@implementation A0bSemaphore

+ (void)initialize
{
    if (self == [A0bSemaphore class]) {
        A0SemaphoreEntityType = [[PajeContainerType alloc] initWithName:[UniqueString stringWithString:@"Semaphore"] containerType:[A0bNode entityType]];
    }
    [super initialize];
}

+ (PajeContainerType *)entityType { return A0SemaphoreEntityType; }

- (id)initWithId:(id)ident node:(A0bNode *)node
{
    self = [super initWithType:A0SemaphoreEntityType
                          name:ident
                     container:node];
    if (self == nil) return nil;
    
    simulator = nil;
    lastEvent = nil;
    currentState = nil;
    blockedThreads = [[NSMutableArray alloc] init];
    waitingVs = [[NSMutableArray alloc] init];
    creationTime = nil;
    destructionTime = nil;
    return self;
}

- (void)dealloc
{
    [lastEvent release];
    [currentState release];
    [blockedThreads release];
    [waitingVs release];
    [creationTime release];
    [destructionTime release];
    [super dealloc];
}

+ (A0bSemaphore *)semaphoreWithId:(id)ident node:(A0bNode *)n
{
    return [[[self alloc] initWithId:ident node:n] autorelease];
}

- (A0bNode *)node { return (A0bNode *)container; }

- (void)setSimulator:(A0bSimul *)simul
{
    NSAssert1(!simulator, @"Simulator being set more than once in semaphore %@", [self description]);
    simulator = simul;
}

- (void)setLastEvent:(A0Event *)event
{
    [event retain];
    [lastEvent release];
    lastEvent = event;
}

- (void)a0NewSemaphore:(A0Event *)event
{    
    NSAssert1(simulator, @"Simulator not set in semaphore for event %@", [event description]);
    NSAssert1(!lastEvent, @"Semaphore being recreated with event %@", [event description]);

    creationTime = [[event time] retain];
    
    value = [[event objectForKey:@"Initial"] intValue];

    if (value > 0)
        currentState = [[A0SemaphoreState unblockedSemaphoreStateFromEvent:event semaphore:self value:value] retain];
    else
        currentState = [[A0SemaphoreState unblockedSemaphoreStateFromEvent:event semaphore:self value:value] retain]; // blockable

    [self setLastEvent:event];

    [simulator outputEntity:currentState];
    [simulator outputEntity:[A0Link newSemaphoreLinkFromEvent:event
                                                      toState:currentState]];
}

- (void)a0PSemaphore:(A0Event *)event
{
    [currentState setEndEvent:event];
//    [simulator outputEntity:currentState];
    [currentState release];

    if (value > 0) {
        value --;
        if (value == 0)
            currentState = [[A0SemaphoreState unblockedSemaphoreStateFromEvent:event semaphore:self value:value] retain]; // blockable
        else
            currentState = [[A0SemaphoreState unblockedSemaphoreStateFromEvent:event semaphore:self value:value] retain];
    } else {
        A0bThread *thread = [simulator threadForEvent:event];

        NSAssert1(thread, @"Unknown thread for event %@", event);
        [thread blockWithEvent:event
               unblockSelector:@selector(thread:unblockedWithEvent:)
                        target:self];

        currentState = [[A0SemaphoreState blockedSemaphoreStateFromEvent:event semaphore:self value:value] retain];
        [currentState addRelatedEntity:[thread currentState]];
        [blockedThreads addObject:thread];
        [blockedThreads makeObjectsPerformSelector:@selector(addRelatedEntity:)
                                        withObject:currentState];
    }
    [self setLastEvent:event];
    [simulator outputEntity:currentState];
}

- (void)a0VSemaphore:(A0Event *)event
{
    int blockedCount;
    A0bThread *blockedThread;
    NSEnumerator *blockedThreadsEnumerator;
    
    [currentState setEndEvent:event];
//    [simulator outputEntity:currentState];
    [currentState release];

    blockedCount = [blockedThreads count] - [waitingVs count];
    if (value != 0 || blockedCount <= 0) {
        value ++;
    } else {
        [waitingVs addObject:event];
    }
    if (blockedCount > 1)
        currentState = [[A0SemaphoreState blockedSemaphoreStateFromEvent:event semaphore:self value:value] retain];
    else if (value <= 0)
        currentState = [[A0SemaphoreState unblockedSemaphoreStateFromEvent:event semaphore:self value:value] retain]; // blockable
    else
        currentState = [[A0SemaphoreState unblockedSemaphoreStateFromEvent:event semaphore:self value:value] retain];

    // add the current state of all threads blocked in sema as related entities.
    // the one of the unblocked thread will be taken out by 
    // -thread:unblockedWithEvent:
    blockedThreadsEnumerator = [blockedThreads objectEnumerator];
    while (blockedThread = [blockedThreadsEnumerator nextObject]) {
        A0ThreadState *blockedState = [blockedThread currentState];
        [blockedState addRelatedEntity:currentState];
        [currentState addRelatedEntity:blockedState];
    }
//    [blockedThreads makeObjectsPerform:@selector(addRelatedEntity:) withObject:currentState];

    [self setLastEvent:event];
    [simulator outputEntity:currentState];
}

- (void)a0TryPSemaphore:(A0Event *)event
{
    if ([[event objectForKey:@"Succed"] boolValue])
        [self a0PSemaphore:event];
}

- (void)a0DisposeSemaphore:(A0Event *)event
{
    destructionTime = [[event time] retain];
    [currentState setEndEvent:event];
//    [simulator outputEntity:currentState];
    [currentState release];
    currentState = nil;
    
    if ([blockedThreads count] - [waitingVs count]) {
        [simulator error:@"disposing semaphore with blocked threads %@", event];
    }
    [self setLastEvent:event];
}

- (void)thread:(A0bThread *)thread unblockedWithEvent:(A0Event *)event
{
    [currentState removeRelatedEntity:[thread currentState]];
    [[thread currentState] removeRelatedEntity:currentState];
    [blockedThreads removeObject:thread];

//    NSAssert1([waitingVs count], @"thread unblocked from semaphore %@ without a VSemaphore", [self description]);

    if ([waitingVs count]) {
        A0Link *newLink;
        newLink = [A0Link VSemaphoreLinkFromEvent:[waitingVs objectAtIndex:0]
                                          toEvent:event];
        [simulator outputEntity:newLink];
        [waitingVs removeObjectAtIndex:0];
    }
}

- (NSDate *)startTime {
    return creationTime;
}
- (NSDate *)endTime {
    if (destructionTime != nil)
        return destructionTime;
    return [NSDate distantFuture];
}


// NSCoding Protocol
- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];

    //    [coder encodeConditionalObject:simulator];
    [coder encodeObject:lastEvent];
    [coder encodeObject:currentState];
    [coder encodeObject:blockedThreads];
    [coder encodeObject:waitingVs];
    [coder encodeValueOfObjCType:@encode(int) at:&value];
    [coder encodeObject:creationTime];
    [coder encodeObject:destructionTime];
}

- (id)initWithCoder:(NSCoder *)coder
{
    self = [super initWithCoder:coder];

    //    simulator = [coder decodeObject];
    lastEvent = [[coder decodeObject] retain];
    currentState = [[coder decodeObject] retain];
    blockedThreads = [[coder decodeObject] retain];
    waitingVs = [[coder decodeObject] retain];
    [coder decodeValueOfObjCType:@encode(int) at:&value];
    creationTime = [[coder decodeObject] retain];
    destructionTime = [[coder decodeObject] retain];

    return self;
}
@end
