/* A0bSimul-Sema.m created by benhur on Mon 21-Apr-1997 */

/*
a0NewSemaphore (Semaphore, Initial)
a0PSemaphore (Semaphore, EndTime)
a0TryPSemaphore (Semaphore, Succeed)
a0VSemaphore (Semaphore)
a0DisposeSemaphore (Semaphore)
a0NewMutex (Mutex)
a0LockMutex (Mutex, EndTime)
a0UnlockMutex (Mutex)
a0DisposeMutex (Mutex)
*/

#import "A0bSimul.h"

@implementation A0bSimul (Synchro)

/*
 * ----------
 * Semaphores
 * ----------
 */

- (void)a0NewSemaphore:(A0Event *)event
{
    A0bSemaphore *sema = [self semaphoreForEvent:event];
    A0bNode *node = [self nodeForEvent:event];

    if (sema) {
        [self error:@"Semaphore %@ being recreated by %@", sema, event];
        return;
    }
    sema = [A0bSemaphore semaphoreWithId:[event objectForKey:@"Semaphore"]
                                    node:node];
    [sema setSimulator:self];
    [sema a0NewSemaphore:event];
    [node addSemaphore:sema];
}

- (void)a0TryPSemaphore:(A0Event *)event
{
    A0bSemaphore *sema = [self semaphoreForEvent:event];
    NSAssert1(sema, @"Unknown semaphore for event %@", [event description]);

    [sema a0TryPSemaphore:event];
}

- (void)a0PSemaphore:(A0Event *)event
{
    A0bSemaphore *sema = [self semaphoreForEvent:event];
    NSAssert1(sema, @"Unknown semaphore for event %@", [event description]);

    [sema a0PSemaphore:event];
}

- (void)a0VSemaphore:(A0Event *)event
/* unblocks first blocked thread or increments semaphore value */
{
    A0bSemaphore *sema = [self semaphoreForEvent:event];
    NSAssert1(sema, @"Unknown semaphore for event %@", [event description]);

    [sema a0VSemaphore:event];
}

- (void)a0DisposeSemaphore:(A0Event *)event
{
    A0bSemaphore *sema = [self semaphoreForEvent:event];
    NSAssert1(sema, @"Unknown semaphore for event %@", [event description]);

    [sema a0DisposeSemaphore:event];
}

- (void)a0NewMutex:(A0Event *)event
{
    A0bMutex *mutex = [self mutexForEvent:event];
    A0bNode *node = [self nodeForEvent:event];

    if (mutex) {
        [self error:@"Mutex %@ being recreated by %@", mutex, event];
        return;
    }
    mutex = [A0bMutex mutexWithId:[event objectForKey:@"Mutex"]
                             node:node];
    [mutex setSimulator:self];
    [mutex a0NewMutex:event];
    [node addMutex:mutex];
}

- (void)a0LockMutex:(A0Event *)event
{
    A0bMutex *mutex = [self mutexForEvent:event];
    //KLUDGE, there is a bug in the tracer
    if (!mutex) {
        [self a0NewMutex:event];
        mutex = [self mutexForEvent:event];
    }
    //ENDKLUDGE
    NSAssert1(mutex, @"Unknown mutex for event %@", [event description]);

    [mutex a0LockMutex:event];
}

- (void)a0UnlockMutex:(A0Event *)event
/* unblocks first blocked thread or increments semaphore value */
{
    A0bMutex *mutex = [self mutexForEvent:event];
    NSAssert1(mutex, @"Unknown mutex for event %@", [event description]);

    [mutex a0UnlockMutex:event];
}

- (void)a0DisposeMutex:(A0Event *)event
{
    A0bMutex *mutex = [self mutexForEvent:event];
    NSAssert1(mutex, @"Unknown mutex for event %@", [event description]);

    [mutex a0DisposeMutex:event];
}

/*
NS_MESSAGE(self, a0NewCondition:),      @"a0NewCondition",
NS_MESSAGE(self, a0SignalCondition:),   @"a0SignalCondition",
NS_MESSAGE(self, a0BroadcastCondition:),@"a0BroadcastCondition",
NS_MESSAGE(self, a0WaitCondition:),     @"a0WaitCondition",
NS_MESSAGE(self, a0TimedWaitCondition:),@"a0TimedWaitCondition",
NS_MESSAGE(self, a0DisposeCondition:),  @"a0DisposeCondition",
*/
- (void)a0NewCondition:(A0Event *)event
/*
    %EventDef	a0NewCondition	52
    %	Time	double
    %	Node	int
    %	Thread	int
    %	Condition	hex
   */
{
    
}
- (void)a0SignalCondition:(A0Event *)event
/*
    %EventDef	a0SignalCondition	90
    %	Time	double
    %	Node	int
    %	Thread	int
    %	Condition	hex
   */
{
    
}
- (void)a0BroadcastCondition:(A0Event *)event
/*
    %EventDef	a0BroadcastCondition	4
    %	Time	double
    %	Node	int
    %	Thread	int
    %	Condition	hex
    %EndEventDef
   */
{
    
}
- (void)a0WaitCondition:(A0Event *)event
/*
    %EventDef	a0WaitCondition	108
    %	Time	double
    %	Node	int
    %	Thread	int
    %	Condition	hex
    %	Mutex	hex
    %	EndTime	double
   */
{
    A0bMutex *mutex = [self mutexForEvent:event];
    NSAssert1(mutex, @"Unknown mutex for event %@", [event description]);

    [mutex a0WaitCondition:event];
}
- (void)a0TimedWaitCondition:(A0Event *)event
/*
    %EventDef	a0TimedWaitCondition	96
    %	Time	double
    %	Node	int
    %	Thread	int
    %	Condition	hex
    %	Mutex	hex
    %	TimeOut_sec	int
    %	TimeOut_nsec	int
    %	EndTime	double
   */
{
    
}
- (void)a0DisposeCondition:(A0Event *)event
/*
    %EventDef	a0DisposeCondition	12
    %	Time	double
    %	Node	int
    %	Thread	int
    %	Condition	hex
*/
{
    
}
@end
