/* A0bSimul-Thread.m created by benhur on Mon 21-Apr-1997 */

#import "A0bSimul.h"

@implementation A0bSimul (Threads)

- (void)a0Init:(A0Event *)event
{
    // create main thread, detach it as it can't be joined
//    [[self startThread:[event thread]
//                atNode:[event node]
//                atTime:[event time]
//             withEvent:event] detach];
    //BUG: should create daemon threads here.
    [[self threadForEvent:event] setSystemThread:NO];
    return;
}

- (void)a0NewSlave:(A0Event *)event
{
    id nodeId = [event node];
    id newThreadId = [event objectForKey:@"OtherThread"];
    A0bThread *newThread;

    newThread = [self startThread:newThreadId
                           atNode:nodeId
                           atTime:[event time]
                        withEvent:event];

    [newThread setSystemThread:NO];
    [event setObject:newThread forKey:@"OtherThread"];

    [self outputEntity:
        [A0Link newSlaveLinkFromEvent:event toState:[newThread currentState]]];
}

- (void)a0StartRemoteThread:(A0Event *)event
{
    id sourceNode, destNode, enumer;
    A0Event *ev;
    id sourceNodeId = [event node];
    id destNodeId = [event objectForKey:@"DestNode"];
    
    // test to see if the thread has already started (if clocks aren't sync'd)
    sourceNode = [nodes objectForKey:sourceNodeId];
    enumer = [[sourceNode pendingThreadCreations] objectEnumerator];
    while (ev = [enumer nextObject]) {
        if ([A0_BeginThreadServiceEventName isEqual:[ev name]]
            && [destNodeId isEqual:[ev node]]) {

            [self outputEntity:
                [A0Link startRemoteThreadLinkFromEvent:event toEvent:ev]];

            [[sourceNode pendingThreadCreations] removeObjectIdenticalTo:ev];
            break;
        }
    }
    
    // if thread not started, insert event in dest node's pending list
    if (!ev) {
        destNode = [nodes objectForKey:destNodeId];
        [[destNode pendingThreadCreations] addObject:event];
    }
}

- (void)a0_BeginThreadService:(A0Event *)event
{
    id sourceNode, destNode, enumer;
    A0Event *ev;
    id sourceNodeId = [event objectForKey:@"SourceNode"];
    id destNodeId = [event node];
    A0bThread *newThread;
    
    // thread created by otherNode has started
    // it is automatically detached, because it can't be joined
    newThread = [self startThread:[event objectForKey:@"StartedThread"]
                           atNode:destNodeId
                           atTime:[event time]
                        withEvent:event];
    [newThread setSystemThread:NO];
    [newThread detach];
    [event setObject:[event thread] forKey:@"OriginalThread"];
    [event setObject:newThread forKey:@"Thread"];
    
    // search for request event, and generate a correspondence, if found
    destNode = [nodes objectForKey:destNodeId];
    enumer = [[destNode pendingThreadCreations] objectEnumerator];
    while (ev = [enumer nextObject]) {
        if (([A0IStartRemoteThreadEventName isEqual:[ev name]]
             || [A0StartRemoteThreadEventName isEqual:[ev name]])
            && [sourceNodeId isEqual:[ev node]]) {

            [self outputEntity:
                [A0Link startRemoteThreadLinkFromEvent:ev toEvent:event]];

            [[destNode pendingThreadCreations] removeObjectIdenticalTo:ev];
            break;
        }
    }

    // if request is not yet received, put event in source node's pending list
    if(!ev) {
        sourceNode = [nodes objectForKey:sourceNodeId];
        [[sourceNode pendingThreadCreations] addObject:event];
    }
}


- (void)a0JoinSlave: (A0Event *)event
{
    A0bThread *joinedThread;

    [self blockThreadWithEvent:event];
    
    joinedThread = [self threadForNode:[event node]
                                thread:[event objectForKey:@"OtherThread"]];

    if (!joinedThread) {
        [self error:@"Joining unknown thread: %@", event];
        return;
    }
    if ([joinedThread isDetached]) {
        [self error:@"Joining detached thread: %@", event];
        return;
    }

    [joinedThread setJoinState:[[self threadForEvent:event] currentState]];
}

- (void)a0DetachSlave:(A0Event *)event
{
    [[[self nodeForEvent:event]
        threadWithId:[event objectForKey:@"OtherThread"]] detach];
}

- (void)a0_EndUserThread:(A0Event *)event
{
    [self terminateThread:[self threadForEvent:event]
                   atTime:[event time]
                withEvent:event];
}

- (void)a0StartRemoteUrgent:(A0Event *)event
{
    id sourceNode, destNode, enumer;
    A0Event *ev;
    id sourceNodeId = [event node];
    id destNodeId = [event objectForKey:@"DestNode"];

    // test to see if the thread has already started (clocks not sync'd)
    sourceNode = [nodes objectForKey:sourceNodeId];
    enumer = [[sourceNode pendingThreadCreations] objectEnumerator];
    while (ev = [enumer nextObject]) {
        if ([A0_BeginUrgentServiceEventName isEqual:[ev name]]
            && [destNodeId isEqual:[ev node]]) {

            [self outputEntity:
                [A0Link startRemoteUrgentLinkFromEvent:event toEvent:ev]];

            [[sourceNode pendingThreadCreations] removeObjectIdenticalTo:ev];
            break;
        }
    }

    // if thread not started, insert event in dest node's pending queue
    if (!ev) {
        destNode = [nodes objectForKey:destNodeId];
        [[destNode pendingThreadCreations] addObject:event];
    }
}

- (void)a0_BeginUrgentService:(A0Event *)event
{
    id sourceNode, destNode, enumer;
    A0Event *ev;
    id sourceNodeId = [event objectForKey:@"SourceNode"];
    id destNodeId = [event node];
    A0bThread *newThread;

    // thread created by otherNode has started
    // how to identify this thread? (it's executed by the daemon)
    // it is detached, because it can't be joined
    newThread = [self startThread:[event thread]
                atNode:destNodeId
                atTime:[event time]
                        withEvent:event];

    [newThread setSystemThread:NO];
    [newThread detach];
    
    // search for request event, and generate a correspondence, if found
    destNode = [nodes objectForKey:destNodeId];
    enumer = [[destNode pendingThreadCreations] objectEnumerator];
    while (ev = [enumer nextObject]) {
        if (([A0IStartRemoteUrgentEventName isEqual:[ev name]]
              || [A0StartRemoteUrgentEventName isEqual:[ev name]])
            && [sourceNodeId isEqual:[ev node]]) {

            [self outputEntity:
                [A0Link startRemoteUrgentLinkFromEvent:ev toEvent:event]];

            [[destNode pendingThreadCreations] removeObjectIdenticalTo:ev];
            break;
        }
    }

    // if request is not yet received, put event in source node's pending queue
    if(!ev) {
        sourceNode = [nodes objectForKey:sourceNodeId];
        [[sourceNode pendingThreadCreations] addObject:event];
    }
}


- (void)a0_EndUrgentService:(A0Event *)event
{
    [self terminateThread:[self threadForEvent:event]
                   atTime:[event time]
                withEvent:event];
}

- (void)a0Terminate:(A0Event *)event
{
    [self terminateAllThreadsOfNode:[event node]
                             atTime:[event time]
                          withEvent:event];
}

- (void)a0UnblockThread:(A0Event *)event
{
    A0bThread *thread = [self threadForEvent:event];
    NSAssert1(thread, @"event in unknown thread %@", event);

    [thread a0UnblockThread:event];
}
@end
