/* A0bSimul-User.m created by benhur on Mon 3-jun-1999 */

#import "A0bSimul.h"
#import "SimulContainer.h"
#import "../General.bproj/PajeType.h"
#import "UserEvent.h"
#import "../General.bproj/Macros.h"

@implementation A0bSimul (UserEvents)
//
// User defined entities
//
// We don't yet have adequate support for user defined entities in the
// trace library of Athapascan-0. Meanwhile, we do what we can, coding
// the wanted events in the string of the a0UserEvent call. This string
// cannot be too big, so the coding is a bit hard to read.
// The possible events are:
//
// a0T_Dxy#      a0T_DefineUserEntity(#, X, Y)
//   define a new type of entity
//      type of entity        x  X
//                            E  A0T_EVENT
//                            S  A0T_STATE
//                            L  A0T_LINK
//      level of attachment   y  Y
//                            T  A0T_THREAD
//                            N  A0T_NODE
//                            G  A0T_GLOBAL
//      entity id             #  #           (must be unique)
//      example: a0UserEvent("a0T_DST0", 0)
//
// a0T_I#,$      a0T_InstantiateUserEntity(#, $)
//   create instance $ of entity type #
//
// a0T_N#,s      a0T_NameUserEntity(#, s)
//   give name s to entity number #
//
// a0T_S#,$,str  a0T_SetUserEntityValue(#, $, str)
//   set entity # to value $ (a number, that can have been named),
//   with comment str
//
// a0T_P#,$,str  a0TPushUserEntityValue(#, $, str)
//
// a0T_p#        a0TPopUserEntity(#)
//
#if 0
- (void)a0UserEventOld:(A0Event *)event
{
    NSString *str = [event objectForKey:@"String"];
    NSString *type, *entityType, *entityName, *name;
    NSNumber *number, *typeNumber, *nameNumber;
    NSArray *comps;
    if (![str hasPrefix:@"a0T_"])
        return;
    switch ([str characterAtIndex:4]) {
        case 'D':
            type = [str substringWithRange:NSMakeRange(5,2)];
            number = [NSNumber numberWithInt:[[str substringFromIndex:7] intValue]];
            [userNumberToType setObject:type forKey:number];
            break;
        case 'I':
            break;
        case 'N':
            comps = [[str substringFromIndex:5] componentsSeparatedByString:@","];
            number = [NSNumber numberWithInt:[[comps objectAtIndex:0] intValue]];
            name = [comps objectAtIndex:1];
            [userNumberToName setObject:name forKey:number];
            break;
        case 'S':
            comps = [[str substringFromIndex:5] componentsSeparatedByString:@","];
            typeNumber = [NSNumber numberWithInt:[[comps objectAtIndex:0] intValue]];
            nameNumber = [NSNumber numberWithInt:[[comps objectAtIndex:1] intValue]];
            entityType = [userNumberToName objectForKey:typeNumber];
            entityName = [userNumberToName objectForKey:nameNumber];
            type = [userNumberToType objectForKey:typeNumber];
            if (!type) return;
            if (!entityType) entityType = typeNumber;
            if (!entityName) entityName = nameNumber;
            [[self threadForEvent:event] changeUserStateOfType:entityType
                                                       toState:entityName
                                                     withEvent:event];
           break;
        case 'P':
            comps = [[str substringFromIndex:5] componentsSeparatedByString:@","];
            typeNumber = [NSNumber numberWithInt:[[comps objectAtIndex:0] intValue]];
            nameNumber = [NSNumber numberWithInt:[[comps objectAtIndex:1] intValue]];
            entityType = [userNumberToName objectForKey:typeNumber];
            entityName = [userNumberToName objectForKey:nameNumber];
            type = [userNumberToType objectForKey:typeNumber];
            if (!type) return;
            if (!entityType) entityType = typeNumber;
            if (!entityName) entityName = nameNumber;
            [[self threadForEvent:event] pushUserStateOfType:entityType
                                                     toState:entityName
                                                   withEvent:event];
            break;
        case 'p':
            typeNumber = [NSNumber numberWithInt:[[str substringFromIndex:5] intValue]];
            entityType = [userNumberToName objectForKey:typeNumber];
            type = [userNumberToType objectForKey:typeNumber];
            if (!type) return;
            if (!entityType) entityType = typeNumber;
            [[self threadForEvent:event] popUserStateOfType:entityType
                                                  withEvent:event];
            break;
    }
}
#endif

//
// new version
//
//

- (void)_initUserSimulator
{
    Assign(userNumberToType, ([NSMutableDictionary dictionaryWithObjectsAndKeys:
        [A0bSimul executionEntityType],     @"0",
        [A0bNode class],      @"1",
        [A0bThread class],    @"2",
        [A0bSemaphore class], @"3",
        [A0bMutex class],     @"4",
        nil]));
    Assign(userNumberToContainer, [NSMutableDictionary dictionary]);
}

- (id)containerOfNumber:(NSNumber *)containerNumber
                   type:(id)containerType
                inEvent:(A0Event *)event
{
    if ([containerType isEqual:[A0bSimul executionEntityType]])
        return [self rootContainer];
    if ([containerType isEqual:[A0bNode class]])
        return [self nodeForEvent:event];
    if ([containerType isEqual:[A0bThread class]])
        return [self threadForEvent:event];
    if ([containerType isEqual:[A0bSemaphore class]])
        return [self semaphoreForEvent:event];
    if ([containerType isEqual:[A0bMutex class]])
        return [self mutexForEvent:event];
    
    return [userNumberToContainer objectForKey:containerNumber];
}


- (void)a0DefineUserContainerType:(A0Event *)event
{
    NSNumber *newContainerTypeNumber;
    NSNumber *containerTypeNumber;
    NSString *newContainerTypeName;

    PajeContainerType *containerType;
    PajeContainerType *newContainerType;

    // get fields from event
    newContainerTypeNumber = [event objectForKey:@"NewType"];
    containerTypeNumber    = [event objectForKey:@"ContainerType"];
    newContainerTypeName   = [event objectForKey:@"NewName"];

    containerType = [userNumberToType objectForKey:containerTypeNumber];
    if (!containerType) {
        NSLog(@"Unknown user container type %@ in event %@",
              containerTypeNumber, event);
        return;
    }

    newContainerType = [PajeContainerType typeWithName:newContainerTypeName
                                         containerType:containerType];
    [userNumberToType setObject:newContainerType forKey:newContainerTypeNumber];
}

- (void)a0DefineUserLinkType:(A0Event *)event
{
    NSNumber *newEntityTypeNumber;
    NSNumber *containerTypeNumber;
    NSNumber *sourceContainerTypeNumber;
    NSNumber *destContainerTypeNumber;
    NSString *newEntityTypeName;

    PajeContainerType *containerType;
    PajeContainerType *sourceContainerType;
    PajeContainerType *destContainerType;
    PajeEntityType *newEntityType;

    // get fields from event
    newEntityTypeNumber       = [event objectForKey:@"NewType"];
    containerTypeNumber       = [event objectForKey:@"ContainerType"];
    sourceContainerTypeNumber = [event objectForKey:@"SourceContainerType"];
    destContainerTypeNumber   = [event objectForKey:@"DestinContainerType"];
    newEntityTypeName         = [event objectForKey:@"NewName"];

    containerType = [userNumberToType objectForKey:containerTypeNumber];
    if (!containerType) {
        NSLog(@"Unknown user container type %@ in event %@",
              containerTypeNumber, event);
        return;
    }

    sourceContainerType = [userNumberToType objectForKey:sourceContainerTypeNumber];
    if (!containerType) {
        NSLog(@"Unknown user container type %@ in event %@",
              sourceContainerTypeNumber, event);
        return;
    }

    destContainerType = [userNumberToType objectForKey:destContainerTypeNumber];
    if (!containerType) {
        NSLog(@"Unknown user container type %@ in event %@",
              destContainerTypeNumber, event);
        return;
    }

    newEntityType = [PajeLinkType typeWithName:newEntityTypeName
                                 containerType:containerType
                           sourceContainerType:sourceContainerType
                             destContainerType:destContainerType];
    [userNumberToType setObject:newEntityType forKey:newEntityTypeNumber];
}

- (void)_defineUserEntityType:(A0Event *)event
                  drawingType:(PajeDrawingType)drawingType
{
    NSNumber *newEntityTypeNumber;
    NSNumber *containerTypeNumber;
    NSString *newEntityTypeName;

    PajeContainerType *containerType;
    PajeEntityType *newEntityType;

    // get fields from event
    newEntityTypeNumber = [event objectForKey:@"NewType"];
    containerTypeNumber = [event objectForKey:@"ContainerType"];
    newEntityTypeName   = [event objectForKey:@"NewName"];

    containerType = [userNumberToType objectForKey:containerTypeNumber];
    if (!containerType) {
        NSLog(@"Unknown user container type %@ in event %@",
              containerTypeNumber, event);
        return;
    }

    if (drawingType == PajeEventDrawingType)
        newEntityType = [PajeEventType typeWithName:newEntityTypeName
                                      containerType:containerType];
    else if (drawingType == PajeStateDrawingType)
        newEntityType = [PajeStateType typeWithName:newEntityTypeName
                                      containerType:containerType];
    else if (drawingType == PajeVariableDrawingType)
           newEntityType = [PajeVariableType typeWithName:newEntityTypeName
                                         containerType:containerType];
    else {
        NSLog(@"Unknown entity type %@ in event %@",
              newEntityTypeNumber, event);
        return;
    }
    [userNumberToType setObject:newEntityType forKey:newEntityTypeNumber];
}

- (void)a0DefineUserEventType:(A0Event *)event
{
    [self _defineUserEntityType:event drawingType:PajeEventDrawingType];
}

- (void)a0DefineUserStateType:(A0Event *)event
{
    [self _defineUserEntityType:event drawingType:PajeStateDrawingType];
}

- (void)a0DefineUserVariableType:(A0Event *)event
{
    [self _defineUserEntityType:event drawingType:PajeVariableDrawingType];
}


- (void)a0DefineUserEntityValue:(A0Event *)event
{
    NSNumber *newEntityValueNumber;
    NSNumber *entityTypeNumber;
    NSString *newEntityValueName;

    PajeEntityType *entityType;

    // get fields from event
    newEntityValueNumber = [event objectForKey:@"NewValue"];
    entityTypeNumber     = [event objectForKey:@"EntityType"];
    newEntityValueName   = [event objectForKey:@"NewName"];

    entityType = [userNumberToType objectForKey:entityTypeNumber];
    if (!entityType) {
        NSLog(@"Unknown user entity type %@ in event %@",
              entityTypeNumber, event);
        return;
    }

    if ([entityType drawingType] == PajeVariableDrawingType) {
        NSLog(@"Cannot set name to variable values of type %@ in event %@",
             [entityType name], event);
        return;
    }
    
    [(PajeEventType *)entityType setName:newEntityValueName
                                forValue:newEntityValueNumber];
}



- (void)a0CreateUserContainer:(A0Event *)event
{
    NSNumber *newContainerNumber;
    NSNumber *newContainerTypeNumber;
    NSNumber *containerNumber;
    NSString *newContainerName;

    PajeContainerType *typeOfNewContainer;
    PajeContainer *container;
    PajeContainer *newContainer;

    // get fields from event
    newContainerNumber     = [event objectForKey:@"NewContainer"];
    newContainerTypeNumber = [event objectForKey:@"NewContainerType"];
    containerNumber        = [event objectForKey:@"Container"];
    newContainerName       = [event objectForKey:@"NewName"];

    typeOfNewContainer = [userNumberToType objectForKey:newContainerTypeNumber];
    if (!typeOfNewContainer) {
        NSLog(@"Unknown user container type %@ in event %@",
              newContainerTypeNumber, event);
        return;
    }
    
    container = [self containerOfNumber:containerNumber
                                   type:[typeOfNewContainer containerType]
                                inEvent:event];
    if (!container) {
        NSLog(@"Unknown user container %@ in event %@",
              containerNumber, event);
        return;
    }
    

    newContainer = [SimulContainer containerWithType:typeOfNewContainer
                                                name:newContainerName
                                           container:container
                                        creationTime:[event time]
                                           simulator:self];
    [container addSubContainer:newContainer];
    [userNumberToContainer setObject:newContainer forKey:newContainerNumber];
}

- (void)a0DestroyUserContainer:(A0Event *)event
{
    NSNumber *containerNumber;

    SimulContainer *container;

    // get fields from event
    containerNumber = [event objectForKey:@"Container"];

    container = [userNumberToContainer objectForKey:containerNumber];
    if (!container) {
        NSLog(@"Unknown user container %@ in event %@",
              containerNumber, event);
        return;
    }

    [container stopWithEvent:event];
}



- (BOOL)_getEntityType:(id *)entityType
                 value:(id *)entityValue
             container:(id *)container
             fromEvent:(A0Event *)event
{
    NSNumber *entityTypeNumber;
    NSNumber *containerNumber;

    if (entityType) {
        entityTypeNumber  = [event objectForKey:@"EntityType"];
        *entityType = [userNumberToType objectForKey:entityTypeNumber];
        if (!*entityType) {
            NSLog(@"Unknown user entity type %@ in event %@",
                  entityTypeNumber, event);
            return NO;
        }
    }

    if (entityType && container) {
        containerNumber   = [event objectForKey:@"Container"];
        *container = [self containerOfNumber:containerNumber
                                        type:[*entityType containerType]
                                     inEvent:event];
        if (!*container) {
            NSLog(@"Unknown user container %@ in event %@",
                  containerNumber, event);
            return NO;
        }
    }

    if (entityValue)
        *entityValue = [event objectForKey:@"Value"];

    return YES;
}

- (void)a0NewUserEvent:(A0Event *)event
{
    PajeEventType *entityType;
    PajeContainer *container;
    id entityValue;
    id entityName;
    UserEvent *userEntity;

    if (![self _getEntityType:&entityType
                        value:&entityValue
                    container:&container
                    fromEvent:event])
        return;

    entityName = [entityType nameForValue:entityValue];
    userEntity = [UserEvent eventWithType:entityType
                                     name:entityName
                                container:container
                                    event:event];

    [self outputEntity:userEntity];
}

- (void)a0SetUserState:(A0Event *)event
{
    PajeStateType *entityType;
    SimulContainer *container;
    id entityValue;
    id entityName;

    if (![self _getEntityType:&entityType
                        value:&entityValue
                    container:&container
                    fromEvent:event])
        return;
    
    entityName = [entityType nameForValue:entityValue];
    [container setUserStateOfType:entityType
                          toValue:entityName
                        withEvent:event];
}

- (void)a0PushUserState:(A0Event *)event
{
    PajeStateType *entityType;
    SimulContainer *container;
    id entityValue;
    id entityName;

    if (![self _getEntityType:&entityType
                        value:&entityValue
                    container:&container
                    fromEvent:event])
        return;

    entityName = [entityType nameForValue:entityValue];
    [container pushUserStateOfType:entityType
                             value:entityName
                         withEvent:event];
}

- (void)a0PopUserState:(A0Event *)event
{
    PajeStateType *entityType;
    SimulContainer *container;

    if (![self _getEntityType:&entityType
                        value:NULL
                    container:&container
                    fromEvent:event])
        return;

    [container popUserStateOfType:entityType
                        withEvent:event];
}

- (void)a0SetUserVariable:(A0Event *)event
{
    PajeVariableType *entityType;
    SimulContainer *container;
    id entityValue;

    if (![self _getEntityType:&entityType
                        value:&entityValue
                    container:&container
                    fromEvent:event])
        return;

    [container setUserVariableOfType:entityType
                             toValue:entityValue
                           withEvent:event];
}
- (void)a0AddUserVariable:(A0Event *)event
{
    PajeVariableType *entityType;
    SimulContainer *container;
    id entityValue;

    if (![self _getEntityType:&entityType
                        value:&entityValue
                    container:&container
                    fromEvent:event])
        return;

    [container addUserVariableOfType:entityType
                               value:entityValue
                           withEvent:event];
}
- (void)a0SubUserVariable:(A0Event *)event
{
    PajeVariableType *entityType;
    SimulContainer *container;
    id entityValue;

    if (![self _getEntityType:&entityType
                        value:&entityValue
                    container:&container
                    fromEvent:event])
        return;

    [container subUserVariableOfType:entityType
                               value:entityValue
                           withEvent:event];
}


- (void)a0StartUserLink:(A0Event *)event
{
    NSNumber *sourceContainerNumber;
    PajeLinkType *entityType;
    SimulContainer *container;
    PajeContainer *sourceContainer;
    id entityValue;
    id entityName;

    if (![self _getEntityType:&entityType
                        value:&entityValue
                    container:&container
                    fromEvent:event])
        return;

    entityName = [entityType nameForValue:entityValue];

    sourceContainerNumber = [event objectForKey:@"SourceContainer"];
    sourceContainer = [self containerOfNumber:sourceContainerNumber
                                         type:[entityType sourceContainerType]
                                      inEvent:event];
    if (!sourceContainer) {
        NSLog(@"Unknown user container %@ in event %@",
              sourceContainerNumber, event);
        return;
    }

    [container startUserLinkOfType:entityType
                             value:entityName
                   sourceContainer:sourceContainer
                               key:[event objectForKey:@"Key"]
                         withEvent:event];
}

- (void)a0EndUserLink:(A0Event *)event
{
    NSNumber *destContainerNumber;
    PajeLinkType *entityType;
    SimulContainer *container;
    PajeContainer *destContainer;
    id entityValue;
    id entityName;

    if (![self _getEntityType:&entityType
                        value:&entityValue
                    container:&container
                    fromEvent:event])
        return;

    entityName = [entityType nameForValue:entityValue];

    destContainerNumber = [event objectForKey:@"DestContainer"];
    destContainer = [self containerOfNumber:destContainerNumber
                                       type:[entityType destContainerType]
                                    inEvent:event];
    if (!destContainer) {
        NSLog(@"Unknown user container %@ in event %@",
              destContainerNumber, event);
        return;
    }

    [container endUserLinkOfType:entityType
                           value:entityName
                   destContainer:destContainer
                             key:[event objectForKey:@"Key"]
                       withEvent:event];
}



- (void)a0UserEvent:(A0Event *)event
{
    NSString *str = [event objectForKey:@"String"];
    NSArray *comps;
    NSArray *names;
    static NSDictionary *eventFieldNames = nil;

//    if ([str hasPrefix:@"a0_T"])
//        return [self a0UserEventOld:event];
    
    if (![str hasPrefix:@"a0"])
        return;

    if (!eventFieldNames)
        eventFieldNames = [[NSDictionary dictionaryWithObjectsAndKeys:
            [NSArray arrayWithObjects:@"Name", @"NewType", @"ContainerType", @"NewName", nil], A0DefineUserContainerType,
            [NSArray arrayWithObjects:@"Name", @"NewType", @"ContainerType", @"NewName", nil], A0DefineUserEventType,
            [NSArray arrayWithObjects:@"Name", @"NewType", @"ContainerType", @"NewName", nil], A0DefineUserStateType,
            [NSArray arrayWithObjects:@"Name", @"NewType", @"ContainerType", @"NewName", nil], A0DefineUserVariableType,
            [NSArray arrayWithObjects:@"Name", @"NewType", @"ContainerType", @"SourceContainerType", @"DestinContainerType", @"NewName", nil], A0DefineUserLinkType,
            [NSArray arrayWithObjects:@"Name", @"NewValue", @"EntityType", @"NewName", nil], A0DefineUserEntityValue,
            [NSArray arrayWithObjects:@"Name", @"NewContainer", @"NewContainerType", @"Container", @"NewName", nil], A0CreateUserContainer,
            [NSArray arrayWithObjects:@"Name", @"Container", nil], A0DestroyUserContainer,
            [NSArray arrayWithObjects:@"Name", @"EntityType", @"Container", @"Value", @"Comment", nil], A0NewUserEvent,
            [NSArray arrayWithObjects:@"Name", @"EntityType", @"Container", @"Value", @"Comment", nil], A0SetUserState,
            [NSArray arrayWithObjects:@"Name", @"EntityType", @"Container", @"Value", @"Comment", nil], A0PushUserState,
            [NSArray arrayWithObjects:@"Name", @"EntityType", @"Container", @"Comment", nil], A0PopUserState,
            [NSArray arrayWithObjects:@"Name", @"EntityType", @"Container", @"Value", @"Comment", nil], A0SetUserVariable,
            [NSArray arrayWithObjects:@"Name", @"EntityType", @"Container", @"Value", @"Comment", nil], A0AddUserVariable,
            [NSArray arrayWithObjects:@"Name", @"EntityType", @"Container", @"Value", @"Comment", nil], A0SubUserVariable,
            [NSArray arrayWithObjects:@"Name", @"EntityType", @"Container", @"Value", @"SourceContainer", @"Key", @"Comment", nil], A0StartUserLink,
            [NSArray arrayWithObjects:@"Name", @"EntityType", @"Container", @"Value", @"DestContainer", @"Key", @"Comment", nil], A0EndUserLink,
            nil] retain];

    comps = [str componentsSeparatedByString:@";"];

    names = [eventFieldNames objectForKey:[comps objectAtIndex:0]];

    if (names) {
        A0Event *newEvent = [A0Event eventWithObjects:comps
                                           forKeys:names];
        [newEvent setNode:[event node]];
        [newEvent setThread:[event thread]];
        [newEvent setTime:[event time]];
        [self inputEntity:newEvent];
    }
}

@end
