/*
 * A0bSimul.h
 *
 * Interface of simulator for Athapascan-0b traces.
 *
 * 19961126 BS  creation
 */

#import <Foundation/Foundation.h>

#import "A0bNode.h"
#import "A0bThread.h"
#import "A0State.h"
#import "A0Comm.h"
#import "A0bSemaphore.h"
#import "A0bMutex.h"
#import "../General.bproj/PajeFilter.h"

#define EVENT_OUTPUT_PORT 0
#define STATE_OUTPUT_PORT 1
#define COMM_OUTPUT_PORT 2
#define SEMA_STATE_OUTPUT_PORT 3

@interface A0bSimul : PajeComponent
{
    PajeContainer *rootContainer;
    NSDictionary *traceInfo;
    NSMutableDictionary *nodes;
    NSMutableDictionary *invocationTable;
    NSMutableDictionary *fileToFilename;
    NSMutableDictionary *filenameToReferences;
    
    NSMutableDictionary *userNumberToType;
    NSMutableDictionary *userNumberToContainer;

    NSMutableDictionary *userEntities; // current user states, mapped by entityType

    int traceVersion;
}

+ (PajeContainerType *)executionEntityType;
- (PajeContainer *)rootContainer;

- init;
- (void)error:(NSString *)format, ...;

- (NSDate *)startTime;
- (NSDate *)endTime;
- (NSString *)description;

- (A0bNode *)nodeForEvent:(A0Event *)event;
- (A0bThread *)threadForNode:(id)nodeId thread:(id)threadId;
- (A0bThread *)threadForEvent:(A0Event *)event;
- (A0bSemaphore *)semaphoreForEvent:(A0Event *)event;
- (A0bMutex *)mutexForEvent:(A0Event *)event;

- (void)inputEntity:(A0Event *)event;
- (void)outputEntity:(id)entity;
- (void)entityChangedEndTime:(id)entity;

- (A0bThread *)startThread:(id)threadId
                    atNode:(id)nodeId
                    atTime:(id)time
                 withEvent:(A0Event *)event;
- (void)blockThreadWithEvent:(A0Event *)event;
- (void)terminateThread:(A0bThread *)thread
                 atTime:(id)time
              withEvent:(A0Event *)event;
- (void)terminateAllThreadsOfNode:(id)nodeId
                           atTime:(id)time
                        withEvent:(A0Event *)event;

- (NSDate *)startTime;
- (NSDate *)endTime;

- (void)encodeCheckPointWithCoder:(NSCoder *)coder;
- (void)decodeCheckPointWithCoder:(NSCoder *)coder;
@end

#include "EventNames.h"
