/*
 * A0bSimul.m
 *
 * Simulator for Athapascan-0b traces.
 *
 * 19961126 BS  creation
 */


#import "A0bSimul.h"
#import "../A0bReader.bproj/A0bThreadId.h"
#import "../General.bproj/UniqueString.h"
#import "../General.bproj/SourceCodeReference.h"
#import "../General.bproj/NSDictionary+Additions.h"
#import "../General.bproj/Macros.h"

#define DEFINE_STRINGS
#include "EventNames.h"
#undef DEFINE_STRINGS

PajeContainerType *A0ExecutionEntityType;

@implementation A0bSimul

+ (void)initialize
{
    if (self == [A0bSimul class]) {
        Assign(A0ExecutionEntityType, [PajeContainerType typeWithName:@"Execution"
                                                    containerType:nil]);
#define INIT_STRINGS
#include "EventNames.h"
#undef INIT_STRINGS
    }
}

- (void)error:(NSString *)format, ...
{
    va_list args;

    va_start(args, format);
    NSLogv(format, args);
    va_end(args);
}

+ (PajeEntityType *)executionEntityType { return A0ExecutionEntityType; }

- (PajeContainer *)rootContainer { return rootContainer; }

- (NSString *)name {
    return NSStringFromClass([self class]);
}
- (NSArray *)relatedEntities {
    return [NSArray array];
}
- (PajeDrawingType)drawingType {
    return PajeContainerDrawingType;
}

#ifndef xxGNUSTEP
#define NS_MESSAGE(obj, meth)                                   \
    ((invocation = [NSInvocation invocationWithMethodSignature: \
    [obj methodSignatureForSelector: @selector(meth)]]),        \
    [invocation setTarget: obj],                                \
    [invocation setSelector: @selector(meth)],                  \
    invocation)
#else
#define NS_MESSAGE(target, message)                             \
    ([[NSInvocation alloc] initWithTarget:target                \
    selector:@selector(message), nil])
#endif

#define U(s) [UniqueString stringWithString:s]

- init
{
    NSInvocation *invocation;
    self = [super init];

    if (self != nil) {
        rootContainer = [PajeContainer alloc];
        [rootContainer initWithType:A0ExecutionEntityType
                               name:@"X"
                          container:rootContainer];
        nodes = [[NSMutableDictionary alloc] init];
        fileToFilename = [[NSMutableDictionary alloc] init];
        filenameToReferences = [[NSMutableDictionary alloc] init];

        [self _initUserSimulator];
    
    #if 0
    invocationTable = [[NSDictionary dictionaryWithObjectsAndKeys:
        NS_MESSAGE(self, a0_AllocTraceBuffer:),   A0_AllocTraceBufferEventName,
        NS_MESSAGE(self, a0_IdentifyFile:),       A0_IdentifyFileEventName,
        // Threads
        NS_MESSAGE(self, a0Init:),                A0InitEventName,
        NS_MESSAGE(self, a0NewSlave:),            A0NewSlaveEventName,
        NS_MESSAGE(self, a0JoinSlave:),           A0JoinSlaveEventName,
        NS_MESSAGE(self, a0DetachSlave:),         A0DetachSlaveEventName,
        NS_MESSAGE(self, a0StartRemoteThread:),   A0StartRemoteThreadEventName,
        NS_MESSAGE(self, a0StartRemoteThread:),   A0IStartRemoteThreadEventName,
        NS_MESSAGE(self, a0_BeginThreadService:), A0_BeginThreadServiceEventName,
//        NS_MESSAGE(self, a0ExitThread:),          A0ExitThreadEventName,
        NS_MESSAGE(self, a0StartRemoteUrgent:),   A0StartRemoteUrgentEventName,
        NS_MESSAGE(self, a0StartRemoteUrgent:),   A0IStartRemoteUrgentEventName,
        NS_MESSAGE(self, a0_EndUserThread:),      A0_EndUserThreadEventName,
        NS_MESSAGE(self, a0_BeginUrgentService:), A0_BeginUrgentServiceEventName,
        NS_MESSAGE(self, a0_EndUrgentService:),   A0_EndUrgentServiceEventName,
        NS_MESSAGE(self, a0Terminate:),           A0TerminateEventName,
        NS_MESSAGE(self, a0UnblockThread:),       A0_UnblockEventName,

        // Messages
        NS_MESSAGE(self, a0Send:),                A0SendEventName,
        NS_MESSAGE(self, a0Send:),                A0ISendEventName,
        NS_MESSAGE(self, a0Send:),                A0SendBufferEventName,
        NS_MESSAGE(self, a0Send:),                A0ISendBufferEventName,
        NS_MESSAGE(self, a0Send:),                A0FSendEventName,
        NS_MESSAGE(self, a0Send:),                A0IFSendEventName,
        NS_MESSAGE(self, a0_SendCompletion:),     A0_SendCompletionEventName,
        NS_MESSAGE(self, a0Receive:),             A0ReceiveEventName,
        NS_MESSAGE(self, a0Receive:),             A0IReceiveEventName,
        NS_MESSAGE(self, a0Receive:),             A0ReceiveBufferEventName,
        NS_MESSAGE(self, a0Receive:),             A0IReceiveBufferEventName,
        NS_MESSAGE(self, a0Receive:),             A0ReceiveAnyEventName,
        NS_MESSAGE(self, a0Receive:),             A0IReceiveAnyEventName,
        NS_MESSAGE(self, a0Receive:),             A0ReceiveBufferAnyEventName,
        NS_MESSAGE(self, a0Receive:),             A0IReceiveBufferAnyEventName,
        NS_MESSAGE(self, a0Receive:),             A0FReceiveEventName,
        NS_MESSAGE(self, a0Receive:),             A0IFReceiveEventName,
        NS_MESSAGE(self, a0Receive:),             A0FReceiveAnyEventName,
        NS_MESSAGE(self, a0Receive:),             A0IFReceiveAnyEventName,
        NS_MESSAGE(self, a0_ReceiveCompletion:),  A0_ReceiveCompletionEventName,
        NS_MESSAGE(self, a0WaitRequest:),         A0WaitRequestEventName,
        NS_MESSAGE(self, a0WaitRequest:),         A0WaitAnyRequestEventName,

        NS_MESSAGE(self, a0NewPipe:),             A0NewPipeEventName,

        // Synchronization
        NS_MESSAGE(self, a0NewSemaphore:),        A0NewSemaphoreEventName,
        NS_MESSAGE(self, a0PSemaphore:),          A0PSemaphoreEventName,
        NS_MESSAGE(self, a0TryPSemaphore:),       A0TryPSemaphoreEventName,
        NS_MESSAGE(self, a0VSemaphore:),          A0VSemaphoreEventName,
        NS_MESSAGE(self, a0DisposeSemaphore:),    A0DisposeSemaphoreEventName,
        NS_MESSAGE(self, a0NewMutex:),            A0NewMutexEventName,
        NS_MESSAGE(self, a0LockMutex:),           A0LockMutexEventName,
        NS_MESSAGE(self, a0UnlockMutex:),         A0UnlockMutexEventName,
        NS_MESSAGE(self, a0DisposeMutex:),        A0DisposeMutexEventName,
//        NS_MESSAGE(self, a0NewCondition:),        A0NewConditionEventName,
//        NS_MESSAGE(self, a0SignalCondition:),     A0SignalConditionEventName,
//        NS_MESSAGE(self, a0BroadcastCondition:),  A0BroadcastConditionEventName,

        NS_MESSAGE(self, a0WaitCondition:),       A0WaitConditionEventName,
//        NS_MESSAGE(self, a0TimedWaitCondition:),  A0TimedWaitConditionEventName,
//        NS_MESSAGE(self, a0DisposeCondition:),    A0DisposeConditionEventName,

        NS_MESSAGE(self, a0UserEvent:),           A0UserEventEventName,

        NS_MESSAGE(self, a0DefineUserContainerType:), A0DefineUserContainerType,
        NS_MESSAGE(self, a0DefineUserEventType:), A0DefineUserEventType,
        NS_MESSAGE(self, a0DefineUserStateType:), A0DefineUserStateType,
        NS_MESSAGE(self, a0DefineUserVariableType:), A0DefineUserVariableType,
        NS_MESSAGE(self, a0DefineUserLinkType:),  A0DefineUserLinkType,
        NS_MESSAGE(self, a0DefineUserEntityValue:), A0DefineUserEntityValue,
        NS_MESSAGE(self, a0CreateUserContainer:), A0CreateUserContainer,
        NS_MESSAGE(self, a0DestroyUserContainer:), A0DestroyUserContainer,
        NS_MESSAGE(self, a0NewUserEvent:),        A0NewUserEvent,
        NS_MESSAGE(self, a0SetUserState:),        A0SetUserState,
        NS_MESSAGE(self, a0PushUserState:),       A0PushUserState,
        NS_MESSAGE(self, a0PopUserState:),        A0PopUserState,
        NS_MESSAGE(self, a0SetUserVariable:),     A0SetUserVariable,
        NS_MESSAGE(self, a0AddUserVariable:),     A0AddUserVariable,
        NS_MESSAGE(self, a0SubUserVariable:),     A0SubUserVariable,
        NS_MESSAGE(self, a0StartUserLink:),       A0StartUserLink,
        NS_MESSAGE(self, a0EndUserLink:),         A0EndUserLink,

        nil] retain];
        #else
    invocationTable = [NSMutableDictionary dictionary];
    [invocationTable setObject:NS_MESSAGE(self, a0_AllocTraceBuffer:)   forKey:A0_AllocTraceBufferEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0_IdentifyFile:)       forKey:A0_IdentifyFileEventName];
        // Threads
    [invocationTable setObject:NS_MESSAGE(self, a0Init:)                forKey:A0InitEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0NewSlave:)            forKey:A0NewSlaveEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0JoinSlave:)           forKey:A0JoinSlaveEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0DetachSlave:)         forKey:A0DetachSlaveEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0StartRemoteThread:)   forKey:A0StartRemoteThreadEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0StartRemoteThread:)   forKey:A0IStartRemoteThreadEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0_BeginThreadService:) forKey:A0_BeginThreadServiceEventName];
//    [invocationTable setObject:NS_MESSAGE(self, a0ExitThread:)          forKey:A0ExitThreadEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0StartRemoteUrgent:)   forKey:A0StartRemoteUrgentEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0StartRemoteUrgent:)   forKey:A0IStartRemoteUrgentEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0_EndUserThread:)      forKey:A0_EndUserThreadEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0_BeginUrgentService:) forKey:A0_BeginUrgentServiceEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0_EndUrgentService:)   forKey:A0_EndUrgentServiceEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Terminate:)           forKey:A0TerminateEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0UnblockThread:)       forKey:A0_UnblockEventName];

        // Messages
    [invocationTable setObject:NS_MESSAGE(self, a0Send:)                forKey:A0SendEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Send:)                forKey:A0ISendEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Send:)                forKey:A0SendBufferEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Send:)                forKey:A0ISendBufferEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Send:)                forKey:A0FSendEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Send:)                forKey:A0IFSendEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0_SendCompletion:)     forKey:A0_SendCompletionEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0ReceiveEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0IReceiveEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0ReceiveBufferEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0IReceiveBufferEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0ReceiveAnyEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0IReceiveAnyEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0ReceiveBufferAnyEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0IReceiveBufferAnyEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0FReceiveEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0IFReceiveEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0FReceiveAnyEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0Receive:)             forKey:A0IFReceiveAnyEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0_ReceiveCompletion:)  forKey:A0_ReceiveCompletionEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0WaitRequest:)         forKey:A0WaitRequestEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0WaitRequest:)         forKey:A0WaitAnyRequestEventName];

    [invocationTable setObject:NS_MESSAGE(self, a0NewPipe:)             forKey:A0NewPipeEventName];

        // Synchronization
    [invocationTable setObject:NS_MESSAGE(self, a0NewSemaphore:)        forKey:A0NewSemaphoreEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0PSemaphore:)          forKey:A0PSemaphoreEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0TryPSemaphore:)       forKey:A0TryPSemaphoreEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0VSemaphore:)          forKey:A0VSemaphoreEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0DisposeSemaphore:)    forKey:A0DisposeSemaphoreEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0NewMutex:)            forKey:A0NewMutexEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0LockMutex:)           forKey:A0LockMutexEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0UnlockMutex:)         forKey:A0UnlockMutexEventName];
    [invocationTable setObject:NS_MESSAGE(self, a0DisposeMutex:)        forKey:A0DisposeMutexEventName];
//    [invocationTable setObject:NS_MESSAGE(self, a0NewCondition:)        forKey:A0NewConditionEventName];
//    [invocationTable setObject:NS_MESSAGE(self, a0SignalCondition:)     forKey:A0SignalConditionEventName];
//    [invocationTable setObject:NS_MESSAGE(self, a0BroadcastCondition:)  forKey:A0BroadcastConditionEventName];

    [invocationTable setObject:NS_MESSAGE(self, a0WaitCondition:)       forKey:A0WaitConditionEventName];
//    [invocationTable setObject:NS_MESSAGE(self, a0TimedWaitCondition:)  forKey:A0TimedWaitConditionEventName];
//    [invocationTable setObject:NS_MESSAGE(self, a0DisposeCondition:)    forKey:A0DisposeConditionEventName];

    [invocationTable setObject:NS_MESSAGE(self, a0UserEvent:)           forKey:A0UserEventEventName];

    [invocationTable setObject:NS_MESSAGE(self, a0DefineUserContainerType:) forKey:A0DefineUserContainerType];
    [invocationTable setObject:NS_MESSAGE(self, a0DefineUserEventType:) forKey:A0DefineUserEventType];
    [invocationTable setObject:NS_MESSAGE(self, a0DefineUserStateType:) forKey:A0DefineUserStateType];
    [invocationTable setObject:NS_MESSAGE(self, a0DefineUserVariableType:) forKey:A0DefineUserVariableType];
    [invocationTable setObject:NS_MESSAGE(self, a0DefineUserLinkType:)  forKey:A0DefineUserLinkType];
    [invocationTable setObject:NS_MESSAGE(self, a0DefineUserEntityValue:) forKey:A0DefineUserEntityValue];
    [invocationTable setObject:NS_MESSAGE(self, a0CreateUserContainer:) forKey:A0CreateUserContainer];
    [invocationTable setObject:NS_MESSAGE(self, a0DestroyUserContainer:) forKey:A0DestroyUserContainer];
    [invocationTable setObject:NS_MESSAGE(self, a0NewUserEvent:)        forKey:A0NewUserEvent];
    [invocationTable setObject:NS_MESSAGE(self, a0SetUserState:)        forKey:A0SetUserState];
    [invocationTable setObject:NS_MESSAGE(self, a0PushUserState:)       forKey:A0PushUserState];
    [invocationTable setObject:NS_MESSAGE(self, a0PopUserState:)        forKey:A0PopUserState];
    [invocationTable setObject:NS_MESSAGE(self, a0SetUserVariable:)     forKey:A0SetUserVariable];
    [invocationTable setObject:NS_MESSAGE(self, a0AddUserVariable:)     forKey:A0AddUserVariable];
    [invocationTable setObject:NS_MESSAGE(self, a0SubUserVariable:)     forKey:A0SubUserVariable];
    [invocationTable setObject:NS_MESSAGE(self, a0StartUserLink:)       forKey:A0StartUserLink];
    [invocationTable setObject:NS_MESSAGE(self, a0EndUserLink:)         forKey:A0EndUserLink];

    [invocationTable retain];
        #endif

    // prepare to receive a notification each time the user selects a file
    [[NSNotificationCenter defaultCenter]
       addObserver:self
          selector:@selector(fileSelected:)
              name:@"PajeFilenameNotification"
            object:nil];

    Assign(userEntities, [NSMutableDictionary dictionary]);
    }
    
    return self;
}

- (void)dealloc
{
    [rootContainer release];
    [nodes release];
    [invocationTable release];
    [userNumberToType release];
    [userNumberToContainer release];
    [fileToFilename release];
    [filenameToReferences release];
    [super dealloc];
}

- (void)reset
{
    [nodes removeAllObjects];
}


- (void)inputEntity:(A0Event *)event
{
    A0bThread *thread;
    NSInvocation *invocation;
    id file;
    NSString *filename;

    // set event's filename
    file = [event objectForKey:@"File"];
    if (file)
        filename = [fileToFilename objectForKey:file];
    else
        filename = [event objectForKey:@"FileName"];
    if (filename) {
        id line;
        int lineNumber;
        SourceCodeReference *sourceRef;
        NSMutableDictionary *refToEvents;

        line = [event objectForKey:@"Line"];
        if (line == nil)
            line = [event objectForKey:@"LineNumber"];
        lineNumber = [line intValue];
        sourceRef = [SourceCodeReference referenceToFilename:filename
                                                  lineNumber:lineNumber];
        refToEvents = [filenameToReferences objectForKey:filename];
        [refToEvents addObject:event forKey:sourceRef];
        [event removeObjectForKey:@"File"];
        [event removeObjectForKey:@"FileName"];
        [event removeObjectForKey:@"Line"];
        [event removeObjectForKey:@"LineNumber"];
        [event setObject:sourceRef forKey:@"FileReference"];
    }

    thread = [self threadForEvent:event];
    if (thread)
        [event setThread:thread];

simulate_bug_event:
    // bacalhau for a bug in the tracer (see A0bThread.h)
    if ([thread isWaitingForUnblockEvent]
        && ![[event name] isEqual:A0_UnblockEventName]) {
        if ([thread bugEvent])
            NSAssert1(0, @"more than one event before unblock %@", event);
        NSLog(@"saving bug event at %@", [[event time] description]);
        [thread setBugEvent:event];
        return;
    }
    // end of fix

    
    // invoke the method that simulates this event
    invocation = [invocationTable objectForKey:[event name]];
    if (invocation) {
        [invocation setArgument:&event atIndex:2];
        [invocation invoke];
    }

    if (!thread) {
        thread = [self threadForEvent:event];
        if (thread)
            [event setThread:thread];
    }

    // send the event out
    [self outputEntity:event];

    // more on the bug fix...
    event = [thread bugEvent];
    if (event) {
        [[event retain] autorelease];
        [thread setBugEvent:nil];
        NSLog(@"using bug event at %@", [[event time] description]);
        goto simulate_bug_event;
    }
}

- (void)a0_AllocTraceBuffer:(A0Event *)event
// the first event in any thread
{
    A0bThread *thread = [self threadForEvent:event];
    if (!thread) {
        id threadId = [event thread];
        id nodeId = [event node];
        id time = [event time];
        thread = [self startThread:threadId
                            atNode:nodeId
                            atTime:time
                         withEvent:event];
    }
}

- (void)fileSelected:(NSNotification *)notification
/* receives PajeFilenameNotification, with file name, trace size, node count. */
{
    // this should not be a notification, but a simulation of the first event in the trace.
    [self reset];
    traceInfo = [[notification userInfo] retain];
    if ([traceInfo objectForKey:@"TraceVersion"])
        traceVersion = [[traceInfo objectForKey:@"TraceVersion"] intValue];
}

- (NSDate *)startTime
{
    return [traceInfo objectForKey:@"StartTime"];
}

- (NSDate *)endTime
{
    return [traceInfo objectForKey:@"EndTime"];
}

- (NSString *)description
{
    return [traceInfo objectForKey:@"Filename"];
}

id only;
+ (id)onlyInstance{
    return only;
}

- (void)a0_IdentifyFile:(A0Event *)event
{
    id filename = [event objectForKey:@"FileName"];
    id file = [event objectForKey:@"File"];
    id existantName;

    only=self;
    
    if (!file) return;

    existantName = [fileToFilename objectForKey:file];

    if (existantName && ![existantName isEqual:filename]) {
        NSLog(@"File %@, defined as %@ beeing redefined as %@ by event %@",
              file, existantName, filename, event);
    } else {
        [fileToFilename setObject:U(filename) forKey:file];
        [filenameToReferences setObject:[NSMutableDictionary dictionary]
                                 forKey:filename];
    }
}

- (void)selectLine:(unsigned)lineNumber inFile:(NSString *)filename
{
    NSLog(@"selected line %d in file %@", lineNumber, filename);
    [[NSClassFromString(@"DrawView") onlyInstance] highlightEntities:[[filenameToReferences objectForKey:[filename lastPathComponent]]
objectForKey:[SourceCodeReference referenceToFilename:[filename lastPathComponent]
                                           lineNumber:lineNumber]]];
}

- (A0bThread *)threadForNode:(id)nodeId thread:(id)threadId
{
    A0bNode *node;
    if ([threadId isKindOfClass:[A0bThread class]])
        return threadId;
    if ([nodeId isKindOfClass:[A0bNode class]])
        node = nodeId;
    else
        node = [nodes objectForKey:nodeId];
    return [node threadWithId:threadId];
}

- (A0bThread *)threadForEvent:(A0Event *)event
{
    id thread = [event thread];
    if ([thread isKindOfClass:[A0bThread class]])
        return thread;
    return [[self nodeForEvent:event] threadWithId:thread];
}

- (A0bSemaphore *)semaphoreForEvent:(A0Event *)event
{
    id sema = [event objectForKey:@"Semaphore"];
    if ([sema isKindOfClass:[A0bSemaphore class]])
        return sema;

    return [[self nodeForEvent:event] semaphoreWithId:sema];
}

- (A0bMutex *)mutexForEvent:(A0Event *)event
{
    id mutex = [event objectForKey:@"Mutex"];
    if ([mutex isKindOfClass:[A0bMutex class]])
        return mutex;

    return [[self nodeForEvent:event] mutexWithId:mutex];
}

- (A0bNode *)nodeForEvent:(A0Event *)event
{
    id node = [event node];
    if ([node isKindOfClass:[A0bNode class]])
        return node;
    return [nodes objectForKey:node];
}

- (void)outputEntity:(id)entity
{
//#if 0
//    if ([entity isKindOfClass:[A0Comm class]])
//        NSLog(@"COMM:%.6f %.6f %@", [[entity startTime] timeIntervalSinceReferenceDate],              [[entity endTime] timeIntervalSinceReferenceDate], [entity size]);
    if ([entity isKindOfClass:[A0Event class]]
        || [entity isKindOfClass:[A0ThreadState class]]) {
        id thread = [entity thread];
        if (![thread isKindOfClass:[A0bThread class]]
            || [thread isSystemThread]) {
//            NSLog(@"ignoring entity:%@", entity);
            return;
        }
      }
//#endif
    [super outputEntity:entity];
}

- (void)entityChangedEndTime:(id)entity
{
    [outputComponent entityChangedEndTime:entity];
}

- (A0bThread *)startThread:(id)threadId
                    atNode:(id)nodeId
                    atTime:(id)time
                 withEvent:(A0Event *)event
{
    id node = [nodes objectForKey:nodeId];
    A0bThread *thread;
    id serviceId;

    if (!node) {
        node = [A0bNode nodeWithId:nodeId
                         simulator:self
                         container:rootContainer];
        [nodes setObject:node forKey:nodeId];
    }

    if (thread = [node threadWithId:threadId]) {
        [self error:@"Restarting thread: %@ with event: %@", thread, event];
        return thread;
    }
    //event can be: NewPipe, Init, NewSlave, BeginThreadService, BeginUrgentService
    //BUG:only the last two identify the service the thread being created will exec.
    //BUG: services (and slave threads) should be identified by name
    serviceId = [event objectForKey:@"Service"];
    if (!serviceId) {
        NSString *evName = [event name];
        if ([evName isEqual:A0InitEventName])
            serviceId = @"main";
        else if ([evName isEqual:A0NewPipeEventName])
            serviceId = @"Pipe";
        else
            serviceId = @"Unknown";
    }
        
    thread = [A0bThread threadWithId:threadId node:node service:serviceId
                        creationTime:time creationEvent:event simulator:self];
    if ([[event name] isEqual:A0_BeginThreadServiceEventName]
        || ![[event thread] isKindOfClass:[A0bThread class]])
        [event setThread:thread];
    [node addThread:thread];

    return thread;
}


- (void)blockThreadWithEvent:(A0Event *)event
    /*
     * event blocks its thread (until "EndTime" in event).
     * generates running (before blocking) and blocked thread states.
     * does nothing if event doesn't have EndTime.
     */
{
    A0bThread *thread;

//    if (![event objectForKey:@"EndTime"])
//        return;

    thread = [self threadForEvent:event];
    if (thread == nil) {
        [self error:@"Event in unexistant thread: %@", event];
        return;
    }

    [thread blockWithEvent:event unblockSelector:NULL target:nil];

    return;
}


- (void)terminateThread:(A0bThread *)thread
                 atTime:(id)time
              withEvent:(A0Event *)event
{
    if (thread == nil) {
        NSLog(@"Terminating non existant thread with event %@\n", event);
        return;
    }

    if (![thread isTerminated]) {
        [thread terminateWithEvent:event atTime:time];

        // send an end-of-thread event
/*        [[self outputNum:EVENT_OUTPUT_PORT] setPortValue:[A0Event eventWithObjects:[NSArray arrayWithObjects:@"EndThread", [thread node], thread, [thread serviceId], time, nil] forKeys:[NSArray arrayWithObjects:@"Name", @"Node", @"Thread", @"ThreadService", @"Time", nil]]];*/
    }
}

- (void)terminateAllThreadsOfNode:(id)nodeId
                           atTime:(id)time
                        withEvent:(A0Event *)event
{
    A0bNode *node = [nodes objectForKey:nodeId];
    id en = [node threadsEnumerator];
    A0bThread *thread;

    while ((thread = [en nextObject]) != nil) {
        [self terminateThread:thread atTime:time withEvent:event];
    }
//    printf("A0bSimul: terminating node: %s\n", [[node description] cString]);
}

- (void)encodeCheckPointWithCoder:(NSCoder *)coder
{
    [coder encodeObject:nodes];
    [coder encodeObject:fileToFilename];
}

- (void)decodeCheckPointWithCoder:(NSCoder *)coder
{
    Assign(nodes, [coder decodeObject]);
    Assign(fileToFilename, [coder decodeObject]);
}

- (id)copyWithZone:(NSZone *)zone
{
    return [self retain];
}
@end
