/* A0bThread.h
 *
 *
 * 19970128 BS creation
 */

#import <Foundation/Foundation.h>

#import "../A0bReader.bproj/A0Event.h"
#import "A0State.h"
#import "UserState.h"
#import "UserLink.h"
@class A0bSimul, A0bNode;

typedef struct {
    unsigned system:1;
    unsigned detached:1;
    unsigned terminated:1;
    unsigned joined:1;
} thread_status_t;

@interface A0bThread : PajeContainer
{
    id  serviceId;
    A0ThreadState *currentState;
    A0bSimul *simulator;
    id lastTime;
    NSDate *creationTime;
    thread_status_t status;
    SEL unblockSelector;
    id unblockTarget;
    // there is a bug in the tracer in which sometimes the unblocking event
    // that should come immediately after any blocking event comes after
    // the following event. For these cases, the event that came in the 
    // wrong order is put here, until the unblocking event comes.
    A0Event *bugEvent;
    // entity to make the join link to; can be the last event of this thread
    // or the join state of the joining thread, whichever comes first in time
    id joinEntity;
    
    NSMutableDictionary *userEntities; // current user states, mapped by entityType
}


+ (A0bThread *)threadWithId:(id)threadId
                       node:(A0bNode *)nodeId
                    service:(id)servId
               creationTime:(id)time
              creationEvent:(A0Event *)event
                  simulator:(A0bSimul *)simul;

- (id)initWithId:(id)thrId
            node:(A0bNode *)nId
         service:(id)servId
    creationTime:(id)time
   creationEvent:(A0Event *)event
       simulator:(A0bSimul *)simul;


- (void)terminateWithEvent:(A0Event *)event
                    atTime:(id)time;

+ (PajeContainerType *)entityType;

- (void)detach;
- (BOOL)isDetached;
- (BOOL)isSystemThread;
- (BOOL)isTerminated;
- (void)setSystemThread:(BOOL)flag;

- (void)setLastTime:(id)time;
- (id)lastTime;
- (void)setJoinState:(id)state;

- (id)threadId;
- (id)serviceId;

- (void)blockWithEvent:(A0Event *)event
       unblockSelector:(SEL)selector
                target:(id)target;

- (void)setCurrentState:(A0ThreadState *)newCurrentState;
- (A0ThreadState *)currentState;

- (void)a0UnblockThread:(A0Event *)event;

- (void)setBugEvent:(A0Event *)event;
- (A0Event *)bugEvent;
- (BOOL)isWaitingForUnblockEvent;

- (id)node;

// NSCoding Protocol
- (void)encodeWithCoder:(NSCoder *)coder;
- (id)initWithCoder:(NSCoder *)coder;
@end

@interface A0bThread (UserStates)
- (NSMutableArray *)userStateArrayForEntityType:(id)entityType;
- (void)changeUserStateOfType:(id)entityType
                      toState:(id)name
                    withEvent:(A0Event*)event;
- (void)pushUserStateOfType:(id)entityType
                    toState:(id)name
                  withEvent:(A0Event*)event;
- (void)popUserStateOfType:(id)entityType
                 withEvent:(A0Event *)event;
@end
