//
// PajeContainer
//
// superclass for containers
//

#import <Foundation/Foundation.h>
#import "../General.bproj/PajeContainer.h"
#import "../A0bReader.bproj/A0Event.h"

@interface SimulContainer : PajeContainer
{
    NSDate *creationTime;
    NSDate *lastTime;
    id simulator;
    NSMutableDictionary *userEntities; // key = entityType
    NSMutableDictionary *minValues; // key = entityType
    NSMutableDictionary *maxValues; // key = entityType
    int logicalTime;
}

+ (SimulContainer *)containerWithType:(PajeEntityType *)type
                                 name:(NSString *)n
                           container:(PajeContainer *)newcontainer
                        creationTime:(NSDate *)time
                           simulator:(id)simul;
- (id)initWithType:(PajeEntityType *)type
              name:(NSString *)n
         container:(PajeContainer *)c
      creationTime:(NSDate *)time
         simulator:(id)simul;

- (void)stopWithEvent:(A0Event*)event;

- (NSDate *)startTime;
- (NSDate *)endTime;

- (void)setLastTime:(NSDate *)time;

- (id)entityOfType:(PajeEntityType *)entityType;
- (void)setEntity:(id)entity ofType:(PajeEntityType *)entityType;
- (void)pushEntity:(id)entity ofType:(PajeEntityType *)entityType;
- (id)popEntityOfType:(PajeEntityType *)entityType;
- (id)popEntity;

- (void)setUserStateOfType:(PajeEntityType *)entityType
                   toValue:(id)value
                 withEvent:(A0Event*)event;
- (void)pushUserStateOfType:(PajeEntityType *)entityType
                      value:(id)value
                  withEvent:(A0Event*)event;
- (void)popUserStateOfType:(PajeEntityType *)entityType
                 withEvent:(A0Event*)event;

- (void)setUserVariableOfType:(PajeVariableType *)entityType
                      toValue:(id)entityName
                    withEvent:(A0Event*)event;
- (void)addUserVariableOfType:(PajeVariableType *)entityType
                        value:(id)entityName
                    withEvent:(A0Event*)event;
- (void)subUserVariableOfType:(PajeVariableType *)entityType
                        value:(id)entityName
                    withEvent:(A0Event*)event;

- (void)startUserLinkOfType:(PajeEntityType *)entityType
                      value:(id)entityName
            sourceContainer:(PajeContainer *)sourceContainer
                        key:(id)key
                  withEvent:(A0Event*)event;
- (void)endUserLinkOfType:(PajeEntityType *)entityType
                    value:(id)entityName
            destContainer:(PajeContainer *)destContainer
                      key:(id)key
                withEvent:(A0Event*)event;

- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType;
- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType;

- (int)logicalTime;
- (void)setLogicalTime:(int)lt;
@end
