//
// UserEvent
//
// Holds a generic user-generated event.
//

#import "UserEvent.h"
#import "../General.bproj/Macros.h"

@implementation UserEvent

+ (UserEvent *)eventWithType:(PajeEntityType *)type
                        name:(id)n
                   container:(PajeContainer *)c
                       event:(A0Event *)e
{
    return [[[self alloc] initWithType:type
                                  name:n
                             container:c
                                 event:e] autorelease];
}

- (id)initWithType:(PajeEntityType *)type
              name:(id)n
         container:(PajeContainer *)c
             event:(A0Event *)e
{
    self = [super initWithType:type
                          name:n
                     container:c];

    if (self)
        Assign(event, e);

    return self;
}

- (void)dealloc
{
    Assign(event, nil);

    [super dealloc];
}

- (NSDate *)time
{
    return [event time];
}


- (NSString *)description
{
    return [NSString stringWithFormat:@"%@ \"%@\"",
                                      name, [event objectForKey:@"Comment"]];
}

- (NSArray *)fieldNames
{
    return [[super fieldNames] arrayByAddingObjectsFromArray: [event fieldNames]];
}

- (id)valueOfFieldNamed:(NSString *)fieldName
{
    id value;
    value = [super valueOfFieldNamed:fieldName];
    if (value != nil)
        return value;
    value = [event valueOfFieldNamed:fieldName];
    return value;
}

@end

@implementation UserEventInspector
- (void)addLocalFields
{
    [super addLocalFields];
}
@end