//
// UserEvent
//
// holds a user-definable link
//

#import "UserLink.h"
#import "../General.bproj/Macros.h"
#import "../General.bproj/PajeContainer.h"


@implementation UserLink

+ (UserLink *)linkOfType:(PajeEntityType *)type
                   value:(id)v
                     key:(id)k
               container:(PajeContainer *)c
         sourceContainer:(PajeContainer *)sc
             sourceEvent:(A0Event *)e
{
    return [[[self alloc] initWithType:type
                                 value:v
                                   key:k
                             container:c
                       sourceContainer:sc
                           sourceEvent:e] autorelease];
}

+ (UserLink *)linkOfType:(PajeEntityType *)type
                   value:(id)v
                     key:(id)k
               container:(PajeContainer *)c
           destContainer:(PajeContainer *)dc
               destEvent:(A0Event *)e;
{
    return [[[self alloc] initWithType:type
                                 value:v
                                   key:k
                             container:c
                         destContainer:dc
                             destEvent:e] autorelease];
}

- (id)initWithType:(PajeEntityType *)type
             value:(id)v
               key:(id)k
         container:(PajeContainer *)c
   sourceContainer:(PajeContainer *)sc
       sourceEvent:(A0Event *)e
{
    self = [super initWithType:type
                         value:v
                     container:c
                    startEvent:e];
    if (self) {
        Assign(key, k);
        Assign(sourceContainer, sc);
    }
    return self;
}

- (id)initWithType:(PajeEntityType *)type
             value:(id)v
               key:(id)k
         container:(PajeContainer *)c
     destContainer:(PajeContainer *)dc
         destEvent:(A0Event *)e
{
    self = [super initWithType:type
                         value:v
                     container:c
                    startEvent:nil];
    if (self) {
        [self setEndEvent:e];
        Assign(key, k);
        Assign(destContainer, dc);
    }
    return self;
}


- (void)dealloc
{
    Assign(key, nil);
    Assign(sourceContainer, nil);
    Assign(destContainer, nil);
    [super dealloc];
}

- (void)setSourceContainer:(PajeContainer *)sc
               sourceEvent:(A0Event *)e
{
    Assign(sourceContainer, sc);
    Assign(event, e);
}

- (void)setDestContainer:(PajeContainer *)dc
               destEvent:(A0Event *)e
{
    Assign(destContainer, dc);
    [self setEndEvent:e];
}

- (BOOL)canBeEndedWithValue:(id)v key:(id)k
{
    return ((destContainer == nil)
            && [[self value] isEqual:v]
            && [key isEqual:k]);
}

- (BOOL)canBeStartedWithValue:(id)v key:(id)k
{
    return ((sourceContainer == nil)
            && [[self value] isEqual:v]
            && [key isEqual:k]);
}

- (PajeContainer *)container
{
    return container;
}
- (PajeContainer *)sourceContainer
{
    return sourceContainer;
}
- (PajeContainer *)destContainer
{
    return destContainer;
}

- (PajeEntityType *)sourceEntityType
{
    return [sourceContainer entityType];//[entityType sourceContainerType];
}

- (PajeEntityType *)destEntityType
{
    return [destContainer entityType];//[entityType destContainerType];
}

- (NSArray *)fieldNames
{
    NSArray *localFields;
    localFields = [NSArray arrayWithObjects:
        @"SourceContainer", @"DestContainer", @"Key",
        @"StartLogical", @"EndLogical",
        nil];
    return [[super fieldNames] arrayByAddingObjectsFromArray:localFields];
}

- (id)valueOfFieldNamed:(NSString *)fieldName
{
    id value;
    if ([fieldName isEqual:@"SourceContainer"])
        return [self sourceContainer];
    else if ([fieldName isEqual:@"DestContainer"])
        return [self destContainer];
    else if ([fieldName isEqual:@"Key"])
        return key;
    else if ([fieldName isEqual:@"StartLogical"])
        return [NSString stringWithFormat:@"%d", [self startLogicalTime]];
    else if ([fieldName isEqual:@"EndLogical"])
        return [NSString stringWithFormat:@"%d", [self endLogicalTime]];
    else value = [super valueOfFieldNamed:fieldName];
    return value;
}

- (void)setStartLogicalTime:(int)t
{
    startLogicalTime = t;
}
- (void)setEndLogicalTime:(int)t
{
    endLogicalTime = t;
}
- (int)startLogicalTime
{
    return startLogicalTime;
}
- (int)endLogicalTime
{
    return endLogicalTime;
}
- (NSDate *)xstartTime
{
    return [NSDate dateWithTimeIntervalSinceReferenceDate:startLogicalTime+991683572];
}
- (NSDate *)xendTime
{
    return [NSDate dateWithTimeIntervalSinceReferenceDate:endLogicalTime+991683572];
}
@end

@implementation UserLinkInspector
- (void)addLocalFields
{
    NSBox *box;
    NSMutableArray *fieldTitles = [NSMutableArray array];
    NSMutableArray *fieldValues = [NSMutableArray array];
    PajeContainer *container;
    PajeContainer *sourceContainer;
    PajeContainer *destContainer;

    NSLog(@"Logical: %d -- %d %@ %@", [inspectedEntity startLogicalTime], [inspectedEntity endLogicalTime], [inspectedEntity startTime], NSStringFromClass([[inspectedEntity startTime] class]));
    [super addLocalFields];
    
    container = [inspectedEntity container];

    // source container information
    sourceContainer = [inspectedEntity sourceContainer];
    if (sourceContainer != nil)
        while (![sourceContainer isEqual:container]) {
            [fieldTitles addObject:[[sourceContainer entityType] name]];
            [fieldValues addObject:[sourceContainer name]];
            if (sourceContainer == [sourceContainer container]) break; //HACK
            sourceContainer = [sourceContainer container];
        }
    [nonDisplayedFields removeObject:@"SourceContainer"];

    box = [self boxWithTitle:@"Source"
                 fieldTitles:fieldTitles
                 fieldValues:fieldValues];
    [self addSubview:box atBottom:NO];

    // dest container information
    [fieldTitles removeAllObjects];
    [fieldValues removeAllObjects];
    destContainer = [inspectedEntity destContainer];
    if (destContainer != nil)
        while (![destContainer isEqual:container]) {
            [fieldTitles addObject:[[destContainer entityType] name]];
            [fieldValues addObject:[destContainer name]];
            if (destContainer == [destContainer container]) break; //HACK
            destContainer = [destContainer container];
        }
    [nonDisplayedFields removeObject:@"DestContainer"];

    box = [self boxWithTitle:@"Destination"
                 fieldTitles:fieldTitles
                 fieldValues:fieldValues];
    [self addSubview:box atBottom:NO];
}
@end
