/* BusyNode.m created by benhur on Fri 27-Feb-1998 */

#import "BusyNode.h"
#import "ReduceEntity.h"
#import "../General.bproj/UniqueString.h"
#import "../General.bproj/Macros.h"

@implementation BusyNode

- (id)init
{
    self = [super init];

    if (self) {

        // load the interface. it initializes view.
        if (![NSBundle loadNibNamed:@"BusyNode" owner:self])
            NSRunAlertPanel(NSStringFromClass([self class]),
                            @"Couldn't load interface file",
                            nil, nil, nil);

        // view is an NSBox. we need its contents.
        view = [view contentView];
#ifndef GNUSTEP
        [[view window] autorelease];
#endif
        [view removeFromSuperview];

        reduceEntityTypes = [[NSMutableSet alloc] init];
        [self readDefaults];

        [[NSNotificationCenter defaultCenter] postNotificationName:@"PajeRegisterFilterNotification" object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
            NSStringFromClass([self class]), @"Name",
            view, @"View", nil]];
        
    }
    return self;
}

- (void)dealloc
{
    [reduceEntityTypes release];
    [super dealloc];
}


- (void)readDefaults
{
    NSString *defaultName;

    defaultName = [NSStringFromClass([self class]) stringByAppendingString:@" ReduceEntityTypes"];
    entityTypesDictionary = [[[NSUserDefaults standardUserDefaults] dictionaryForKey:defaultName] mutableCopy];
    if (entityTypesDictionary == nil)
        entityTypesDictionary = [[NSMutableDictionary alloc] init];
}

- (void)registerDefaults
{
    NSString *defaultName;

    defaultName = [NSStringFromClass([self class]) stringByAppendingString:@" ReduceEntityTypes"];

    [[NSUserDefaults standardUserDefaults] setObject:entityTypesDictionary
                                              forKey:defaultName];
}

//TODO treat -dataChangedForEntityType:
//- (void)dataChangedForEntityType:(id)entityType
//{
//    [self hierarchyChanged];
//}

- (void)hierarchyChanged
{
    [hierarchyTree release];
    hierarchyTree = [[inputComponent hierarchyTree] copy];
    [self calcEntityTypes];
    [self calcEntityNamePopUp];
    [self calcEntityTypePopUp];
    [self calcGroupPopUp];
    [self calcReduceModePopUp];
    [self refreshMatrix];

    [super hierarchyChanged];
}

- (void)calcEntityTypes
{
    NSEnumerator *filterEnum;
    NSDictionary *filter;
    ReduceEntityType *entityType;

    [reduceEntityTypes removeAllObjects];
    filterEnum = [entityTypesDictionary objectEnumerator];
    while (filter = [filterEnum nextObject]) {
        entityType = [ReduceEntityType typeFromDictionary:filter component:self];
        if (entityType) {
            [reduceEntityTypes addObject:entityType];
            [self addToHierarchy:entityType];
        }
    }
}

- (void)calcEntityNamePopUp
{
    NSEnumerator *typeEnumerator;
    ReduceEntityType *selectedType;
    ReduceEntityType *type;

    selectedType = [[entityNamePopUp selectedItem] representedObject];
    [entityNamePopUp removeAllItems];
    typeEnumerator = [reduceEntityTypes objectEnumerator];
    while (type = [typeEnumerator nextObject]) {
        [entityNamePopUp addItemWithTitle:[type name]];
        [[entityNamePopUp itemWithTitle:[type name]] setRepresentedObject:type];
    }
    if (selectedType != nil)
        [entityNamePopUp selectItemWithTitle:[selectedType name]];
    if ([entityNamePopUp selectedItem] == nil
        && [entityNamePopUp numberOfItems] > 0)
        [entityNamePopUp selectItemAtIndex:0];
    if ([entityNamePopUp selectedItem] != nil)
        [entityNameField setStringValue:[entityNamePopUp titleOfSelectedItem]];
}

- (void)calcEntityTypePopUp
{
    NSEnumerator *subenum;
    MiscTree *subtree;
    ReduceEntityType *type;

    type = [[entityNamePopUp selectedItem] representedObject];
    [entityTypePopUp removeAllItems];
//    subenum = [[inputComponent hierarchyTree] preorderAllSubtreeEnumerator];
    subenum = [[self hierarchyTree] preorderAllSubtreeEnumerator];
    while (subtree = [subenum nextObject]) {
        if ([subtree isLeaf]) {
            id entityType = [subtree entityType];
            if (entityType) {
                [entityTypePopUp addItemWithTitle:[entityType name]];
                [[entityTypePopUp itemWithTitle:[entityType name]] setRepresentedObject:entityType];
            }
        }
    }
    [entityTypePopUp selectItemWithTitle:[[type entityTypeToReduce] name]];
}

- (void)calcGroupPopUp
{
    MiscTree *subtree;
    ReduceEntityType *type;

    type = [[entityNamePopUp selectedItem] representedObject];
    [groupByPopUp removeAllItems];
    subtree = [[inputComponent hierarchyTree] subtreeWithEntityType:[type entityTypeToReduce]];
    while (subtree = [subtree parentTree]) {
        id entityType = [subtree entityType];
        [groupByPopUp insertItemWithTitle:[entityType name] atIndex:0];
        [[groupByPopUp itemAtIndex:0] setRepresentedObject:entityType];
    }
    [groupByPopUp selectItemWithTitle:[[type containerType] name]];
}

- (void)calcReduceModePopUp
{
    ReduceEntityType *type;
    Class entityClass;
    NSArray *reduceEntityClassNames = [NSArray arrayWithObjects:
        @"CountReduceEntity",
        @"SumReduceEntity",
        @"AverageReduceEntity",
        nil];
    NSEnumerator *classNameEnum;
    NSString *className;
    
    [reduceModePopUp removeAllItems];
    classNameEnum = [reduceEntityClassNames objectEnumerator];
    while (className = [classNameEnum nextObject]) {
        entityClass = NSClassFromString(className);
        if (!entityClass) {
            NSLog(@"%@: class named %@ not found!",
                  NSStringFromClass([self class]), className);
            continue;
        }
        [reduceModePopUp addItemWithTitle:[entityClass titleForPopUp]];
        [[reduceModePopUp lastItem] setRepresentedObject:entityClass];
    }
    
    type = [[entityNamePopUp selectedItem] representedObject];
    entityClass = [type entityClass];
    if (entityClass)
        [reduceModePopUp selectItemWithTitle:[entityClass titleForPopUp]];
}

- (void)calcHierarchyTree
{
    ReduceEntityType *entityType;
    NSEnumerator *entityTypeEnumerator;
    
    [hierarchyTree release];
    hierarchyTree = [[inputComponent hierarchyTree] copy];

    entityTypeEnumerator = [reduceEntityTypes objectEnumerator];
    while (entityType = [entityTypeEnumerator nextObject])
        [self addToHierarchy:entityType];

    [super hierarchyChanged];
}

- (void)addToHierarchy:(ReduceEntityType *)entityType
{
    MiscTree *subtree;

    subtree = [hierarchyTree subtreeWithEntityType:[entityType containerType]];

    if (subtree != nil) {
        MiscTree *newTree = [MiscTree tree];
        [newTree setEntityType:entityType];
        [subtree addSubtree:newTree];
    }
}

//
// interaction with interface
//
- (IBAction)createEntityType:(id)sender;
{
    ReduceEntityType *selectedEntityType;
    ReduceEntityType *newEntityType;
    NSString *newEntityName;
    int index;

    newEntityName = [entityNameField stringValue];
    index = [entityNamePopUp indexOfItemWithTitle:newEntityName];
    if (index != -1) {
        [entityNamePopUp selectItemAtIndex:index];
        [self entityNamePopUpChanged:self];
        return;
    }

    selectedEntityType = [[entityNamePopUp selectedItem] representedObject];
    if (selectedEntityType) {
        newEntityType = [ReduceEntityType typeWithName:newEntityName
                                         containerType:[selectedEntityType containerType]
                                             component:self];
        [newEntityType setEntityClass:[selectedEntityType entityClass]];
        [newEntityType setEntityTypeToReduce:[selectedEntityType entityTypeToReduce]];
        [newEntityType addNamesToFilter:[[selectedEntityType filterNames] allObjects]];        
    } else {
        newEntityType = [ReduceEntityType typeWithName:newEntityName
                                         containerType:(PajeContainerType *)[hierarchyTree entityType]
                                             component:self];
        [newEntityType setEntityClass:[CountReduceEntity class]];
        [newEntityType setEntityTypeToReduce:nil]; //FIXME
    }
    [reduceEntityTypes addObject:newEntityType];
    [entityTypesDictionary setObject:[newEntityType dictionaryForDefaults]
                              forKey:[newEntityType name]];
    [self registerDefaults];
    [self addToHierarchy:newEntityType];
    [super hierarchyChanged];

    [self calcEntityNamePopUp];
    [entityNamePopUp selectItemWithTitle:[newEntityType name]];
    [self entityNamePopUpChanged:self];
}

- (IBAction)deleteEntityType:(id)sender
{
    PajeEntityType *entityType;
    entityType = [[entityNamePopUp selectedItem] representedObject];
    if (entityType == nil) {
        NSBeep();
        return;
    }
    [entityTypesDictionary removeObjectForKey:[entityType name]];
    [self registerDefaults];
    [self hierarchyChanged];
}

- (IBAction)renameEntityType:(id)sender
{
    ReduceEntityType *entityType;
    NSString *oldName;
    NSString *newName;
    entityType = [[entityNamePopUp selectedItem] representedObject];
    oldName = [entityType name];
    newName = [entityNameField stringValue];
    if ((oldName == nil) || (newName == nil) || [newName isEqual:oldName]) {
        NSBeep();
        return;
    }
    [entityType setName:newName];
    [entityTypesDictionary removeObjectForKey:oldName];
    [entityTypesDictionary setObject:[entityType dictionaryForDefaults]
                              forKey:newName];
    [self registerDefaults];
    [[entityNamePopUp selectedItem] setTitle:newName];
    [entityNamePopUp synchronizeTitleAndSelectedItem];
    [super hierarchyChanged];
}

- (IBAction)entityNamePopUpChanged:(id)sender
{
    [entityNameField setStringValue:[entityNamePopUp titleOfSelectedItem]];
    [self calcEntityTypePopUp];
    [self calcGroupPopUp];
    [self refreshMatrix];
}

- (IBAction)entityTypePopUpChanged:(id)sender
{
    ReduceEntityType *type;
    PajeEntityType *newEntityTypeToReduce;
    
    type = [[entityNamePopUp selectedItem] representedObject];
    if (type == nil) {
        NSBeep();
        return;
    }
    newEntityTypeToReduce = [[entityTypePopUp selectedItem] representedObject];
    if (![[type entityTypeToReduce] isEqual:newEntityTypeToReduce]) {
        [type setEntityTypeToReduce:newEntityTypeToReduce];
        [entityTypesDictionary setObject:[type dictionaryForDefaults]
                                  forKey:[type name]];
        [self registerDefaults];
        [self calcGroupPopUp];
        [self refreshMatrix];
        [self calcHierarchyTree];
    }
}

- (IBAction)groupByPopUpChanged:(id)sender
{
    ReduceEntityType *type;
    PajeContainerType *newContainerType;

    type = [[entityNamePopUp selectedItem] representedObject];
    if (type == nil) {
        NSBeep();
        return;
    }
    newContainerType = [[groupByPopUp selectedItem] representedObject];
    if (![newContainerType isEqual:[type containerType]]) {
        [type setContainerType:newContainerType];
        [entityTypesDictionary setObject:[type dictionaryForDefaults]
                                  forKey:[type name]];
        [self registerDefaults];
        [self refreshMatrix];
        [self calcHierarchyTree];
    }
}

- (IBAction)reduceModePopUpChanged:(id)sender
{
    ReduceEntityType *type;
    Class entityClass;

    type = [[entityNamePopUp selectedItem] representedObject];
    if (type == nil) {
        NSBeep();
        return;
    }
    entityClass = [[reduceModePopUp selectedItem] representedObject];
    if (entityClass == nil) {
        NSBeep();
        return;
    }
    [type setEntityClass:entityClass];
    
    [entityTypesDictionary setObject:[type dictionaryForDefaults]
                              forKey:[type name]];
    [self registerDefaults];

    [super dataChangedForEntityType:type];
}

- (IBAction)matrixChanged:(id)sender
// some cell has been (de)selected in matrix.
{
    ReduceEntityType *type;
    NSButtonCell *cell;

    cell = [sender selectedCell];

    if (nil == cell)
        return;

    type = [[entityNamePopUp selectedItem] representedObject];
    if (type == nil) {
        NSBeep();
        return;
    }
    if ([cell state])
        [type removeNameFromFilter:[cell representedObject]];
    else
        [type addNameToFilter:[cell representedObject]];
    [entityTypesDictionary setObject:[type dictionaryForDefaults]
                              forKey:[type name]];
    [self registerDefaults];

    [super dataChangedForEntityType:type];
}


- (void)refreshMatrix
{
    unsigned i, n;
    NSButtonCell *cell;
    NSArray *allArray;
    ReduceEntityType *type;

    type = [[entityNamePopUp selectedItem] representedObject];
    allArray = [inputComponent allNamesForEntityType:[type entityTypeToReduce]];

    n = [allArray count];
    [nameMatrix renewRows:n columns:1];
    for (i = 0; i < n; i++) {
        NSString *name = [allArray objectAtIndex:i];
        cell = [nameMatrix cellAtRow:i column:0];
        [cell setTitle:name];
        [cell setRepresentedObject:name];
        [cell setState:![[type filterNames] containsObject:name]];
    }
    [nameMatrix sizeToFit];
    [nameMatrix setNeedsDisplay:YES];
}

- (MiscTree *)hierarchyTree
{
    return hierarchyTree;
}


//
// Trace messages that are filtered
//
- (NSEnumerator *)enumeratorOfEntityType:(PajeEntityType *)entityType
                             inContainer:(PajeContainer *)container
                                fromTime:(NSDate *)start
                                  toTime:(NSDate *)end
{
    if (![entityType isKindOfClass:[ReduceEntityType class]])
        return [inputComponent enumeratorOfEntityType:entityType
                                          inContainer:container
                                             fromTime:start
                                               toTime:end];

    return [(ReduceEntityType *)entityType
                        enumeratorOfEntitiesInContainer:container
                                               fromTime:start
                                                 toTime:end];
}


- (BOOL)canHighlightEntity:(id<PajeEntity>)obj
{
    if ([obj isKindOfClass:[ReduceEntity class]]) {
        return NO;
    } else {
        return [inputComponent canHighlightEntity:obj];
    }
}

- (PajeDrawingType)drawingTypeForEntityType:(PajeEntityType *)entityType
{
    if ([entityType isKindOfClass:[ReduceEntityType class]])
        return [entityType drawingType];
    else
        return [super drawingTypeForEntityType:entityType];
}

- (NSNumber *)valueForEntity:(id<PajeEntity>)entity
{
    if ([entity isKindOfClass:[ReduceEntity class]]) {
        return [(ReduceEntity *)entity value];
    } else {
        return [inputComponent valueForEntity:entity];
    }
}

- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType
{
    if ([entityType isKindOfClass:[ReduceEntityType class]])
        return [(ReduceEntityType *)entityType minValue];
    else
        return [super minValueForEntityType:entityType];
}

- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType
{
    if ([entityType isKindOfClass:[ReduceEntityType class]])
        return [(ReduceEntityType *)entityType maxValue];
    else
        return [super maxValueForEntityType:entityType];
}

- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType
                        inContainer:(PajeContainer *)container
{
    if ([entityType isKindOfClass:[ReduceEntityType class]])
        return [(ReduceEntityType *)entityType minValue];
    else
        return [super minValueForEntityType:entityType inContainer:container];
}

- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType
                        inContainer:(PajeContainer *)container
{
    if ([entityType isKindOfClass:[ReduceEntityType class]])
        return [(ReduceEntityType *)entityType maxValue];
    else
        return [super maxValueForEntityType:entityType inContainer:container];
}

@end
