#import "ReduceEntity.h"
#import "ReduceEntityType.h"
#import "../General.bproj/Macros.h"
#import "../General.bproj/NSObject+Additions.h"

@implementation ReduceEntity

+ (ReduceEntity *)entityWithArray:(BusyArray *)a index:(int)i
{
    return [[[self alloc] initWithArray:a index:i] autorelease];
}

- (id)initWithArray:(BusyArray *)a index:(int)i
{
    self = [super initWithType:[a entityType] name:nil container:[a container]];
    if (self) {
        Assign(array, a);
        index = i;
    }
    return self;
}

+ (PajeDrawingType)drawingType
{
    return PajeVariableDrawingType;
}

- (PajeDrawingType)drawingType
{
    return PajeVariableDrawingType;
}

- (PajeEntityType *)entityType
{
    return [array entityType];
}

- (NSDate *)startTime
{
    return [[array objectAtIndex:index] time];
}

- (NSDate *)endTime;
{
    return [[array objectAtIndex:index+1] time];
}

- (PajeContainer *)container
{
    return [array container];
}

- (NSString *)name
{
    return [[self value] stringValue];
}

- (NSColor *)color
{
    // FIXME: should color be calculated from value? If so, who keeps the color limits?
    return [NSColor blueColor];
}

- (NSArray *)relatedEntities
{
    return [[array objectAtIndex:index] allObjects];
}

+ (void)getMinValue:(NSNumber **)min
           maxValue:(NSNumber **)max
           forArray:(BusyArray *)a
      pajeComponent:(PajeFilter *)filter
{
    NSNumber *minValue = nil;
    NSNumber *maxValue = nil;
    NSNumber *v;
    int i;
    int c;

    c = [a count];
    for (i = 1; i < c; i++) {
        v = [self valueForRelatedEntities:[[a objectAtIndex:i] allObjects]
                            pajeComponent:filter];
        if ((minValue == nil) || ([minValue compare:v] == NSOrderedDescending))
            minValue = v;
        if ((maxValue == nil) || ([maxValue compare:v] == NSOrderedAscending))
            maxValue = v;
    }
    *min = minValue;
    *max = maxValue;
}

// to be implemented by subclasses
- (NSNumber *)value
{
    return [[self class] valueForRelatedEntities:[self relatedEntities]
                                   pajeComponent:[(ReduceEntityType *)[self entityType] component]];
    return nil;
}

@end

@implementation CountReduceEntity
+ (NSString *)titleForPopUp
{
    return @"Count";
}
+ (NSNumber *)valueForRelatedEntities:(NSArray *)entities
                        pajeComponent:(PajeFilter *)filter
{
    return [NSNumber numberWithInt:[entities count]];
}
@end

@implementation SumReduceEntity
+ (NSString *)titleForPopUp
{
    return @"Sum";
}
+ (NSNumber *)valueForRelatedEntities:(NSArray *)entities
                        pajeComponent:(PajeFilter *)filter
{
    double sum = 0.0;
    PajeEntity *entity;
    NSEnumerator *entityEnum;

    entityEnum = [entities objectEnumerator];
    while (entity = [entityEnum nextObject]) {
        sum += [[filter valueForEntity:entity] doubleValue];
    }
    return [NSNumber numberWithDouble:sum];
}
@end
@implementation AverageReduceEntity
+ (NSString *)titleForPopUp
{
    return @"Average";
}
+ (NSNumber *)valueForRelatedEntities:(NSArray *)entities
                        pajeComponent:(PajeFilter *)filter
{
    double sum = 0.0;
    int n=0;
    PajeEntity *entity;
    NSEnumerator *entityEnum;

    entityEnum = [entities objectEnumerator];
    while (entity = [entityEnum nextObject]) {
        sum += [[filter valueForEntity:entity] doubleValue];
        n++;
    }
    if (n != 0)
        return [NSNumber numberWithDouble:sum/n];
    else
        return [NSNumber numberWithDouble:sum];
}
@end
