#import "ReduceEntityType.h"
#import "ReduceEntity.h"
#import "../General.bproj/Macros.h"

@implementation ReduceEntityType

+ (ReduceEntityType *)typeFromDictionary:(NSDictionary *)dict
                               component:(PajeFilter *)comp
{
    ReduceEntityType *newType;
    NSString *n;
    PajeContainerType *cont;
    PajeEntityType *red;
    NSEnumerator *en;
    NSString *nameToFilter;
    Class reductionClass;

    cont = (PajeContainerType *)[[[comp hierarchyTree] subtreeWithEntityType:[dict objectForKey:@"ContainerType"]] entityType];
    if (cont == nil)
        return nil;

    red = [[[comp hierarchyTree] subtreeWithEntityType:[dict objectForKey:@"EntityTypeToReduce"]] entityType];
    if (red == nil)
        return nil;
    
    n = [dict objectForKey:@"EntityName"];
    if (n == nil)
        return nil;

    newType = [self typeWithName:n
                   containerType:cont
                       component:comp];
    if (newType == nil)
        return nil;

    [newType setEntityTypeToReduce:red];

    reductionClass = NSClassFromString([dict objectForKey:@"ReductionClass"]);
    if (reductionClass == Nil)
        reductionClass = [CountReduceEntity class];
    [newType setEntityClass:reductionClass];

    en = [[dict objectForKey:@"NamesToFilter"] objectEnumerator];
    while (nameToFilter = [en nextObject])
        [newType addNameToFilter:nameToFilter];
    return newType;
}

+ (ReduceEntityType *)typeWithName:(NSString *)n
                     containerType:(PajeContainerType *)cont
                         component:(PajeFilter *)comp
{
    return [[[self alloc] initWithName:n
                         containerType:cont
                             component:comp] autorelease];
}

- (id)initWithName:(NSString *)n
     containerType:(PajeContainerType *)cont
         component:(PajeFilter *)comp
{
    self = [super initWithName:n containerType:cont];
    if (self) {
        component = comp;
        filterNames = [[NSMutableSet alloc] init];
    }
    return self;
}

- (void)dealloc
{
    Assign(filterNames, nil);
    Assign(entityTypeToReduce, nil);
    Assign(array, nil);
    Assign(minValue, nil);
    Assign(maxValue, nil);
    [super dealloc];
}

- (PajeFilter *)component
{
    return component;
}

- (NSDictionary *)dictionaryForDefaults
{
    return [NSDictionary dictionaryWithObjectsAndKeys:
        [self name], @"EntityName",
        [[self containerType] name], @"ContainerType",
        [[self entityTypeToReduce] name], @"EntityTypeToReduce",
        [filterNames allObjects], @"NamesToFilter",
        NSStringFromClass(entityClass), @"ReductionClass",
        nil];
}

- (void)setName:(NSString *)n
{
    Assign(name, n);
}

- (void)setContainerType:(PajeContainerType *)newContainerType
{
    NSString *defaultName;

    Assign(containerType, newContainerType);

    defaultName = [[self name] stringByAppendingString:@"ContainerType"];
    [[NSUserDefaults standardUserDefaults] setObject:[containerType name]
                                              forKey:defaultName];

    Assign(minValue, nil);
    Assign(maxValue, nil);
    Assign(array, nil);
}

- (void)setEntityClass:(Class)c
{
    entityClass = c;
    Assign(minValue, nil);
    Assign(maxValue, nil);
    if (array != nil) {
        [c getMinValue:&minValue maxValue:&maxValue forArray:array pajeComponent:component];
        [minValue retain];
        [maxValue retain];
    }
}
- (Class)entityClass
{
    return entityClass;
}
- (PajeDrawingType)drawingType
{
    return PajeVariableDrawingType;
}

- (void)setEntityTypeToReduce:(PajeEntityType *)newEntityTypeToReduce
{
    Assign(entityTypeToReduce, newEntityTypeToReduce);

    [filterNames removeAllObjects];
    Assign(array, nil);
    Assign(minValue, nil);
    Assign(maxValue, nil);
}
- (PajeEntityType *)entityTypeToReduce
{
    return entityTypeToReduce;
}

- (void)addNameToFilter:(NSString *)n
{
    [filterNames addObject:n];
    Assign(array, nil);
    Assign(minValue, nil);
    Assign(maxValue, nil);
}
- (void)addNamesToFilter:(NSArray *)names
{
    [filterNames addObjectsFromArray:names];
    Assign(array, nil);
    Assign(minValue, nil);
    Assign(maxValue, nil);
}

- (void)removeNameFromFilter:(NSString *)n
{
    [filterNames removeObject:n];
    Assign(array, nil);
    Assign(minValue, nil);
    Assign(maxValue, nil);
}
- (NSSet *)filterNames
{
    return filterNames;
}

- (NSNumber *)minValue
{
    return minValue;
}

- (NSNumber *)maxValue
{
    return maxValue;
}


- (NSEnumerator *)enumeratorOfEntitiesInContainer:(PajeContainer *)container
                                         fromTime:(NSDate *)start
                                           toTime:(NSDate *)end
{
    NSEnumerator *origEnum;
    NSNumber *min;
    NSNumber *max;
    BOOL limitsChanged = NO;

    if (array != nil
        && [container isEqual:[array container]]
        && ![start earlierThanDate:[array startTime]]
        && ![end laterThanDate:[array endTime]]) {
        return [array objectEnumeratorOfClass:entityClass
                                     fromTime:start toTime:end];
    }

    if (array) [array release];
    origEnum = [component enumeratorOfEntityType:entityTypeToReduce
                                     inContainer:container
                                        fromTime:start
                                          toTime:end];
    array = [[BusyArray alloc] initWithEntityType:self
                                        container:container
                                        startTime:start
                                          endTime:end
                                       enumerator:origEnum
                                       nameFilter:filterNames];

    [entityClass getMinValue:&min maxValue:&max forArray:array pajeComponent:component];
    if ((minValue == nil) || ([minValue compare:min] == NSOrderedDescending)) {
        Assign(minValue, min);
        limitsChanged = YES;
    }
    if ((maxValue == nil) || ([maxValue compare:max] == NSOrderedAscending)) {
        Assign(maxValue, max);
        limitsChanged = YES;
    }
    if (limitsChanged)
        [component dataChangedForEntityType:self];

    return [array objectEnumeratorOfClass:entityClass];
}
@end