/*
I've taken this from http://jeremy.hksys.com/openstep/index.html

NSMatrix within a NSScrollView

by Jeremy Bettis <jeremy@hksys.com>
(Updated Feb 6, 1998)

_

When a NSMatrix (such as a NSForm) is positioned inside of a NSScrollView, pulling the scrollbar down and clicking on a text cell selects the wrong cell. Here is a fix.

*/


#import <AppKit/AppKit.h>

// ====================================================================
// A NSForm inside of a NSScrollView does not handle mouse clicks
// correctly. This subclass fixes that.  It scrolls the old selected
// cell to visible while activating the new one.
// ====================================================================

static int dontscroll = 0;

@interface HackedMatrix : NSMatrix
- (void) scrollCellToVisibleAtRow:(int)r column:(int)c;
- (void) textDidEndEditing:(NSNotification *)notify;
@end

@implementation HackedMatrix
+ (void) load
{
//    id pool = [NSAutoreleasePool new];
//    [self poseAsClass:[NSMatrix class]];
//    [pool release];
}

- (void) scrollCellToVisibleAtRow:(int)r column:(int)c
{
    if (dontscroll <= 0) {
        [super scrollCellToVisibleAtRow:r column:c];
    }
    dontscroll = 0;
}

- (void) textDidEndEditing:(NSNotification *)notify
{
    dontscroll=1;
    NS_DURING
        [super textDidEndEditing:notify];
    NS_HANDLER
        dontscroll = 0;
        [localException raise];
    NS_ENDHANDLER
    dontscroll = 0;
}

@end
