//

#import "HierarchyBrowser.h"
#import "Macros.h"

@implementation HierarchyBrowser

- (id)initWithFilter:(PajeFilter *)f
{
    self = [super init];
    if (self) {
        Assign(filter, f);
        if (![NSBundle loadNibNamed:@"HierarchyBrowser" owner:self])
            NSRunAlertPanel(@"HierarchyBrowser", @"Couldn't load interface file",
                            nil, nil, nil);
    }
    return self;
}

- (void)dealloc
{
    Assign(filter, nil);
    Assign(containerTypesBrowser, nil);
    Assign(containersBrowser, nil);
    Assign(splitView, nil);
    containersOnly = NO;
    [super dealloc];
}

- (void)awakeFromNib
{
    NSWindow *window;
    window = [splitView window];
    [splitView retain];
    [splitView removeFromSuperview];
#ifndef GNUSTEP
    [window release];
#endif
}

- (void)refresh
{
    [containersBrowser loadColumnZero];
    [containerTypesBrowser loadColumnZero];
    //if (selection)
    //    [self select:selection];
}

- (void)refreshLastColumn
{
    [containersBrowser reloadColumn:[containersBrowser selectedColumn]];
}

- (void)setFilter:(id)f
{
    Assign(filter, f);
    [self refresh];
}

- (void)setContainersOnly:(BOOL)flag
{
    containersOnly = flag;
}

- (NSView *)view
{
    return splitView;
}


- (PajeEntityType *)selectedEntityType
{
    return [[[containersBrowser selectedCell] representedObject] entityType];
}

- (NSArray *)selectedContainers
{
    NSMutableArray *array = [NSMutableArray array];
    NSEnumerator *cellEnum = [[containersBrowser selectedCells] objectEnumerator];
    NSCell *cell;
    while (cell = [cellEnum nextObject])
        [array addObject:[cell representedObject]];
    return array;
}

- (NSArray *)selectedNames
{
    NSMutableArray *array = [NSMutableArray array];
    NSEnumerator *cellEnum = [[containersBrowser selectedCells] objectEnumerator];
    NSCell *cell;
    while (cell = [cellEnum nextObject])
        [array addObject:[cell stringValue]];
    return array;
}

- (PajeContainer *)selectedParentContainer
{
    return [[[containersBrowser selectedCell] representedObject] container];
}



- (void)_syncBottom:(id)sender
    // Make visible columns in bottom browser same as top one
{
    int first = [containerTypesBrowser firstVisibleColumn];
    int last = [containerTypesBrowser lastVisibleColumn];
//    while ([containersBrowser lastColumn] < last)
//        [containersBrowser addColumn];
    if ([containersBrowser firstVisibleColumn] > first)
        [containersBrowser scrollColumnToVisible:first];
    if ([containersBrowser lastVisibleColumn] < last)
        [containersBrowser scrollColumnToVisible:last];
}

- (void)_syncTop:(id)sender
    // Make visible columns in top browser same as bottom one
{
    int first = [containersBrowser firstVisibleColumn];
    int last = [containersBrowser lastVisibleColumn];
    return;
    if ([containerTypesBrowser firstVisibleColumn] > first)
        [containerTypesBrowser scrollColumnToVisible:first];
    if ([containerTypesBrowser lastVisibleColumn] < last)
        [containerTypesBrowser scrollColumnToVisible:last];
}

// NSBrowser actions

- (IBAction)containerTypeSelected:(NSBrowser *)sender
{
    [containersBrowser reloadColumn:[sender selectedColumn]];
    [containersBrowser reloadColumn:[sender selectedColumn] + 1];
}

- (IBAction)containerSelected:(NSBrowser *)sender
{
}



// NSBrowser  delegate

- (void)browser:(NSBrowser *)sender createRowsForColumn:(int)column
       inMatrix:(NSMatrix *)matrix
{
//    NSLog(@"chamada a [%@ %@]", NSStringFromClass([self class]), NSStringFromSelector(_cmd));
    if (sender == containerTypesBrowser)
        [self containerTypesBrowser:sender createRowsForColumn:column
                           inMatrix:matrix];
    else if (sender == containersBrowser)
        [self containersBrowser:sender createRowsForColumn:column
                       inMatrix:matrix];
    else
        NSLog(@"Unknown browser in HierarchyBrowser");
}

- (void)containersBrowser:(NSBrowser *)sender createRowsForColumn:(int)column inMatrix:(NSMatrix *)matrix
{
    id upperContainer;
    NSEnumerator *containerEnum;
    id container;
    id containerType;
    int row;
    NSBrowserCell *cell;
    BOOL isContainer;

    if (column == 0) {
        upperContainer = [filter rootInstance];
    } else {
        upperContainer = [[sender selectedCellInColumn:column-1] representedObject];
    }

    cell = [containerTypesBrowser selectedCellInColumn:column];
    containerType = [cell representedObject];
    if (containerType == nil)
        return;
    isContainer = ![cell isLeaf];

    if (isContainer || containersOnly)
        containerEnum = [filter enumeratorOfEntityType:containerType
                                           inContainer:upperContainer];
    else
        containerEnum = [[filter allNamesForEntityType:containerType] objectEnumerator];
    for (row = 0; container = [containerEnum nextObject]; row++) {
        NSBrowserCell *cell;
        [matrix renewRows:row+1 columns:1];
        cell = [matrix cellAtRow:row column:0];
        [cell setRepresentedObject:container];
        [cell setStringValue:[container name]];
        [cell setLeaf:!isContainer];
    }
    if ([containerTypesBrowser lastColumn] < column) {
//        NSLog(@"antesB: %d, %d", [containerTypesBrowser lastColumn], column);
        [containerTypesBrowser addColumn];
//        NSLog(@"depoisB: %d, %d", [containerTypesBrowser lastColumn], column);
    }
}

- (void)containerTypesBrowser:(NSBrowser *)sender createRowsForColumn:(int)column inMatrix:(NSMatrix *)matrix
{
    int row;
    int col;
    MiscTree *tree;
    MiscTree *subtree;
    NSEnumerator *subtreeEnum;
    
    tree = [filter hierarchyTree];
    // must find the right subtree
    for (col=0; col<column; col++) {
        id containerType = [[sender selectedCellInColumn:col] representedObject];
        BOOL found = NO;
        subtreeEnum = [tree subtreeEnumerator];
        while (subtree = [subtreeEnum nextObject]) {
            if ([[subtree entityType] isEqual:containerType]) {
                tree = subtree;
                found = YES;
                break;
            }
        }
        if (!found) {
            NSLog(@"Hierarchy inconsistency!");
            return;
        }
    }

    subtreeEnum = [tree subtreeEnumerator];
    for (row = 0; subtree = [subtreeEnum nextObject]; row++) {
        NSBrowserCell *cell;
        id containerType = [subtree entityType];
        if (containersOnly && [subtree isLeaf]) // ![containerType isContainer]
            continue;
        [matrix renewRows:row+1 columns:1];
        cell = [matrix cellAtRow:row column:0];
        [cell setRepresentedObject:containerType];
        [cell setStringValue:[containerType name]];
        [cell setLeaf:containersOnly ? [subtree treeHeight]<=2 : [subtree isLeaf]];
    }
}

- (void)browserDidScroll:(NSBrowser *)sender
{
    // one browser scrolled. sync the other one.
    // do delayed performing because visibleColumns are not yet updated when this method is called (at least in OS4.2).
    if (sender == containerTypesBrowser)
        [self performSelector:@selector(_syncBottom:) withObject:self afterDelay:1.0];
    else
        [self performSelector:@selector(_syncTop:) withObject:self afterDelay:1.0];
}
@end
