// NSMatrix+Additions.h
//
// NSMatrix (NSDraggingDestination)
// Category to NSMatrix that allows it's delegate to receive
// dragging destination messages.
// To receive draggingEntered: messages, the delegate has to implement
// -matrix:(NSMatrix *)m draggingEntered:(id <NSDraggingInfo>)sender
// messages, and so on (see NSDraggingDestination protocol).
//
// NSMatrix (Additions)
// defines a method that returns the cell in a point, or nil

#import <AppKit/AppKit.h>

@interface NSMatrix (NSDraggingDestination)
- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender;
- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender;
- (void)draggingExited:(id <NSDraggingInfo>)sender;
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender;
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender;
- (void)concludeDragOperation:(id <NSDraggingInfo>)sender;
@end

@interface NSMatrix (Additions)
- (id)cellAtPoint:(NSPoint)p;
@end
