#import "NSMatrix+Additions.h"

@implementation NSMatrix (NSDraggingDestination)
- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
    id del = [self delegate];
    SEL sel = @selector(matrix:draggingEntered:);
    if (del && [del respondsToSelector:sel])
        return (unsigned int)[del performSelector:sel withObject:self withObject:sender];
    else
        return [super draggingEntered:sender];
}
- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
    id del = [self delegate];
    SEL sel = @selector(matrix:draggingUpdated:);
    if (del && [del respondsToSelector:sel])
        return (unsigned int)[del performSelector:sel withObject:self withObject:sender];
    else
        return [super draggingUpdated:sender];
}
- (void)draggingExited:(id <NSDraggingInfo>)sender
{
    id del = [self delegate];
    SEL sel = @selector(matrix:draggingExited:);
    if (del && [del respondsToSelector:sel])
        [del performSelector:sel withObject:self withObject:sender];
    else
        [super draggingExited:sender];
}
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    id del = [self delegate];
    SEL sel = @selector(matrix:prepareForDragOperation:);
    if (del && [del respondsToSelector:sel])
        return (BOOL)(int)[del performSelector:sel withObject:self withObject:sender];
    else
        return [super prepareForDragOperation:sender];
}
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    id del = [self delegate];
    SEL sel = @selector(matrix:performDragOperation:);
    if (del && [del respondsToSelector:sel])
        return (BOOL)(int)[del performSelector:sel withObject:self withObject:sender];
    else
        return [super performDragOperation:sender];
}
- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
    id del = [self delegate];
    SEL sel = @selector(matrix:concludeDragOperation:);
    if (del && [del respondsToSelector:sel])
        [del performSelector:sel withObject:self withObject:sender];
    else
        [super concludeDragOperation:sender];
}
@end

@implementation NSMatrix (Additions)
- (id)cellAtPoint:(NSPoint)p
{
    int row, column;

    if ([self getRow:&row column:&column forPoint:p])
        return [self cellAtRow:row column:column];
    return nil;
}
@end
