#import "NSUserDefaults+Colors.h"

#ifndef GNUSTEP
@interface NSColor (Additions)
+ (NSColor *)colorFromString:(NSString *)value;
@end
@implementation NSColor (Additions)
+ (NSColor *)colorFromString:(NSString *)value
{
    if ([value isKindOfClass:[NSString class]]) {
        NSScanner *scanner = [NSScanner scannerWithString:value];
        NSString *colorSpaceName;
        NSString *s1, *s2;
        float a, b, c, d, e;

        if ([scanner scanUpToString:@" " intoString:&colorSpaceName]) {
            if ([colorSpaceName isEqual:NSDeviceCMYKColorSpace]) {
                if (   [scanner scanFloat:&a] && [scanner scanFloat:&b]
                       && [scanner scanFloat:&c] && [scanner scanFloat:&d]
                       && [scanner scanFloat:&e])
                    return [NSColor colorWithDeviceCyan:a magenta:b yellow:c black:d alpha:e];
            } else if ([colorSpaceName isEqual:NSDeviceWhiteColorSpace]) {
                if ([scanner scanFloat:&a] && [scanner scanFloat:&b])
                    return [NSColor colorWithDeviceWhite:a alpha:b];
            } else if ([colorSpaceName isEqual:NSCalibratedWhiteColorSpace]) {
                if ([scanner scanFloat:&a] && [scanner scanFloat:&b])
                    return [NSColor colorWithCalibratedWhite:a alpha:b];
            } else if ([colorSpaceName isEqual:NSDeviceRGBColorSpace]) {
                if (   [scanner scanFloat:&a] && [scanner scanFloat:&b]
                       && [scanner scanFloat:&c] && [scanner scanFloat:&d])
                    return [NSColor colorWithDeviceRed:a green:b blue:c alpha:d];
            } else if ([colorSpaceName isEqual:NSCalibratedRGBColorSpace]) {
                if (   [scanner scanFloat:&a] && [scanner scanFloat:&b]
                       && [scanner scanFloat:&c] && [scanner scanFloat:&d])
                    return [NSColor colorWithCalibratedRed:a green:b blue:c alpha:d];
            } else if ([colorSpaceName isEqual:NSNamedColorSpace]) {
                if (   [scanner scanUpToString:@" " intoString:&s1]
                       && [scanner scanUpToString:@" " intoString:&s2])
                    return [NSColor colorWithCatalogName:s1 colorName:s2];
            }
        }
    }
    return nil;
}

@end
#endif

@implementation NSUserDefaults (Colors)
- (void)setColor:(NSColor *)color forKey:(NSString *)key
{
    [self setObject:[color description] forKey:key];
#if 0
    id value;
#ifdef COLORWORKS
    NSString *colorSpaceName = [color colorSpaceName];
    float a, b, c, d, e;

    if ([colorSpaceName isEqual:NSDeviceCMYKColorSpace]) {
        [color getCyan:&a magenta:&b yellow:&c black:&d alpha:&e];
        value = [NSString stringWithFormat:@"%@ %f %f %f %f %f",
            colorSpaceName, a, b, c, d, e];
    } else if ([colorSpaceName isEqual:NSDeviceWhiteColorSpace]
               ||[colorSpaceName isEqual:NSCalibratedWhiteColorSpace]) {
        [color getWhite:&a alpha:&b];
        value = [NSString stringWithFormat:@"%@ %f %f",
            colorSpaceName, a, b];
    } else if ([colorSpaceName isEqual:NSDeviceRGBColorSpace]
               ||[colorSpaceName isEqual:NSCalibratedRGBColorSpace]) {
        [color getRed:&a green:&b blue:&c alpha:&d];
        value = [NSString stringWithFormat:@"%@ %f %f %f %f",
            colorSpaceName, a, b, c, d];
    } else if ([colorSpaceName isEqual:NSNamedColorSpace]) {
        value = [NSString stringWithFormat:@"%@ %@ %@", colorSpaceName, [color catalogNameComponent], [color colorNameComponent]];
    } else
#endif
    {
        value = [NSArchiver archivedDataWithRootObject:color];
    }
    
    [self setObject:value forKey:key];
#endif
}

- (NSColor *)colorForKey:(NSString *)key
{
    return [NSColor colorFromString:[self objectForKey:key]];
#if 0
    id value = [self objectForKey:key];

    if (nil == value)
        return nil;

    if ([value isKindOfClass:[NSData class]])
        return [NSUnarchiver unarchiveObjectWithData:value];
#ifdef COLORWORKS
    if ([value isKindOfClass:[NSString class]]) {
        NSScanner *scanner = [NSScanner scannerWithString:value];
        NSString *colorSpaceName;
        NSString *s1, *s2;
        float a, b, c, d, e;

        if ([scanner scanUpToString:@" " intoString:&colorSpaceName]) {
            if ([colorSpaceName isEqual:NSDeviceCMYKColorSpace]) {
                if (   [scanner scanFloat:&a] && [scanner scanFloat:&b]
                    && [scanner scanFloat:&c] && [scanner scanFloat:&d]
                    && [scanner scanFloat:&e])
                    return [NSColor colorWithDeviceCyan:a magenta:b yellow:c black:d alpha:e];
            } else if ([colorSpaceName isEqual:NSDeviceWhiteColorSpace]) {
                if ([scanner scanFloat:&a] && [scanner scanFloat:&b])
                    return [NSColor colorWithDeviceWhite:a alpha:e];
            } else if ([colorSpaceName isEqual:NSCalibratedWhiteColorSpace]) {
                if ([scanner scanFloat:&a] && [scanner scanFloat:&b])
                    return [NSColor colorWithCalibratedWhite:a alpha:e];
            } else if ([colorSpaceName isEqual:NSDeviceRGBColorSpace]) {
                if (   [scanner scanFloat:&a] && [scanner scanFloat:&b]
                    && [scanner scanFloat:&c] && [scanner scanFloat:&d])
                    return [NSColor colorWithDeviceRed:a green:b blue:c alpha:e];
            } else if ([colorSpaceName isEqual:NSCalibratedRGBColorSpace]) {
                if (   [scanner scanFloat:&a] && [scanner scanFloat:&b]
                    && [scanner scanFloat:&c] && [scanner scanFloat:&d])
                    return [NSColor colorWithCalibratedRed:a green:b blue:c alpha:e];
            } else if ([colorSpaceName isEqual:NSNamedColorSpace]) {
                if (   [scanner scanUpToString:@" " intoString:&s1]
                    && [scanner scanUpToString:@" " intoString:&s2])
                    return [NSColor colorWithCatalogName:s1 colorName:s2];
            }
        }
    }
#endif
    return nil;
#endif GNUSTEP
}

- (void)setColorDictionary:(NSDictionary *)dict forKey:(NSString *)key
{
    NSMutableDictionary *newdict = [NSMutableDictionary dictionary];
    NSEnumerator *en = [dict keyEnumerator];
    id colorKey;
    while (colorKey = [en nextObject]) {
        NSColor *c = [dict objectForKey:colorKey];
        if ([c isKindOfClass:[NSColor class]])
            [newdict setObject:[c description] forKey:colorKey];
    }
    [self setObject:newdict forKey:key];
}

- (NSDictionary *)colorDictionaryForKey:(NSString *)key
{
    NSDictionary *dict = [self dictionaryForKey:key];
    if (dict) {
        NSEnumerator *en=[dict keyEnumerator];
        id key;
        NSColor *c;
        NSMutableDictionary *newdict = [NSMutableDictionary dictionary];
        while (key = [en nextObject]) {
            c = [NSColor colorFromString:[dict objectForKey:key]];
            if (c)
                [newdict setObject:c forKey:key];
        }
        dict = newdict;
    }
    return dict;
}

- (void)setRect:(NSRect)rect forKey:(NSString *)key
{
    [self setObject:NSStringFromRect(rect) forKey:key];
}

- (NSRect)rectForKey:(NSString *)key
{
    NSString *s = [self stringForKey:key];
    if (s != nil)
        return NSRectFromString(s);
    else
        return NSZeroRect;
}
@end

@implementation NSUserDefaults (ArchivedObjects)
- (void)setArchivedObject:(id)anObject forKey:(NSString *)aKey
{
    NSData *archivedObject;

    archivedObject = [NSArchiver archivedDataWithRootObject:anObject];
    [self setObject:archivedObject forKey:aKey];
}

- (id)unarchiveObjectForKey:(NSString *)aKey
{
    NSData *archivedObject;

    archivedObject = [self objectForKey:aKey];
    if ([archivedObject isKindOfClass:[NSData class]])
        return [NSUnarchiver unarchiveObjectWithData:archivedObject];
    else
        return archivedObject;
} 
@end 
