//
// PajeEntity
//
// Generic entities for Paje
//

#import "PajeEntity.h"
#import "PajeContainer.h"
#import "PajeEntityInspector.h"
#import "Macros.h"

@implementation PajeEntity
- (id)initWithType:(PajeEntityType *)type
              name:(id)n
         container:(PajeContainer *)c
{
    self = [super init];
    if (self) {
        Assign(entityType, type);
        Assign(name, n);
        Assign(container, c);
    }
    return self;
}

- (void)dealloc
{
    Assign(entityType, nil);
    Assign(name, nil);
    Assign(container, nil);
    [super dealloc];
}

- (BOOL)isContainer
{
    return NO;
}

- (PajeDrawingType)drawingType
{
    return [entityType drawingType];
}

- (NSString *)name
{
    return name;
}

- (PajeEntityType *)entityType
{
    return entityType;
}

- (PajeContainer *)container
{
    return container;
}

- (BOOL)isContainedBy:(PajeContainer *)cont
{
    // FIXME: [root container] should be nil, not root!
    if ((container == nil)/* || [container isEqual:self]*/)
        return NO;

    if ([cont isEqual:container])
        return YES;

    if ([[cont entityType] isEqual:[container entityType]])
        return NO;

    return [container isContainedBy:cont];
}

- (NSString *)description
{
    return [name description];
}

- (NSDate *)time
{
    [self _subclassResponsibility:_cmd];
    return nil;
}
- (NSDate *)startTime
{
    return [self time];
}
- (NSDate *)endTime
{
    return [self time];
}
- (NSDate *)firstTime
{
    return [[self startTime] earlierDate:[self endTime]];
}
- (NSDate *)lastTime
{
    return [[self startTime] laterDate:[self endTime]];
}


- (NSColor *)color
{
    NSString *n = [self name];
    NSColor *c = [entityType colorForName:n];
    if (!c) {
        c = [NSColor yellowColor];
        [entityType setColor:c forName:n];
    }
    return c;
}

- (void)setColor:(NSColor *)c
{
    [entityType setColor:c forName:[self name]];
}

- (void)takeColorFrom:(id)sender
{
    [self setColor:[sender color]];
}

- (NSArray *)relatedEntities
{
    return [NSArray array];
}

- (int)imbricationLevel
{
    return 0;
}


- (void)inspect
{
    Class inspectorClass = nil;
    Class class = [self class];

    while (!inspectorClass && !(class == [class superclass])) {
        NSString *inspName;
        inspName = [NSStringFromClass(class) stringByAppendingString:@"Inspector"];
        inspectorClass = NSClassFromString(inspName);
        class = [class superclass];
    }
    if (inspectorClass)
        //        [[[inspectorClass alloc] init] inspect:self];
        [[inspectorClass inspector] inspect:self];
}

- (NSArray *)fieldNames
{
    return [NSMutableArray arrayWithObjects:
        @"EntityType", @"Name", @"Container",
        nil];
}

- (id)valueOfFieldNamed:(NSString *)fieldName
{
    if ([fieldName isEqualToString:@"EntityType"])
        return [self entityType];
    else if ([fieldName isEqualToString:@"Name"])
        return [self name];
    else if ([fieldName isEqualToString:@"Container"])
        return [self container];
    else
        return nil; 
}

- (IBAction)inspect:(id)sender
{
    [self inspect];
}

- (NSComparisonResult)compare:(id)other
{
    return [[self name] compare:[other name]];
}

- (unsigned)hash
{
    return [name hash];
}

- (BOOL)isEqual:(id)other
{
    if (other == self) return YES;
    if (![other isKindOfClass:[PajeEntity class]]) return NO;
    return [entityType isEqual:[(PajeEntity *)other entityType]]
        && [name isEqual:[(PajeEntity *)other name]]
        && [container isEqual:[(PajeEntity *)other container]];
}
@end
