//#define COMPATIBILITY
#import "PajeFilter.h"



@implementation PajeComponent

- (id)init
{
    self = [super init];
    outputSelector = @selector(inputEntity:);
    return self;
}

- (void)setInputComponent:(id)component
{
    inputComponent = component;
}

- (void)setOutputComponent:(id)component
{
    if (outputComponent) { // HACK
        if ([outputComponent isKindOfClass: [NSMutableArray class]])
            [(NSMutableArray *)outputComponent addObject:component];
        else
            outputComponent = [[NSMutableArray arrayWithObjects:outputComponent, component, nil] retain];
    } else
        outputComponent = component;
}

- (void)inputEntity:(id)entity
{
    NSAssert(0, @"should be implemented in subclass!");
}

- (void)outputEntity:(id)entity
{
    if ([outputComponent isKindOfClass:[NSArray class]]) // HACK
        [(NSArray*)outputComponent makeObjectsPerformSelector:outputSelector withObject:entity];
    else
        [outputComponent performSelector:outputSelector withObject:entity];
}

@end



@implementation PajeFilter

- (NSDate *)startTime
{
    return [inputComponent startTime];
}

- (NSDate *)endTime
{
    return [inputComponent endTime];
}
- (id)rootInstance
{
    return [inputComponent rootInstance];
}
//#endif


//
// Deal with notifications
//
- (void)dataChangedForEntityType:(PajeEntityType *)entityType
{
    if ([outputComponent isKindOfClass:[PajeFilter class] /*respondsToSelector:@selector(dataChangedForEntityType:)*/])
        [outputComponent dataChangedForEntityType:entityType];
    else
        [self outputEntity:self];
}

- (void)colorChangedForEntityType:(PajeEntityType *)entityType
{
    if ([outputComponent isKindOfClass:[PajeFilter class] /*respondsToSelector:@selector(colorChangedForEntityType:)*/])
        [outputComponent colorChangedForEntityType:entityType];
    else
        [self outputEntity:self];
}

- (void)hierarchyChanged
{
    if ([outputComponent isKindOfClass:[PajeFilter class] /*respondsToSelector:@selector(hierarchyChanged)*/])
        [outputComponent hierarchyChanged];
    else if ([outputComponent isKindOfClass:[NSArray class]]) // HACK
        [(NSArray*)outputComponent makeObjectsPerformSelector:_cmd];
    else
        [self outputEntity:self];
}

- (void)filterContainer:(PajeContainer *)container show:(BOOL)show
{
    [outputComponent filterContainer:container show:show];
}


//
// Deal with queries
//

//
// Accessing entities
//
- (MiscTree *)hierarchyTree
{
    return [inputComponent hierarchyTree];
}

- (NSEnumerator *)enumeratorOfEntityType:(PajeEntityType *)entityType
                             inContainer:(PajeContainer *)container
                                fromTime:(NSDate *)start
                                  toTime:(NSDate *)end
{
    return [inputComponent enumeratorOfEntityType:entityType
                                      inContainer:container
                                         fromTime:start
                                           toTime:end];
}

- (NSEnumerator *)enumeratorOfEntityType:(PajeEntityType *)entityType
                             inContainer:(PajeContainer *)container
{
    return [inputComponent enumeratorOfEntityType:entityType
                                      inContainer:container];
}

- (NSArray *)allNamesForEntityType:(PajeEntityType *)entityType
{
    return [inputComponent allNamesForEntityType:entityType];
}

- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType
{
    return [inputComponent minValueForEntityType:entityType];
}

- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType
{
    return [inputComponent maxValueForEntityType:entityType];
}

- (NSNumber *)minValueForEntityType:(PajeEntityType *)entityType
                        inContainer:(PajeContainer *)container
{
    return [inputComponent minValueForEntityType:entityType
                                     inContainer:container];
}

- (NSNumber *)maxValueForEntityType:(PajeEntityType *)entityType
                        inContainer:(PajeContainer *)container
{
    return [inputComponent maxValueForEntityType:entityType
                                     inContainer:container];
}



- (PajeDrawingType)drawingTypeForEntityType:(PajeEntityType *)entityType
{
    return [inputComponent drawingTypeForEntityType:entityType];
}

- (NSArray *)fieldNamesForEntityType:(PajeEntityType *)entityType
{
    return [inputComponent fieldNamesForEntityType:entityType];
}

- (NSArray *)fieldNamesForEntityType:(PajeEntityType *)entityType
                                name:(NSString *)name
{
    return [inputComponent fieldNamesForEntityType:entityType name:name];
}

- (NSColor *)colorForName:(NSString *)name ofEntityType:(PajeEntityType *)entityType
{
    return [inputComponent colorForName:name ofEntityType:entityType];
}

- (void)setColor:(NSColor *)color
         forName:(NSString *)name
    ofEntityType:(PajeEntityType *)entityType
{
    [inputComponent setColor:color
                     forName:name
                ofEntityType:entityType];
}

- (NSColor *)colorForEntityType:(PajeEntityType *)entityType
{
    return [inputComponent colorForEntityType:entityType];
}

- (void)setColor:(NSColor *)color
   forEntityType:(PajeEntityType *)entityType
{
    [inputComponent setColor:color
               forEntityType:entityType];
}

- (void)setColor:(NSColor *)color forEntity:(id<PajeEntity>)entity;
{
    [inputComponent setColor:color
                   forEntity:entity];
}

//
// Getting info from entity
//
- (NSArray *)fieldNamesForEntity:(id<PajeEntity>)entity
{
    return [inputComponent fieldNamesForEntity:entity];
}

- (id)valueOfFieldNamed:(NSString *)fieldName forEntity:(id<PajeEntity>)entity
{
    return [inputComponent valueOfFieldNamed:fieldName forEntity:entity];
}


- (PajeContainer *)containerForEntity:(id<PajeEntity>)entity
{
    return [inputComponent containerForEntity:entity];
}

- (PajeEntityType *)entityTypeForEntity:(id<PajeEntity>)entity
{
    return [inputComponent entityTypeForEntity:entity];
}

- (PajeContainer *)sourceContainerForEntity:(id<PajeLink>)entity
{
    return [inputComponent sourceContainerForEntity:entity];
}

- (PajeEntityType *)sourceEntityTypeForEntity:(id<PajeLink>)entity
{
    return [inputComponent sourceEntityTypeForEntity:entity];
}

- (PajeContainer *)destContainerForEntity:(id<PajeLink>)entity
{
    return [inputComponent destContainerForEntity:entity];
}

- (PajeEntityType *)destEntityTypeForEntity:(id<PajeLink>)entity
{
    return [inputComponent destEntityTypeForEntity:entity];
}

- (NSArray *)relatedEntitiesForEntity:(id<PajeEntity>)entity
{
    return [inputComponent relatedEntitiesForEntity:entity];
}

- (NSColor *)colorForEntity:(id<PajeEntity>)entity
{
    return [inputComponent colorForEntity:entity];
}

- (NSDate *)startTimeForEntity:(id<PajeEntity>)entity
{
    return [inputComponent startTimeForEntity:entity];
}

- (NSDate *)endTimeForEntity:(id<PajeEntity>)entity
{
    return [inputComponent endTimeForEntity:entity];
}

- (NSDate *)timeForEntity:(id<PajeEntity>)entity
{
    return [inputComponent timeForEntity:entity];
}

- (PajeDrawingType)drawingTypeForEntity:(id<PajeEntity>)entity
{
    return [inputComponent drawingTypeForEntity:entity];
}

- (NSString *)nameForEntity:(id<PajeEntity>)entity
{
    return [inputComponent nameForEntity:entity];
}

- (NSNumber *)valueForEntity:(id<PajeEntity>)entity // for variables
{
    return [inputComponent valueForEntity:entity];
}

- (NSString *)descriptionForEntity:(id<PajeEntity>)entity
{
    return [inputComponent descriptionForEntity:entity];
}

- (int)imbricationLevelForEntity:(id<PajeEntity>)entity
{
    return [inputComponent imbricationLevelForEntity:entity];
}

- (BOOL)canHighlightEntity:(id<PajeEntity>)entity
{
    return [inputComponent canHighlightEntity:entity];
}



- (void)verifyStartTime:(NSDate *)start endTime:(NSDate *)end
{
    [inputComponent verifyStartTime:start endTime:end];
}



//
// Inspecting an entity
//
- (void)inspectEntity:(id<PajeInspecting>)entity
{
    [inputComponent inspectEntity:entity];
}


@end
