//
// PajeType
//
// represents the type of (user created) entities and containers
//

#import <Foundation/Foundation.h>
#import "Protocols.h"

@class PajeContainerType;

@interface PajeEntityType : NSObject
{
    NSString *name;
    PajeContainerType *containerType;
    NSColor *color;
}

+ (PajeEntityType *)typeWithName:(NSString *)n
                   containerType:(PajeContainerType *)type;
- (id)initWithName:(NSString *)n
     containerType:(PajeContainerType *)type;

- (BOOL)isContainer;

- (NSString *)name;
- (PajeContainerType *)containerType;

- (PajeDrawingType)drawingType;

- (NSColor *)colorForName:(id)n;
- (void)setColor:(NSColor*)color forName:(id)n;
- (NSColor *)color;
- (void)setColor:(NSColor*)color;
@end


@interface PajeContainerType : PajeEntityType
+ (PajeContainerType *)typeWithName:(NSString *)n
                      containerType:(PajeContainerType *)type;
@end


@interface PajeCategorizedEntityType : PajeEntityType
{
    NSMutableDictionary *valueToName;
    NSMutableDictionary *nameToColor;
}

- (void)setName:(id)n
       forValue:(id)v;
- (id)nameForValue:(id)v;
- (NSArray *)allNames;

- (NSColor *)colorForName:(id)n;
- (void)setColor:(NSColor*)color forName:(id)n;
- (void)readDefaultColors;
@end



@interface PajeEventType : PajeCategorizedEntityType
- (PajeDrawingType)drawingType;
@end


@interface PajeStateType : PajeCategorizedEntityType
- (PajeDrawingType)drawingType;
@end


@interface PajeVariableType : PajeEntityType
{
    NSNumber *minValue;
    NSNumber *maxValue;
}
- (PajeDrawingType)drawingType;
- (void)possibleNewMinValue:(NSNumber *)value;
- (void)possibleNewMaxValue:(NSNumber *)value;
- (NSNumber *)minValue;
- (NSNumber *)maxValue;
@end


@interface PajeLinkType : PajeCategorizedEntityType
{
    PajeContainerType *sourceContainerType;
    PajeContainerType *destContainerType;
}

+ (PajeLinkType *)typeWithName:(id)n
                 containerType:(PajeContainerType *)type
           sourceContainerType:(PajeContainerType *)sourceType
             destContainerType:(PajeContainerType *)destType;
-    (id)initWithName:(id)n
        containerType:(PajeContainerType *)type
  sourceContainerType:(PajeContainerType *)sourceType
    destContainerType:(PajeContainerType *)destType;
- (PajeContainerType *)sourceContainerType;
- (PajeContainerType *)destContainerType;
- (PajeDrawingType)drawingType;
@end
