//
// PajeType
//
// represents the type of (user created) entities and containers
//

#import <Foundation/Foundation.h>
#import "PajeType.h"
#import "Macros.h"
#import "NSUserDefaults+Colors.h"

@implementation PajeEntityType

+ (PajeEntityType *)typeWithName:(NSString *)n
                   containerType:(PajeContainerType *)type
{
    return [[[self alloc] initWithName:n
                         containerType:type] autorelease];
}

- (id)initWithName:(NSString *)n
     containerType:(PajeContainerType *)type
{
    if (self == [super init]) {
        NSColor *c;
        Assign(name, n);
        Assign(containerType, type);
        c = [[NSUserDefaults standardUserDefaults] colorForKey:[name stringByAppendingString:@" Color"]];
        if (c)
            Assign(color, c);
        else
            Assign(color, [NSColor blackColor]);
    }
    return self;
}

- (void)dealloc
{
    Assign(name, nil);
    Assign(containerType, nil);
    Assign(color, nil);
    [super dealloc];
}

- (BOOL)isContainer
{
    return NO;
}

- (NSArray *)allNames
{
    return [NSArray array];
}

- (NSString *)name
{
    return name;
}

- (PajeEntityType *)containerType
{
    if (containerType == nil) return self;    //HACK
    return containerType;
}

- (PajeDrawingType)drawingType
{
    [self _subclassResponsibility:_cmd];
    return PajeNonDrawingType;
}


- (id)copyWithZone:(NSZone *)zone
{
    return [self retain];
}

- (NSString *)description
{
    return [name description];
}

- (NSColor *)colorForName:(id)n
{
    return [self color];
}
- (void)setColor:(NSColor*)c forName:(id)n
{
    [self setColor:c];
}
- (NSColor *)color
{
    return color;
}
- (void)setColor:(NSColor*)c
{
    Assign(color, c);
    [[NSUserDefaults standardUserDefaults]
            setColor:color forKey:[name stringByAppendingString:@" Color"]];
}

- (unsigned)hash
{
    return [name hash];
}

- (BOOL)isEqual:(id)other
{
    if (other == self || other == name) return YES;
    return [other isEqual:name];
}
@end


@implementation PajeContainerType
+ (PajeContainerType *)typeWithName:(NSString *)n
                      containerType:(PajeContainerType *)type
{
    return [[[self alloc] initWithName:n
                         containerType:type] autorelease];
}

- (BOOL)isContainer
{
    return YES;
}

- (PajeDrawingType)drawingType
{
    return PajeContainerDrawingType;
}
@end





@implementation PajeCategorizedEntityType
- (id)initWithName:(NSString *)n
     containerType:(PajeContainerType *)type
{
    self = [super initWithName:n containerType:type];
    Assign(valueToName, [NSMutableDictionary dictionary]);
    [self readDefaultColors];
    return self;
}

- (void)dealloc
{
    Assign(valueToName, nil);
    Assign(nameToColor, nil);
    [super dealloc];
}

- (void)setName:(id)n
       forValue:(id)v
{
    [valueToName setObject:n forKey:v];
}

- (id)nameForValue:(id)v
{
    return [valueToName objectForKey:v];
}

- (NSArray *)allNames
{
    return [nameToColor allKeys];
}

- (NSColor *)colorForName:(id)n
{
    return [nameToColor objectForKey:n];
}

- (void)setColor:(NSColor*)c forName:(id)n
{
    [nameToColor setObject:c forKey:n];
    [[NSUserDefaults standardUserDefaults]
        setColorDictionary:nameToColor
                    forKey:[name stringByAppendingString:@" Colors"]];
}

- (void)readDefaultColors
{
    NSMutableDictionary *dict;
    dict = [[[[NSUserDefaults standardUserDefaults]
        colorDictionaryForKey:[name stringByAppendingString:@" Colors"]] mutableCopy] autorelease];
    if (!dict)
        dict = [NSMutableDictionary dictionary];
    Assign(nameToColor, dict);
}

@end



@implementation PajeEventType
- (PajeDrawingType)drawingType
{
    return PajeEventDrawingType;
}
@end


@implementation PajeStateType
- (PajeDrawingType)drawingType
{
    return PajeStateDrawingType;
}
@end


@implementation PajeVariableType
+ (NSArray *)allNames
{
    return [NSArray array];
}

- (void)dealloc
{
    Assign(minValue, nil);
    Assign(maxValue, nil);
    [super dealloc];
}

- (PajeDrawingType)drawingType
{
    return PajeVariableDrawingType;
}

- (void)possibleNewMinValue:(NSNumber *)value
{
    if ((minValue == nil) || ([minValue compare:value] == NSOrderedDescending)) {
        Assign(minValue, value);
    }
}

- (void)possibleNewMaxValue:(NSNumber *)value
{

    if ((maxValue == nil) || ([maxValue compare:value] == NSOrderedAscending)) {
        Assign(maxValue, value);
    }
}

- (NSNumber *)minValue
{
    return minValue;
}

- (NSNumber *)maxValue
{
    return maxValue;
}

@end


@implementation PajeLinkType

+ (PajeLinkType *)typeWithName:(id)n
                 containerType:(PajeContainerType *)type
           sourceContainerType:(PajeContainerType *)sourceType
             destContainerType:(PajeContainerType *)destType
{
    return [[[self alloc] initWithName:n
                         containerType:type
                   sourceContainerType:sourceType
                     destContainerType:destType] autorelease];
}

-    (id)initWithName:(id)n
        containerType:(PajeContainerType *)type
  sourceContainerType:(PajeContainerType *)sourceType
    destContainerType:(PajeContainerType *)destType
{
    self = [super initWithName:n containerType:type];
    Assign(sourceContainerType, sourceType);
    Assign(destContainerType, destType);
    return self;
}

- (PajeContainerType *)sourceContainerType
{
    return sourceContainerType;
}

- (PajeContainerType *)destContainerType
{
    return destContainerType;
}

- (PajeDrawingType)drawingType
{
    return PajeLinkDrawingType;
}
@end
