// 

#import <AppKit/AppKit.h>    // NXColor
#import <Foundation/Foundation.h>
#import "MiscTree.h"

#import "Protocols.h"
#import "TimeList.h"
#import "Comparing.h"

@interface Trace: NSObject<PajeTiming>
{
    NSDate              *startTime;
    NSDate              *endTime;

    NSMutableDictionary *entityLists;   // dictionary (by entity type) of
                                        // dictionaries (by container) of 
                                        // TimeList's of entities

    NSDate              *startTimeInMemory;
    NSDate              *endTimeInMemory;

    NSMutableDictionary *entityTypeToInstances;
    NSMutableDictionary *entityTypeToClass;

    MiscTree            *hierarchyTree;
    id                   rootInstance;
}


//
// initializing trace
//
- (id)initWithStartTime:(NSDate *)t1 endTime:(NSDate *)t2;

//
// Adding elements to the trace
//
- (void)addEntity:(id<PajeEntity>)entity;

- (void)entityChangedEndTime:(id<PajeEntity>)entity;

//
// Accessing objects in trace
//
- (MiscTree *)hierarchyTree;

- (NSEnumerator *)enumeratorOfEntityType:(id)entityType
                             inContainer:(id)container
                                fromTime:(NSDate *)start
                                  toTime:(NSDate *)end;

- (NSEnumerator *)enumeratorOfEntityType:(id)entityType
                             inContainer:(id)container;

- (NSArray *)allNamesForEntityType:(id)entityType;
- (PajeDrawingType)drawingTypeForEntityType:(id)entityType;
- (NSColor *)colorForName:(NSString *)name ofEntityType:(id)entityType;
- (void)setColor:(NSColor *)color
         forName:(id)name
    ofEntityType:(id)entityType;

//
// Getting info from object in trace
//
- (id)containerForEntity:(id<PajeEntity>)entity;
- (id)entityTypeForEntity:(id<PajeEntity>)entity;
- (id)sourceContainerForEntity:(id<PajeLink>)entity;
- (id)sourceEntityTypeForEntity:(id<PajeLink>)entity;
- (id)destContainerForEntity:(id<PajeLink>)entity;
- (id)destEntityTypeForEntity:(id<PajeLink>)entity;
- (NSArray *)relatedEntitiesForEntity:(id<PajeEntity>)entity;
- (NSColor *)colorForEntity:(id<PajeEntity>)entity;
- (NSDate *)startTimeForEntity:(id<PajeEntity>)entity;
- (NSDate *)endTimeForEntity:(id<PajeEntity>)entity;
- (NSDate *)timeForEntity:(id<PajeEntity>)entity;
- (PajeDrawingType)drawingTypeForEntity:(id<PajeEntity>)entity;
- (NSString *)nameForEntity:(id<PajeEntity>)entity;
- (NSString *)descriptionForEntity:(id<PajeEntity>)entity;

- (BOOL)canHighlightEntity:(id<PajeEntity>)entity;

- (void)verifyStartTime:(NSDate *)start endTime:(NSDate *)end;

//
// Inspecting object in trace
//
- (void)inspectEntity:(id<PajeInspecting>)entity;

//
// Removing entities from a trace
//
- (void)removeObjectsBeforeTime:(NSDate *)time;
- (void)removeObjectsAfterTime:(NSDate *)time;

//
// Getting time information
//
- (NSDate *)startTimeInMemory;
- (NSDate *)endTimeInMemory;
@end
