/*
 */

#import "Trace.h"
#import "Comparing.h"
#import "FoundationAdditions.h"

@implementation MiscTree(Description)
- (MiscTree *)subtreeWithUserInfo:(id)info
{
    NSEnumerator *allEnum = [self preorderAllSubtreeEnumerator];
    MiscTree *subtree;
    while (subtree = [allEnum nextObject]) {
        if ([[subtree userInfo] isEqual:info])
            return subtree;
    }
    return nil;
}

- (NSString *)descriptionWithLocale:(NSDictionary *)locale
                             indent:(unsigned int)level
{
    NSMutableDictionary *newDictionary = [NSMutableDictionary dictionary];
    if ([self userInfo])
        [newDictionary setObject:[self userInfo] forKey:@"UserInfo"];
    [newDictionary setObject:[self subtrees] forKey:@"Subtrees"];
    return [newDictionary descriptionWithLocale:locale indent:level];
}

// it doesn't copy the userInfo in the original MiscKit!
- (id)copyWithZone:(NSZone *)zone {
    MiscTree *newTree = [[isa allocWithZone:zone] init];
    NSEnumerator *enumerator = [self subtreeEnumerator];
    id obj;
    NSAssert(nil != newTree, @"could not create new tree");
    while ((obj = [enumerator nextObject])) {
        id newSubtree = [obj copyWithZone:zone];
        NSAssert(nil != newSubtree, @"could not copy subtree");
        [newTree addSubtree:newSubtree];
    }
    [newTree setUserInfo:[self userInfo]];
    return newTree;
}
@end

@implementation Trace

- (id)initWithStartTime:(NSDate *)t1 endTime:(NSDate *)t2
{
    [super init];

    entityLists = [[NSMutableDictionary alloc] init];
    entityTypeToInstances = [[NSMutableDictionary alloc] init];
    entityTypeToClass = [[NSMutableDictionary alloc] init];
    startTime = [t1 retain];
    endTime = [t2 retain];
    startTimeInMemory = [startTime retain];
    endTimeInMemory = [startTime retain];
    hierarchyTree = [[MiscTree alloc] init];
/*
    endTime = [[NSDate distantPast] retain];
    startTime = [[NSDate distantFuture] retain];;
    endTimeInMemory = [[NSDate distantPast] retain];
    startTimeInMemory = [[NSDate distantFuture] retain];;
*/
    
    return self;
}

- (void)dealloc
{
    [entityLists release];
    [entityTypeToInstances release];
    [entityTypeToClass release];
    [startTime release];
    [endTime release];
    [startTimeInMemory release];
    [endTimeInMemory release];
    [hierarchyTree release];

    [super dealloc];
}

- (NSDate *)time { return startTime; }

// time when the trace starts 
- (NSDate *)startTime { return startTime; }

// time when the trace ends 
- (NSDate *)endTime { return endTime; }

- (NSDate *)firstTime { return startTime; }
- (NSDate *)lastTime { return endTime; }

- (NSDate *)startTimeInMemory { return startTimeInMemory; }
- (NSDate *)endTimeInMemory { return endTimeInMemory; }

- (MiscTree *)hierarchyTree { return hierarchyTree; }

- (void)_verifyGlobals:(id<PajeEntity>)event
{
    NSDate *time = [event time];

    if([time compare:endTimeInMemory] == NSOrderedDescending) {
        [endTimeInMemory release];
        endTimeInMemory = [time retain];
    }
}

- (id)rootInstance
{
    return rootInstance;
}

- (void)_addToHierarchyTree:(id)entity
{
    NSMutableArray *a = [NSMutableArray array];
    NSEnumerator *types;
    id entityType;
    id father;
    MiscTree *tree = [self hierarchyTree];

    while ((father = [entity container]) != entity) {
        [a addObject:[entity entityType]];
        entity = father;
    }

    if (![tree userInfo]) {
        rootInstance = father;
        [tree setUserInfo:[entity entityType]];
    } else
        if (![[tree userInfo] isEqual:[entity entityType]])
            NSLog(@"Inconsistency in root: %@ != %@", [tree userInfo], [entity entityType]);
    
    types = [a reverseObjectEnumerator];
    while (entityType = [types nextObject]) {
        NSEnumerator *subtreeEnum = [tree subtreeEnumerator];
        MiscTree *subtree;

        while (subtree = [subtreeEnum nextObject]) {
            if ([[subtree userInfo] isEqual:entityType])
                break;
        }
        if (!subtree) {
            subtree = [MiscTree tree];
            [subtree setUserInfo:entityType];
            [tree addSubtree:subtree];
        }
        tree = subtree;
    }
}

NSMutableDictionary *countEntities;

- (void)addEntity:(id<PajeEntity>)entity
{
    NSString *entityType = [entity entityType];
    NSMutableDictionary *dict = [entityLists objectForKey:entityType];
    id container = [entity container];
    TimeList *list;

    //KLUDGE
    if ([entity drawingType] == PajeEventDrawingType)
        [self _verifyGlobals:entity];
    //ENDKLUDGE


/*
    if (!countEntities) {
        countEntities = [[NSMutableDictionary alloc] init];
        [countEntities setObject:[NSNumber numberWithDouble:[endTime timeIntervalSinceDate:startTime]] forKey:@"Time"];
    }
    if (![countEntities objectForKey:entityType])
        [countEntities setObject:[NSNumber numberWithInt:1] forKey:entityType];
    else
        [countEntities setObject:[NSNumber numberWithInt:[[countEntities objectForKey:entityType] intValue]+1] forKey:entityType];
    if (([entity drawingType] == PajeEventDrawingType)&&[[entity time] isEqual:endTime])
        NSLog(@"Entity Count: %@", countEntities);
*/

    
    if (!dict) {
        dict = [[NSMutableDictionary alloc] init];
        [entityLists setObject:dict forKey:entityType];
        [dict release];

        [self _addToHierarchyTree:entity];

        [entityTypeToClass setObject:[entity class] forKey:entityType];
    }

    list = [dict objectForKey:container];

    if (!list) {
        list = [[TimeList alloc] init];
        [dict setObject:list forKey:container];
        [list release];

        {
            // Generate entityTypeToInstances for all containers
            id instance = entity;
            while (container != instance) {
                id entityType = [container entityType];
                NSMutableSet *set = [entityTypeToInstances objectForKey:entityType];
                if (!set) {
                    set = [NSMutableSet setWithObject:container];
                    [entityTypeToInstances setObject:set forKey:entityType];
                } else {
                    [set addObject:container];
                }
                instance = container;
                container = [instance container];
            }
        }
    }
    [list addObject:entity];
}

- (void)entityChangedEndTime:(id<PajeEntity>)entity
{
    NSString *entityType = [entity entityType];
    NSMutableDictionary *dict = [entityLists objectForKey:entityType];
    id container = [entity container];
    TimeList *list;

    list = [dict objectForKey:container];

    if (!list) {
        NSLog(@"entity %@ not found (entityChangedEndTime)");
        return;
    }

    [list objectChangedEndTime:entity];
}

- (void)verifyStartTime:(NSDate *)start endTime:(NSDate *)end
{
    NSDictionary *userInfo = nil;
    if ([start earlierThanDate:startTimeInMemory]
        && [startTimeInMemory laterThanDate:startTime]) {
        userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
            [start laterDate:startTime], @"StartTime",
            [end earlierDate:endTime], @"EndTime",
            nil];
    } else if ([end laterThanDate:endTimeInMemory]
               && [endTimeInMemory earlierThanDate:endTime]) {
        userInfo = [NSDictionary dictionaryWithObjectsAndKeys:
            [start laterDate:startTime], @"StartTime",
            [end earlierDate:endTime], @"EndTime",
            nil];
    }
    if (userInfo) {
//        NSLog(@"Trace notificando:%@ p(%@ %@) m(%@ %@)", userInfo, [start description], [end description], [startTimeInMemory description], [endTimeInMemory description]);
        [[NSNotificationCenter defaultCenter]
                      postNotificationName:@"PajeTraceNotInMemoryNotification"
                                    object:self
                                  userInfo:userInfo];
//        NSLog(@"fim da notificacao");
    }
}


- (NSEnumerator *)enumeratorOfEntityType:(id)entityType
                             inContainer:(id)container
                                fromTime:(NSDate *)start
                                  toTime:(NSDate *)end
{
    NSDictionary *dict;
    [self verifyStartTime:start endTime:end];
    dict = [entityLists objectForKey:entityType];
    return [[dict objectForKey:container] timeEnumeratorFromTime:start
                                                          toTime:end];
}

- (NSEnumerator *)enumeratorOfEntityType:(id)entityType
                             inContainer:(id)container
{
    NSEnumerator *ienum;
    id instance;
    NSMutableArray *instancesInContainer = [NSMutableArray array];

    ienum = [[entityTypeToInstances objectForKey:entityType] objectEnumerator];
    while (instance = [ienum nextObject]) {
        if ([[instance container] isEqual:container]) {
            [instancesInContainer addObject:instance];
        }
    }
    return [[instancesInContainer sortedArrayUsingSelector:@selector(compare:)] objectEnumerator];
}


- (id)containerForEntity:(id<PajeEntity>)entity;
{
    return [entity container];
}
- (id)entityTypeForEntity:(id<PajeEntity>)entity;
{
    return [entity entityType];
}
- (id)sourceContainerForEntity:(id<PajeLink>)entity;
{
    return [entity sourceContainer];
}
- (id)sourceEntityTypeForEntity:(id<PajeLink>)entity;
{
    return [entity sourceEntityType];
}
- (id)destContainerForEntity:(id<PajeLink>)entity;
{
    return [entity destContainer];
}
- (id)destEntityTypeForEntity:(id<PajeLink>)entity;
{
    return [entity destEntityType];
}

- (NSArray *)relatedEntitiesForEntity:(id<PajeEntity>)entity
{
    return [entity relatedEntities];
}

- (NSColor *)colorForEntity:(id<PajeEntity>)entity
{
    return [entity color];
}

- (NSDate *)startTimeForEntity:(id<PajeEntity>)entity;
{
    return [entity startTime];
}

- (NSDate *)endTimeForEntity:(id<PajeEntity>)entity;
{
    return [entity endTime];
}

- (NSDate *)timeForEntity:(id<PajeEntity>)entity
{
    return [entity time];
}

- (PajeDrawingType)drawingTypeForEntity:(id<PajeEntity>)entity
{
    return [entity drawingType];
}

- (NSString *)nameForEntity:(id<PajeEntity>)entity
{
    return [entity name];
}

- (NSString *)descriptionForEntity:(id<PajeEntity>)entity
{
    return [NSString stringWithFormat:@"%@ [%@ in %@ %@]",
        [entity name], [entity entityType],
        [[entity container] entityType], [entity container]];
}

- (BOOL)canHighlightEntity:(id<PajeEntity>)entity
{
    return YES;
}



- (NSArray *)allNamesForEntityType:(id)entityType
{
    if ([entityType respondsToSelector:@selector(allNames)])
        return [entityType allNames];
    return [[entityTypeToClass objectForKey:entityType] allNames];
}

- (PajeDrawingType)drawingTypeForEntityType:(id)entityType
{
    Class entityClass;

    if ([entityType respondsToSelector:@selector(drawingType)])
        return [entityType drawingType];
    entityClass = [entityTypeToClass objectForKey:entityType];
    if ([entityClass respondsToSelector:@selector(drawingType)])
        return [entityClass drawingType];
    else
        return PajeStateDrawingType;
}

- (NSColor *)colorForName:(NSString *)name ofEntityType:(NSString *)entityType
{
    if ([entityType respondsToSelector:@selector(colorForName:)])
        return [entityType colorForName:name];
    return [[entityTypeToClass objectForKey:entityType] colorForName:name];
}

- (void)setColor:(NSColor *)color
         forName:(NSString *)name
    ofEntityType:(NSString *)entityType
{
    if ([entityType respondsToSelector:@selector(setColor:forName:)])
        [entityType setColor:color forName:name];
    else
        [[entityTypeToClass objectForKey:entityType] setColor:color
                                                      forName:name];
}


- (void)removeObjectsBeforeTime:(NSDate *)time
{
    NSEnumerator *e = [entityLists objectEnumerator];
    NSMutableDictionary *d;

    if ([time earlierDate:startTimeInMemory] == time)
        return;
    
//    NSLog(@"Trace removing before %@", [time description]);
    while (d = [e nextObject])
#if 0
        //xxx waiting for NSDictionary to implement makeObjectsPerformSelector:
        [d makeObjectsPerformSelector:@selector(removeObjectsBeforeTime:) withObject:time];
#else
    {
        NSEnumerator *e2 = [d objectEnumerator];
        TimeList *l;
        while (l = [e2 nextObject])
            [l removeObjectsBeforeTime:time];
    }
#endif
    
    [time retain];
    [startTimeInMemory release];
    startTimeInMemory = time;

    if ([time laterDate:endTimeInMemory] == time) {
        [endTimeInMemory release];
        endTimeInMemory = [time retain];
    }
}

- (void)removeObjectsAfterTime:(NSDate *)time
{
    NSEnumerator *e = [entityLists objectEnumerator];
    NSMutableDictionary *d;

    if ([time laterDate:endTimeInMemory] == time)
        return;
    
//    NSLog(@"Trace removing after %@", [time description]);
    while (d = [e nextObject])
#if 0
        //xxx waiting for NSDictionary to implement makeObjectsPerformSelector:
        [d makeObjectsPerformSelector:@selector(removeObjectsAfterTime:) withObject:time];
#else
    {
        NSEnumerator *e2 = [d objectEnumerator];
        TimeList *l;
        while (l = [e2 nextObject])
            [l removeObjectsAfterTime:time];
    }
#endif

    [time retain];
    [endTimeInMemory release];
    endTimeInMemory = time;

    if ([time earlierDate:startTimeInMemory]) {
        [startTimeInMemory release];
        startTimeInMemory = [time retain];
    }
}

- (void)inspectEntity:(id<PajeInspecting>)entity
{
    [entity inspect];
}
@end
