/* InsetLimit.m created by benhur on 25-feb-2001 */

#import "InsetLimit.h"
#import "../General.bproj/FoundationAdditions.h"
#import "../General.bproj/UniqueString.h"


@interface ImbricationFilteringEnumerator : NSEnumerator
{
    NSEnumerator *origEnum;
    PajeFilter *inputComponent;
    int minValue;
    int maxValue;
}
+ (ImbricationFilteringEnumerator *)enumeratorWithEnumerator:(NSEnumerator *)e
                                              inputComponent:(PajeFilter *)comp
                                                    minValue:(int)min
                                                    maxValue:(int)max;
- (id)initWithEnumerator:(NSEnumerator *)e
          inputComponent:(PajeFilter *)comp
                minValue:(int)min
                maxValue:(int)max;
- (id)nextObject;
@end
@implementation ImbricationFilteringEnumerator
+ (ImbricationFilteringEnumerator *)enumeratorWithEnumerator:(NSEnumerator *)e
                                              inputComponent:(PajeFilter *)comp
                                                    minValue:(int)min
                                                    maxValue:(int)max
{
    return [[[self alloc] initWithEnumerator:e
                              inputComponent:comp
                                    minValue:min
                                    maxValue:max] autorelease];
}
- (id)initWithEnumerator:(NSEnumerator *)e
          inputComponent:(PajeFilter *)comp
                minValue:(int)min
                maxValue:(int)max
{
    self = [super init];
    if (self) {
        origEnum = [e retain];
        inputComponent = comp;
        minValue = min;
        maxValue = max;
    }
    return self;
}

- (void)dealloc
{
    [origEnum release];
    [super dealloc];
}

- (id)nextObject
{
    id obj;
    while (obj = [origEnum nextObject]) {
        int val = [inputComponent imbricationLevelForEntity:obj];
        if (val >= minValue && val <= maxValue)
            break;
    }
    return obj;
}
@end


@implementation InsetLimit

- (id)init
{
    self = [super init];

    if (self) {
        NSWindow *win;

        [self readDefaults];

        if (![NSBundle loadNibNamed:@"InsetLimit" owner:self]) {
            NSRunAlertPanel(@"InsetLimit", @"Couldn't load interface file",
                            nil, nil, nil);
            return self;
        }

        [entityTypePopUp removeAllItems];

        win = [view window];
        
        // view is an NSBox. we need its contents.
        view = [(NSBox *)view contentView];

        [[NSNotificationCenter defaultCenter]
             postNotificationName:@"PajeRegisterFilterNotification"
                           object:self
                         userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
                             @"Imbrication Limit",  @"Name",
                             view,       @"View",
                             self,       @"Delegate", nil]];

#ifndef GNUSTEP
        [win release];
#endif
    }

    return self;
}

- (void)dealloc
{
    [limits release];
    [super dealloc];
}


//
// Notifications sent by other filters
//
- (void)hierarchyChanged
{
    [self calcPopUp];
    [super hierarchyChanged];
}

- (void)calcPopUp
{
    MiscTree *hierarchy;
    NSEnumerator *hierEnum;
    PajeEntityType *selectedEntityType;
    PajeEntityType *entityType;

    selectedEntityType = [self selectedEntityType];
    [entityTypePopUp removeAllItems];
    hierEnum = [[self hierarchyTree] preorderAllSubtreeEnumerator];
    while (hierarchy = [hierEnum nextObject]) {
        if ([hierarchy isLeaf]) {
            entityType = [hierarchy entityType];
            // only states have indentationLevel
            if ([entityType drawingType] == PajeStateDrawingType) {
                [entityTypePopUp addItemWithTitle:[entityType name]];
                [[entityTypePopUp lastItem] setRepresentedObject:entityType];
            }
        }
    }
    if ([entityTypePopUp numberOfItems] > 0) {
        if (selectedEntityType)
            [entityTypePopUp selectItemWithTitle:[selectedEntityType name]];
        if (![entityTypePopUp selectedItem])
            [entityTypePopUp selectItemAtIndex:0];
    }
    [self synchronizeValues];
}

- (void)synchronizeValues
{
    PajeEntityType *selectedEntityType;

    selectedEntityType = [self selectedEntityType];
    if (selectedEntityType == nil) {
        [minField setIntValue:0];
        [maxField setIntValue:0];
        [minField setEnabled:NO];
        [maxField setEnabled:NO];
    } else {
        NSString *value;
        NSRange range;
        value = [limits objectForKey:[selectedEntityType name]];
        if (value) {
            range = [value rangeValue];
        } else {
            range = NSMakeRange(0,0);
        }
        [minField setIntValue:range.location];
        [maxField setIntValue:range.length];
        [minField setEnabled:YES];
        [maxField setEnabled:YES];
    }
}

- (PajeEntityType *)selectedEntityType
{
    return [[entityTypePopUp selectedItem] representedObject];
}



//
//
//

- (void)readDefaults
{
    NSString *defaultName = [NSStringFromClass([self class]) stringByAppendingString:@" Limits"];
    NSDictionary *limitsDict = [[NSUserDefaults standardUserDefaults] dictionaryForKey:defaultName];
    [limits release];
    if (limitsDict != nil)
        limits = [limitsDict mutableCopy];
    else
        limits = [[NSMutableDictionary alloc] init];
}

- (void)registerDefaults
{
    NSString *defaultName = [NSStringFromClass([self class]) stringByAppendingString:@" Limits"];
    [[NSUserDefaults standardUserDefaults] setObject:limits forKey:defaultName];
}


//
// Handle interface messages
//
- (IBAction)entityTypeSelected:(id)sender
{
    [self synchronizeValues];
}

- (IBAction)valueChanged:(id)sender
{
    PajeEntityType *selectedEntityType;
    NSRange ran;
    selectedEntityType = [self selectedEntityType];
    ran.location = [minField intValue];
    ran.length = [maxField intValue];
    if ((ran.location == 0) && (ran.length == 0))
        [limits removeObjectForKey:[selectedEntityType name]];
    else
        [limits setObject:[NSString stringWithRange:ran]
                   forKey:[selectedEntityType name]];
    [self registerDefaults];
    [self dataChangedForEntityType:selectedEntityType];
}

- (IBAction)increaseMinValue:(id)sender
{
    [minField setIntValue:[minField intValue]+1];
    [self valueChanged:self];
}

- (IBAction)increaseMaxValue:(id)sender
{
    [maxField setIntValue:[maxField intValue]+1];
    [self valueChanged:self];
}

- (IBAction)decreaseMinValue:(id)sender
{
    int oldValue = [minField intValue];
    if (oldValue > 0)
        [minField setIntValue:oldValue-1];
    [self valueChanged:self];
}

- (IBAction)decreaseMaxValue:(id)sender
{
    int oldValue = [maxField intValue];
    if (oldValue > 0)
        [maxField setIntValue:oldValue-1];
    [self valueChanged:self];
}


//
// interact with interface
//

- (void)viewWillBeSelected:(NSView *)selectedView
// message sent by PajeController when a view will be selected for display
{
    [self synchronizeValues];        
}


//
// Trace messages that are filtered
//

- (NSEnumerator *)enumeratorOfEntityType:(PajeEntityType *)entityType
                             inContainer:(PajeContainer *)container
                                fromTime:(NSDate *)start
                                  toTime:(NSDate *)end
{
    NSString *val;
    int min, max;
    NSRange ran;
    NSEnumerator *origEnum;
    origEnum = [inputComponent enumeratorOfEntityType:entityType
                                          inContainer:container
                                             fromTime:start
                                               toTime:end];
    val = [limits objectForKey:[entityType name]];
    if (val == nil)
        return origEnum;
    ran = [val rangeValue];
    min = ran.location;
    max = ran.length;
    return [ImbricationFilteringEnumerator enumeratorWithEnumerator:origEnum
                                                     inputComponent:inputComponent
                                                           minValue:min
                                                           maxValue:max];
}


- (int)imbricationLevelForEntity:(PajeEntity *)entity;
{
    int origValue;
    NSString *val;
    NSRange ran;

    origValue = [inputComponent imbricationLevelForEntity:entity];
    val = [limits objectForKey:[[self entityTypeForEntity:entity] name]];

    if (val) {
        ran = [val rangeValue];
        if (origValue > ran.length) origValue = ran.length;
        origValue -= ran.location;
    }
    return origValue;
}


@end
