#import "ContainerGroup.h"
#import "../General.bproj/Macros.h"
#import "../General.bproj/PajeType.h"

@implementation ContainerGroup

+ (ContainerGroup *)groupWithObject:(id)obj
{
    return [[[self alloc] initWithObject:obj] autorelease];
}

- (id)initWithObject:(id)obj
{
    [super init];
    members = [[NSMutableArray alloc] init];
    [members addObject:obj];
    startTime = [obj startTime];
    endTime = [obj endTime];
    return self;
}

- (void)dealloc
{
    [members release];
    [super dealloc];
}

- (NSArray *)members
{
    return members;
}

- (void)addObject:(id)obj
{
    [members addObject:obj];
    startTime = [startTime earlierDate:[obj startTime]];
    endTime = [endTime laterDate:[obj endTime]];
}

- (void)removeObject:(id)obj
{
    // BUG: startTime and endTime can be wrong after this.
    [members removeObject:obj];
}

- (unsigned)count
{
    return [members count];
}

- (NSEnumerator *)objectEnumerator
{
    return [members objectEnumerator];
}

- (NSString *)description
{
    int count, n;
    NSMutableString *s;
    count = [members count];
    if (count == 0)
        return @"";
    s = [NSMutableString stringWithString:[[members objectAtIndex:0] description]];
    for (n = 1; n < count; n++) {
        [s appendString:@","];
        [s appendString:[[members objectAtIndex:n] description]];
    }
    return s;
}

- (void)setContainer:(PajeContainer *)c
{
    Assign(container, c);
}

- (PajeContainer *)container
{
    if (container)
        return container;
    return [[members objectAtIndex:0] container];
}

- (PajeEntityType *)entityType
{
    return [[members objectAtIndex:0] entityType];
}

// for VirtualThread...
- (NSDate *)startTime
{
    return startTime;
}
- (NSDate *)endTime
{
    return endTime;
}


- (id)copyWithZone:(NSZone *)zone
{
    ContainerGroup *newGroup = [[ContainerGroup allocWithZone:zone] init];
    newGroup->members = [members copyWithZone:zone];
    newGroup->startTime = startTime;
    newGroup->endTime = endTime;
    return newGroup;
}

- (unsigned)hash
{
    return [members hash];
}

- (BOOL)isEqual:(id)obj
{
    if ([obj isKindOfClass:[ContainerGroup class]])
        return [members isEqual:[obj members]];
    return [super isEqual:obj];
}
@end