#import "NGController.h"
#import "NodeGroup.h"
#import "ContainerGroup.h"

@implementation NGController

- (id)initWithFilter:(NodeGroup *)comp
{
    [super init];
    filter = comp;

    // load the interface
    if (![NSBundle loadNibNamed:@"NodeGroup" owner:self])
        NSRunAlertPanel(@"NodeGroup",
                        @"Couldn't load interface file",
                        nil, nil, nil);
    return self;
}

- (void)dealloc
{
    [super dealloc];
}

- (void)awakeFromNib
{
    // view is an NSBox. we need its contents.
    view = [(NSBox *)view contentView];
#ifndef GNUSTEP
    [[view window] autorelease];
#endif
    [view removeFromSuperview];

    [[NSNotificationCenter defaultCenter] postNotificationName:@"PajeRegisterFilterNotification" object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
                        @"Node Groups", @"Name",
                        view, @"View", self, @"Delegate", nil]];
}


- (void)reflectChanges
// something has changed, update display.
// message sent by the filter.
{
    NSArray *groups = [filter allGroups];
    int count = [groups count];
    int row;
    
    [matrix renewRows:count columns:1];
    for (row = 0; row < count; row++) {
        NSButtonCell *cell = [matrix cellAtRow:row column:0];
        ContainerGroup *group = [groups objectAtIndex:row];
        [cell setTitle:[group description]];
        [cell setRepresentedObject:group];
    }
    [matrix sizeToFit];
    [matrix setNeedsDisplay:YES];
}

- (IBAction)changeSelection:(id)sender
// user has changed the selected lines in matrix.
// enable buttons accordingly
{
    int count = [[matrix selectedCells] count];
    [groupButton setEnabled:count > 1];
    [ungroupButton setEnabled:count > 0];
}


- (IBAction)group:(id)sender
// Create a new group with the selected nodes.
{
    NSMutableArray *groups = [NSMutableArray array];
    NSEnumerator *cellEnum = [[matrix selectedCells] objectEnumerator];
    NSCell *cell;
    
    while (cell = [cellEnum nextObject])
        [groups addObject:[cell representedObject]];

    [filter mergeGroups:groups];

    [self reflectChanges];
}

- (IBAction)ungroup:(id)sender
// ungroup selected group(s)
{
    NSEnumerator *cellEnum = [[matrix selectedCells] objectEnumerator];
    NSCell *cell;

    while (cell = [cellEnum nextObject])
        [filter ungroupGroup:[cell representedObject]];

    [self reflectChanges];
}

@end
