  /*
   * NodeGroup.m
   *
   * Paje filter that groups the nodes in a trace.
   *
   * 19980105 BS  creation
   */

#import "NodeGroup.h"
#import "NGController.h"
#import "NGCombinedEnumerator.h"
#import "../General.bproj/MultiEnumerator.h"
#import "../General.bproj/Macros.h"


@implementation NodeGroup

- (id)init
{
    [super init];

    allGroups = [[NSMutableSet alloc] init];
    nodeToGroup = [[NSMutableDictionary alloc] init];

    // initialize the controller.
    controller = [[NGController alloc] initWithFilter:self];

    return self;
}

- (void)dealloc
{
    [allGroups release];
    [nodeToGroup release];
    [super dealloc];
}

- (NSArray *)allGroups
{
    return [allGroups allObjects];
}

- (void)_removeNode:(id)node
{
    ContainerGroup *group = [nodeToGroup objectForKey:node];

    if (!group) return;

    [group removeObject:node];
    if ([group count] == 0) {
        [allGroups removeObject:group];
    }
    
    [nodeToGroup removeObjectForKey:node];
}

//////////////////////////////
NSMutableDictionary *superThreads;

- (void)destroySuperThreadOfGroup:(id)group
{
    ContainerGroup *superThread = [superThreads objectForKey:group];
    if (superThread) {
        NSEnumerator *e = [superThread objectEnumerator];
        id m;
        while (m = [e nextObject]) {
            [nodeToGroup removeObjectForKey:m];
        }
        [superThreads removeObjectForKey:group];
    }
}
//////////////////////////////

- (void)_addNode:(id)node
{
    if (nil == [nodeToGroup objectForKey:node]) {
        ContainerGroup *newGroup = [ContainerGroup groupWithObject:node];
        [nodeToGroup setObject:newGroup forKey:node];
        [allGroups addObject:newGroup];
    }
}

- (void)mergeGroups:(NSArray *)groups
{
    ContainerGroup *newGroup;
    int i;
    int count = [groups count];

    if (count < 2)
        return;

    newGroup = [[groups objectAtIndex:0] retain];
    [allGroups removeObject:newGroup];

    for (i = 1; i < count; i++) {
        ContainerGroup *group = [groups objectAtIndex:i];
        NSEnumerator *groupEnum = [group objectEnumerator];
        id member;

        while (member = [groupEnum nextObject]) {
            [newGroup addObject:member];
            [nodeToGroup setObject:newGroup forKey:member];
        }
/////
        [self destroySuperThreadOfGroup:group];
//////
        [allGroups removeObject:group];
    }
    [allGroups addObject:newGroup];
    [newGroup release];

    [super hierarchyChanged];
}

- (void)ungroupGroup:(ContainerGroup *)group
{
    NSEnumerator *groupEnum = [group objectEnumerator];
    id member;

    while (member = [groupEnum nextObject]) {
        ContainerGroup *newGroup = [ContainerGroup groupWithObject:member];
        [nodeToGroup setObject:newGroup forKey:member];
        [allGroups addObject:newGroup];
    }
    [allGroups removeObject:group];

/////
       [self destroySuperThreadOfGroup:group];
//////

    [super hierarchyChanged];
}


- (void)hierarchyChanged
{
    NSEnumerator *nodeEnum;
    id nodeId;
#ifdef GNUSTEP
    NSMutableSet *nodesFromTrace = [[[NSMutableSet alloc] init] autorelease];
#else
    NSMutableSet *nodesFromTrace = [NSMutableSet set];
#endif
    id nodes;

    // insert all nodes from trace
#if 1
    nodeEnum = [super enumeratorOfEntityType:@"Node"
                                 inContainer:[super rootInstance]];
#else
    NSEnumerator *enum1 = [super enumeratorOfEntityType:@"Node"
                                            inContainer:[super rootInstance]];
    MultiEnumerator *enum2 = [MultiEnumerator enumerator];
    id n;
    while (n = [enum1 nextObject])
        [enum2 addEnumerator:[super enumeratorOfEntityType:@"Thread"
                                               inContainer:n]];
    nodeEnum = enum2;
#endif

    while (nodeId = [nodeEnum nextObject]) {
        [self _addNode:nodeId];
        [nodesFromTrace addObject:nodeId];
    }

    // remove all nodes not in trace
    nodes = [[nodeToGroup allKeys] copy];
    nodeEnum = [nodes objectEnumerator];
    while (nodeId = [nodeEnum nextObject]) {
        if (![nodesFromTrace containsObject:nodeId])
            [self _removeNode:nodeId];
    }
    [nodes release];
        
    [controller reflectChanges];
    [super hierarchyChanged];
}


// From Trace

- (NSEnumerator *)enumeratorOfEntityType:(PajeEntityType *)entityType
                             inContainer:(PajeContainer *)container
{
#if 1
    if ([entityType isEqual:@"Node"])
        return [allGroups objectEnumerator];
#else
    if ([entityType isEqual:@"Thread"]) {
        NSMutableArray *a = [NSMutableArray array];
        ContainerGroup *g;
        NSEnumerator *e=[allGroups objectEnumerator];
        while (g = [e nextObject])
            if ([container isEqual:[g container]])
                [a addObject:g];
        return [a objectEnumerator];
    }
#endif
    else if ([container isKindOfClass:[ContainerGroup class]]) {
        ContainerGroup *group = (ContainerGroup *)container;
        MultiEnumerator *multienum = [MultiEnumerator enumerator];
        NSEnumerator *groupEnum = [group objectEnumerator];
        id member;

        while (member = [groupEnum nextObject])
            [multienum addEnumerator:[super enumeratorOfEntityType:entityType
                                                       inContainer:member]];
//////////////////////////////////
if ([group count] > 1) {
   if ([[entityType description] isEqual:@"Thread"]) {
       id superThread;
       if (!superThreads) superThreads = [[NSMutableDictionary alloc] init];
       superThread = [superThreads objectForKey:group];
       if (!superThread) {
           member = [multienum nextObject];
           superThread = [ContainerGroup groupWithObject:member];
           [superThread setContainer:group];
           [nodeToGroup setObject:superThread forKey:member];
           while(member = [multienum nextObject]) {
               [nodeToGroup setObject:superThread forKey:member];
               [superThread addObject:member];
           }
           [superThreads setObject:superThread forKey:group];
       }
       return [[NSArray arrayWithObject:superThread] objectEnumerator];
   }
}
//////////////////////////////////

        return multienum;
    } else
        return [super enumeratorOfEntityType:entityType
                                 inContainer:container];
}


- (NSEnumerator *)enumeratorOfEntityType:(PajeEntityType *)entityType
                             inContainer:(PajeContainer *)container
                                fromTime:(NSDate *)start
                                  toTime:(NSDate *)end
{
////////
    if ([[entityType description] isEqual:@"Event"]
        && ([[[container container] description] isEqual:@"0"]
            || ([container isKindOfClass:[ContainerGroup class]]
                && [(ContainerGroup *)container count]>1)))
        return nil;
    ////////////

    if ([container isKindOfClass:[ContainerGroup class]]) {
        ContainerGroup *group = (ContainerGroup *)container;
        NGCombinedEnumerator *multienum = [NGCombinedEnumerator enumerator];
        NSEnumerator *groupEnum = [group objectEnumerator];
        id member;
        
        while (member = [groupEnum nextObject])
            [multienum addEnumerator:[super enumeratorOfEntityType:entityType
                                                       inContainer:member
                                                          fromTime:start
                                                            toTime:end]];
        return multienum;
    } else
        return [super enumeratorOfEntityType:entityType
                                 inContainer:container
                                    fromTime:start
                                      toTime:end];
}



- (PajeContainer *)_convertContainer:(PajeContainer *)container
{
    id group;
//#if 1
//    if ([[container entityType] isEqual:@"Node"])
//#else
//    if ([[container entityType] isEqual:@"Thread"])
//#endif
        group = [nodeToGroup objectForKey:container];
        if (group)
            return group;
    ///////////////////
//        else {
//            if ([[container entityType] isEqual:@"Thread"]) {
//                if ([[superThread members] containsObject:container])
//                    return superThread;
//            }
//        }
    ///////////////////

    return container;
}

- (PajeContainer *)containerForEntity:(id<PajeEntity>)entity
{
    return [self _convertContainer:[super containerForEntity:entity]];
}

- (PajeContainer *)sourceContainerForEntity:(id<PajeLink>)entity
{
    return [self _convertContainer:[super sourceContainerForEntity:entity]];
}

- (PajeContainer *)destContainerForEntity:(id<PajeLink>)entity
{
    return [self _convertContainer:[super destContainerForEntity:entity]];
}

@end
