/* Order.m created by benhur on 20-mar-2001 */

/*
 * A Paje filter to change the order of containers
 */

#import "Order.h"
#import "OrderKey.h"

@implementation Order

- (id)init
{
    NSWindow *win;
    NSNotificationCenter *notifCenter;
    NSDictionary *filterDict;
    
    self = [super init];

    if (self != nil) {
        containers = [[NSMutableDictionary alloc] init];

        if (![NSBundle loadNibNamed:@"Order" owner:self]) {
            NSRunAlertPanel(@"Order", @"Couldn't load interface file",
                            nil, nil, nil);
            return self;
        }

        win = [view window];

        // view is an NSBox. we need its contents.
        view = [view contentView];

        hierarchyBrowser = [[HierarchyBrowser alloc] initWithFilter:self];
        [hierarchyBrowser setContainersOnly:YES];
        [[hierarchyBrowser view] setFrame:[browserBox frame]];
        [[browserBox superview] replaceSubview:browserBox
                                          with:[hierarchyBrowser view]];

        notifCenter = [NSNotificationCenter defaultCenter];
        filterDict = [NSDictionary dictionaryWithObjectsAndKeys:
            @"Order",   @"Name",
            view,       @"View",
            self,       @"Delegate", nil];
        [notifCenter postNotificationName:@"PajeRegisterFilterNotification"
                                   object:self
                                 userInfo:filterDict];

#ifndef GNUSTEP
        [win release];
#endif

    }
    return self;
}

- (void)dealloc
{
    [containers release];
    [super dealloc];
}


//
// Notifications sent by other filters
//
- (void)hierarchyChanged
{
    MiscTree *hierarchy;
    NSEnumerator *hierEnum;
    /*
    hierEnum = [[inputComponent hierarchyTree] preorderAllSubtreeEnumerator];
    while (hierarchy = [hierEnum nextObject]) {
        if (![hierarchy isLeaf] && [hierarchy entityType])
            [self setFilterForEntityType:[hierarchy entityType]];
    }
    [self synchronizeMatrix];
     */
#ifdef GNUSTEP
    [hierarchyBrowser setFilter:self];
#endif
    [super hierarchyChanged];
}



//
// Handle interface messages
//
- (void)selectAll:(id)sender
{
    PajeEntityType *entityType;
    PajeContainer *container;
    OrderKey *key;
    NSMutableArray *containerArray;
    NSArray *selectedContainers;
    unsigned index;

    selectedContainers = [hierarchyBrowser selectedContainers];
    if ((selectedContainers == nil) || ([selectedContainers count] == 0)) {
        NSBeep();
        return;
    }
    
    entityType = [hierarchyBrowser selectedEntityType];
    container = [hierarchyBrowser selectedParentContainer];
    key = [OrderKey keyWithEntityType:entityType container:container];
    containerArray = [containers objectForKey:key];

    if (containerArray == nil) {
        NSEnumerator *unfilteredEnum;
        unfilteredEnum = [super enumeratorOfEntityType:entityType
                                          inContainer:container];
        containerArray = [NSMutableArray arrayWithArray:[unfilteredEnum allObjects]];
        [containers setObject:containerArray forKey:key];
    }

    index = [containerArray indexOfObjectIdenticalTo:[selectedContainers objectAtIndex:0]];
    if (index == NSNotFound) {
        [NSException raise:@"Order: Internal Inconsistency"
                    format:@"Container Not Found: %@", [selectedContainers objectAtIndex:0]];
    }
    if (index > 0) index--;
    [containerArray removeObjectsInArray:selectedContainers];
    [containerArray replaceObjectsInRange:NSMakeRange(index, 0)
                     withObjectsFromArray:selectedContainers];
//    [containers setObject:containerArray forKey:key];
//    [hierarchyBrowser reloadColumn:[hierarchyBrowser selectedColumn]];
    [hierarchyBrowser refreshLastColumn];
    [super dataChangedForEntityType:entityType];
}

- (void)unselectAll:(id)sender
{
    PajeEntityType *entityType;
    PajeContainer *container;
    OrderKey *key;
    NSMutableArray *containerArray;
    NSArray *selectedContainers;
    unsigned index;

    selectedContainers = [hierarchyBrowser selectedContainers];
    if ((selectedContainers == nil) || ([selectedContainers count] == 0)) {
        NSBeep();
        return;
    }

    entityType = [hierarchyBrowser selectedEntityType];
    container = [hierarchyBrowser selectedParentContainer];
    key = [OrderKey keyWithEntityType:entityType container:container];
    containerArray = [containers objectForKey:key];

    if (containerArray == nil) {
        NSEnumerator *unfilteredEnum;
        unfilteredEnum = [super enumeratorOfEntityType:entityType
                                          inContainer:container];
        containerArray = [NSMutableArray arrayWithArray:[unfilteredEnum allObjects]];
        [containers setObject:containerArray forKey:key];
    }

    index = [containerArray indexOfObjectIdenticalTo:[selectedContainers objectAtIndex:0]];
    if (index == NSNotFound) {
        [NSException raise:@"Order: Internal Inconsistency"
                    format:@"Container Not Found: %@", [selectedContainers objectAtIndex:0]];
    }
    if (index < [containerArray count] - [selectedContainers count]) index++;
    [containerArray removeObjectsInArray:selectedContainers];
    [containerArray replaceObjectsInRange:NSMakeRange(index, 0)
                     withObjectsFromArray:selectedContainers];
    [containers setObject:containerArray forKey:key];
//    [hierarchyBrowser reloadColumn:[hierarchyBrowser selectedColumn]];
    [hierarchyBrowser refreshLastColumn];
    [super dataChangedForEntityType:entityType];
}


/*
- (void)entityTypeSelected:(id)sender
{
    [self synchronizeMatrix];
}

- (void)synchronizeMatrix
{
    [hierarchyBrowser setFilter:self];
}
*/

//
// interact with interface
//

/*
- (void)viewWillBeSelected:(NSView *)selectedView
// message sent by PajeController when a view will be selected for display
{
    [self synchronizeMatrix];        
}
*/

//
// Queries that are filtered
//
- (NSEnumerator *)enumeratorOfEntityType:(PajeEntityType *)entityType
                             inContainer:(PajeContainer *)container
{
    OrderKey *key;
    NSArray *containerArray;

    key = [OrderKey keyWithEntityType:entityType container:container];
    containerArray = [containers objectForKey:key];
    
    if (containerArray == nil) {
        return [super enumeratorOfEntityType:entityType
                                 inContainer:container];
    } else {
        return [containerArray objectEnumerator];
    }
}

@end
