#import "PajeController.h"
#import "PajeCheckPoint.h"
#if 0
#ifndef GNUSTEP
#import "OEObject.h"
#endif
#endif
#import "../General.bproj/Protocols.h"
#import "../General.bproj/Macros.h"
#import "../General.bproj/PajeFilter.h"

@implementation PajeController

- (NSBundle *)loadBundleNamed:(NSString*)name
{
    NSString *bundleName;
    NSMutableArray *bundlePaths;
    NSEnumerator *pathEnumerator;
    NSString *path;
    NSString *bundlePath;
    NSBundle *bundle;

    bundleName = [@"Bundles" /*stringByAppendingPathComponent:@"Paje"];
    bundleName = [bundleName*/ stringByAppendingPathComponent:name];
    bundleName = [bundleName stringByAppendingPathExtension:@"bundle"];

    bundlePaths = [[NSUserDefaults standardUserDefaults]
                                       arrayForKey:@"BundlePaths"];
    if (!bundlePaths) {
        bundlePaths = NSStandardLibraryPaths();
    }

    // insert application path (eases development)
    bundlePaths = [[NSArray arrayWithObject:[[NSBundle mainBundle] bundlePath]]
                                 arrayByAddingObjectsFromArray:bundlePaths];

    pathEnumerator = [bundlePaths objectEnumerator];
    while (path = [pathEnumerator nextObject]) {
        bundlePath = [path stringByAppendingPathComponent:bundleName];
        bundle = [NSBundle bundleWithPath:bundlePath];
        if ([bundle load]) {
            return bundle;
        }

    }
    [NSException raise:@"PajeException" format:@"Bundle '%@' not found", name];
    return nil;
}

- (void)connectComponent:(id)c1 toComponent:(id)c2
{
    [c1 setOutputComponent:c2];
    [c2 setInputComponent:c1];
}

id encapsulator;
- (void)applicationWillFinishLaunching:(NSNotification *)notification
{
    id virtualizer;
    id simul;
    id displayfilter;
    id containerselector;
    id order;
    id statViewer;
    id nodeGroup;
    id busyNode;
    id spacetime;
    id insetlimit;

#if 0
#ifndef GNUSTEP
    // initialize EOObject, the Crash Trapper
    [OEObject setup];
    [OEObject setContinueAfterError:YES];
#endif
#endif

    [filtersPopUp removeAllItems];
    [filtersMenu removeItem:[filtersMenu itemWithTag:0]];
    filterViews = [[NSMutableArray array] retain];
    filterDelegates = [[NSMutableArray array] retain];
    components = [[NSMutableArray array] retain];

    // accept notifications defining new filters
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(registerFilter:) name:@"PajeRegisterFilterNotification" object:nil];

    // accept notifications posted when we should read more of the trace file
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(traceFault:) name:@"PajeTraceNotInMemoryNotification" object:nil];

    // accept notifications defining new tools
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(registerTool:) name:@"PajeRegisterToolNotification" object:nil];

//    [NSBundle stripAfterLoading:NO];

    [self loadBundleNamed:@"General"];
    [self loadBundleNamed:@"A0bReader"];
    [self loadBundleNamed:@"A0bSimul"];
    [self loadBundleNamed:@"Encapsulate"];
    [self loadBundleNamed:@"SpaceTime"];
    [self loadBundleNamed:@"DisplayFilter"];
    [self loadBundleNamed:@"ContainerSelector"];
    [self loadBundleNamed:@"Order"];
//    [self loadBundleNamed:@"VirtualThread"];
    [self loadBundleNamed:@"BusyNode"];
//    [self loadBundleNamed:@"StatViewer"];
//    [self loadBundleNamed:@"NodeGroup"];
    [self loadBundleNamed:@"InsetLimit"];

    reader =        [[NSClassFromString(@"A0bReader")     alloc] init];
    [components addObject:reader];
    simul =         [[NSClassFromString(@"A0bSimul")      alloc] init];
    [components addObject:simul];
    encapsulator =  [[NSClassFromString(@"Encapsulate")   alloc] init];
    [components addObject:encapsulator];
//    virtualizer =   [[NSClassFromString(@"VirtualThread") alloc] init];
//    [components addObject:virtualizer];
    displayfilter = [[NSClassFromString(@"DisplayFilter") alloc] init];
    [components addObject:displayfilter];
    containerselector = [[NSClassFromString(@"ContainerSelector") alloc] init];
    [components addObject:containerselector];
    order = [[NSClassFromString(@"Order") alloc] init];
    [components addObject:order];
//    nodeGroup =     [[NSClassFromString(@"NodeGroup")     alloc] init];
//    [components addObject:nodeGroup];
    busyNode =      [[NSClassFromString(@"BusyNode")      alloc] init];
    [components addObject:busyNode];
    insetlimit =    [[NSClassFromString(@"InsetLimit")      alloc] init];
    [components addObject:insetlimit];
    spacetime =     [[NSClassFromString(@"STController")  alloc] init];
    [components addObject:spacetime];
//    statViewer =    [[NSClassFromString(@"StatViewer")    alloc] init];
//    [components addObject:statViewer];

    [reader        setOutputComponent:simul];
    [simul         setOutputComponent:encapsulator];
    [self connectComponent:encapsulator      toComponent:containerselector];
    [self connectComponent:containerselector toComponent:order            ];
    [self connectComponent:order             toComponent:busyNode         ];
    [self connectComponent:busyNode          toComponent:displayfilter    ];
    [self connectComponent:displayfilter     toComponent:insetlimit       ];
    [self connectComponent:insetlimit        toComponent:spacetime        ];

//    spacetime =     [[NSClassFromString(@"STController")  alloc] init];
//    [components addObject:spacetime];
//    [self connectComponent:insetlimit        toComponent:spacetime        ];
    //virtualizer
    //statViewer    
}

- (void)applicationWillTerminate:(NSNotification *)notif
{
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (void)print:(id)sender
{
    [[[NSApplication sharedApplication] keyWindow] print:sender];
}

- (void)open:(id)sender
{
    id		openFilePanel;
    int		result;
    NSString   *filename = [[NSUserDefaults standardUserDefaults] stringForKey:@"LastOpenFile"];

    if (!filename) filename = @"";
    [filename retain];
    
    openFilePanel = [NSOpenPanel openPanel];
    [openFilePanel setCanChooseDirectories:NO];
    [openFilePanel setCanChooseFiles:YES];

    result = [openFilePanel runModalForDirectory:[filename stringByDeletingLastPathComponent] file:[filename lastPathComponent] types:[NSArray arrayWithObjects:@"trace", nil]];
    
    if (result == NSCancelButton)
        return;

    [filename release];
    filename = [openFilePanel filename];

    [self application:[NSApplication sharedApplication] openFile:filename];
}

- (void)readOne:(id)sender
{
    int i;
    NS_DURING
        for (i=0; i<1 && [reader readNextEvent]; i++)
            ;
    NS_HANDLER
        if (NSRunAlertPanel([localException name], @"%@\n%@",
                            @"Continue", @"Abort", nil,
                            [localException reason],
                            [[[localException userInfo] objectEnumerator] allObjects]) != NSAlertDefaultReturn)
            [[NSApplication sharedApplication] terminate:self];
    NS_ENDHANDLER
}

//
// message sent by NSApplication if the user double-clicks a trace file
//
- (BOOL)application:(NSApplication *)sender openFile:(NSString *)filename
{
    NSDictionary *filedict;
    NS_DURING
        filedict = [reader setInputFilename:filename];

        [[NSUserDefaults standardUserDefaults] setObject:filename
                                                  forKey:@"LastOpenFile"];
        
        [[NSNotificationCenter defaultCenter] postNotificationName:@"PajeFilenameNotification" object:self userInfo:filedict];

        [self performSelector:@selector(readAll:)
                   withObject:self
                   afterDelay:0.0];

        [checkPoints release];
        checkPoints = [[NSClassFromString(@"PSortedArray") alloc] initWithSelector:@selector(time)];
        [self writeCheckPoint];

        NS_VALUERETURN(YES, BOOL);
    NS_HANDLER
        if (NSRunAlertPanel([localException name], @"%@\n%@",
                            @"Continue", @"Abort", nil,
                            [localException reason],
                            [[[localException userInfo] objectEnumerator] allObjects]) != NSAlertDefaultReturn)
            [[NSApplication sharedApplication] terminate:self];
        NS_VALUERETURN(NO, BOOL);
    NS_ENDHANDLER
}

- (void)readAll:(id)sender
{
    int i = 0;
    NSDate *start = [NSDate date], *end;
    double t;
    NSAutoreleasePool *pool = [NSAutoreleasePool new];
    
    NS_DURING
        while([reader readNextEvent]) {
            i++;
            if ((i) % 1000 == 0) {
                //[self writeCheckPoint];
                [pool release];
                pool = [NSAutoreleasePool new];
//break;
            }
        }

        [pool release];
        end = [NSDate date];
        t = [end timeIntervalSinceDate:start];
        NSLog(@"%@: %d events in %f seconds = %f e/s", [reader inputFilename], i, t, i/t);

    NS_HANDLER
        if (NSRunAlertPanel([localException name], @"%@\n%@",
                            @"Continue", @"Abort", nil,
                            [localException reason],
                            [[[localException userInfo] objectEnumerator] allObjects]) != NSAlertDefaultReturn)
            [[NSApplication sharedApplication] terminate:self];
    NS_ENDHANDLER
}

- (void)writeCheckPoint
{
#if 0
    NSMutableData *d;
    NSArchiver *a;
    id comp_enum = [components objectEnumerator];
    id component;
    NSDate *time = [reader currentTime];
    NSString *fileName = [NSString stringWithFormat:@"/tmp/Paje-%@.ckp", [time description]];
    PajeCheckPoint *checkPoint = [PajeCheckPoint checkPointWithTime:time
                                                           fileName:fileName];

    if ([checkPoints indexOfObject:checkPoint] != NSNotFound)
        return;

    d = [[NSMutableData alloc] init];
    a = [[NSArchiver alloc] initForWritingWithMutableData:d];
    while (component = [comp_enum nextObject]) {
        if ([component respondsToSelector:@selector(encodeCheckPointWithCoder:)]) {
            [component encodeCheckPointWithCoder:a];
        }
    }
    [d writeToFile:fileName atomically:NO];
    [a release];
    [d release];
    
    [checkPoints addObject:checkPoint];
#endif
}

- (void)gotoCheckPoint:(PajeCheckPoint *)checkPoint
{
#if 0
    NSData *d = [[NSData alloc] initWithContentsOfFile:[checkPoint fileName]];
    NSUnarchiver *a=[[NSUnarchiver alloc] initForReadingWithData:d];
    id comp_enum = [components objectEnumerator];
    id component;
    while (component = [comp_enum nextObject])
        if ([component respondsToSelector:@selector(decodeCheckPointWithCoder:)]) {
            NSLog(@"%@ lendo %@", NSStringFromClass([component class]), [checkPoint fileName]);
            [component decodeCheckPointWithCoder:a];
        }
    [a release];
    [d release];
    [encapsulator removeObjectsAfterTime:[checkPoint time]];


    //KLUDGE
    [self performSelector:@selector(readAll:)
               withObject:self
               afterDelay:0.0];

#endif
}

- (void)traceFault:(NSNotification *)notification
{
    NSDictionary *info;
    NSDate *startTime;
    NSDate *endTime;
    unsigned index;
    int index2;
    PajeCheckPoint *checkPoint;
    int i;
    BOOL endOfFile;

    return;
//#if 0
    info = [notification userInfo];
    startTime = [info objectForKey:@"StartTime"];
    endTime = [info objectForKey:@"EndTime"];
    index = [checkPoints indexOfLastObjectNotAfterValue:startTime];

    // let's remove some old stuff (should have some more intelligence in this!)
    index2 = index - 5;
    if (index2 > 0)
        [encapsulator removeObjectsBeforeTime:
            [[checkPoints objectAtIndex: index2] time]];

    if (index != NSNotFound) {
        checkPoint = [checkPoints objectAtIndex:index];
        if (([startTime earlierThanDate: [encapsulator startTimeInMemory]])
            || ([[checkPoint time] laterThanDate:[reader currentTime]]))
            [self gotoCheckPoint:checkPoint];
    }
//#endif
    
//    NSLog(@"traceFault: will read, starting at %@", [reader currentTime]);
//    [reader readUntilTime:endTime];
    endOfFile = NO;
    do {
        NSAutoreleasePool *pool;
        pool = [[NSAutoreleasePool alloc] init];
        for (i=0; i<2000 && !endOfFile; i++) {
            endOfFile = ![reader readNextEvent];
            if (((i+1) % 50) == 49) {
                [pool release];
                pool = [[NSAutoreleasePool alloc] init];
            }
        }
        [pool release];
        if (!endOfFile)
            [self writeCheckPoint];
    } while (!endOfFile && [[reader currentTime] earlierThanDate:endTime]);
//    NSLog(@"read until %@ (asked for %@)", [reader currentTime], endTime);
}

- (void)filterMenuSelected:(id)sender
{
    [filtersPopUp selectItemAtIndex:[sender tag]];
    [self filterChanged:self];
}

- (void)filterChanged:(id)sender
{
    int selected = [filtersPopUp indexOfSelectedItem];
    NSBox *newView = [filterViews objectAtIndex:selected];
    id delegate = [filterDelegates objectAtIndex:selected];

    if ([filtersDummyView contentView] == newView) return;
    if ([delegate respondsToSelector:@selector(viewWillBeSelected:)])
        [delegate viewWillBeSelected:newView];
#ifdef GNUSTEP
    [filtersPopUp setEnabled:YES];
#endif
    [filtersDummyView setContentView:newView];
    [filtersWindow setDelegate:delegate];
    [filtersWindow makeKeyAndOrderFront:self];
}

- (void)registerFilter:(NSNotification *)notification
{
    NSDictionary *info = [notification userInfo];
    NSString *filterName = [info objectForKey:@"Name"];
    NSView *filterView = [info objectForKey:@"View"];
    NSString *keyEquivalent = [info objectForKey:@"KeyEquivalent"];
    id delegate = [info objectForKey:@"Delegate"];
    id<NSMenuItem> menuItem;
    NSString *menuTitle;

    if (!keyEquivalent) keyEquivalent = @"";
    if (!delegate) delegate = self;
    
    [filterViews addObject:filterView];
    [filterDelegates addObject:delegate];
    [filtersPopUp addItemWithTitle:filterName];
    menuTitle = [filterName stringByAppendingString:@"..."];
    menuItem = [filtersMenu addItemWithTitle:menuTitle
                                      action:@selector(filterMenuSelected:)
                               keyEquivalent:keyEquivalent];
    [menuItem setTag:[filterViews count]-1];
    [filtersMenu sizeToFit];
[self filterMenuSelected:menuItem];
}

- (void)registerTool:(NSNotification *)notification
{
    NSDictionary *info = [notification userInfo];
    NSString *toolName = [info objectForKey:@"Name"];
    NSString *tool = [info objectForKey:@"Tool"];

    [[toolsMenu addItemWithTitle:[toolName stringByAppendingString:@"..."]
                          action:@selector(activateTool:)
                   keyEquivalent:@""] setTarget:tool];
    [toolsMenu sizeToFit];
}
@end
