#import "ProcRuler.h"

@implementation ProcRuler
- (id)initWithScrollView:(NSScrollView *)scrollView
{
    self = [super initWithScrollView:scrollView orientation:NSVerticalRuler];
    [self setRuleThickness:100];
    return self;
}

- (void)dealloc
{
    [super dealloc];
}

- (void)drawHashMarksAndLabelsInRect:(NSRect)drawRect
{
    DrawView *drawView = [self clientView];
    MiscTree *tree;
    float width;
    float offset;
    
    NSAssert([drawView isKindOfClass:[DrawView class]],
             @"ProcRuler's clientView is not a DrawView");

    tree = [drawView hierarchyTree];
    width = [self ruleThickness] / ([tree treeHeight] - 4/*2@@@*/);
    offset = NSMinY([drawView visibleRect]);
//    [self drawTree:tree inRect:drawRect level:0 width:width offset:offset];
    [self drawInstance:[drawView rootInstance]
                ofTree:tree
                inRect:drawRect
                 level:0
                 width:width
                offset:offset];
//    edescEnum = [[drawView verticalRulerDescriptors] objectEnumerator];
//    while (edesc = [edescEnum nextObject])
//        [self drawInstancesOfDescriptor:edesc];

}

- (void)drawTree:(MiscTree *)tree
          inRect:(NSRect)drawRect
           level:(int)level
           width:(float)width
          offset:(float)offset
{
    STEntityDescriptor * edesc;
    NSEnumerator *subtreeenum;
    MiscTree *subtree;

    if ([tree isLeaf])
        return;

    edesc = [(DrawView *)[self clientView] descriptorForEntityType:[tree entityType]];
    [self drawInstancesOfDescriptor:edesc inRect:drawRect
                              level:level width:width offset:offset];
    subtreeenum = [[tree subtrees] objectEnumerator];
    while (subtree = [subtreeenum nextObject])
        [self drawTree:subtree inRect:drawRect
                 level:level+1 width:width offset:offset];
}


- (void)drawInstancesOfDescriptor:(STEntityDescriptor *)edesc
                           inRect:(NSRect)drawRect
                            level:(int)level
                            width:(float)width
                           offset:(float)offset
{
    NSCell *cell = [[NSCell alloc] initTextCell:@""];
    NSEnumerator *ienum;
    id instance;
    
    [cell setBordered:YES];
    [cell setFont:[NSFont systemFontOfSize:8.0]];
//    [cell setAlignment:NSCenterTextAlignment];
    
    // check all instances on this descriptor
    ienum = [edesc instanceEnumerator];
    while (instance = [ienum nextObject]) {
        NSString *name = [instance description];
        NSRect r = [edesc rectForInstance:instance];

        r.origin.x = -1 + width * (level - 2/*@@@*/);
        r.origin.y -= offset;
        r.size.width = width+1;
        r.size.height += 1;
        if (1||/*@@@*/!NSIsEmptyRect(NSIntersectionRect(r, drawRect))) {
            [cell setStringValue:name];
            [cell drawWithFrame:r inView:self];
        }
    }
    
    [cell release];
}


- (void)drawInstance:(id)instance
              ofTree:(MiscTree *)tree
              inRect:(NSRect)drawRect
               level:(int)level
               width:(float)width
              offset:(float)offset
{
    NSEnumerator *subtreeEnum;
    MiscTree *subtree;

    subtreeEnum = [[tree subtrees] objectEnumerator];
    while (subtree = [subtreeEnum nextObject]) {
        [self drawContainer:instance
                     ofTree:subtree
                     inRect:drawRect
                      level:level
                      width:width
                     offset:offset];
    }
}

- (void)drawContainer:(PajeContainer *)container
               ofTree:(MiscTree *)tree
               inRect:(NSRect)drawRect
                level:(int)level
                width:(float)width
               offset:(float)offset
{
    id entityType;
    STEntityDescriptor *edesc;
    NSRect r;
    
    if ([tree isLeaf])
        return;

    entityType = [tree entityType];
    edesc = [(DrawView *)[self clientView] descriptorForEntityType:entityType];

    r = [edesc rectInContainer:container];

    r.origin.x = -1 + width * (level - 2/*@@@*/);
    r.origin.y -= offset;
    r.size.width = width+1;
    r.size.height += 1;
    if (1||/*@@@*/!NSIsEmptyRect(NSIntersectionRect(r, drawRect))) {
        NSCell *cell = [[NSCell alloc] initTextCell:@""];
        NSEnumerator *ienum;
        id instance;

        [cell setBordered:YES];
        /*
        [cell setFont:[NSFont systemFontOfSize:12.0]];
        {
            NSAttributedString *s;
            s = [[NSAttributedString alloc] initWithString:[entityType description] attributes:[NSDictionary dictionaryWithObjectsAndKeys:[NSColor redColor], NSForegroundColorAttributeName, [NSFont systemFontOfSize:12], NSFontAttributeName, nil]];
            [cell setAttributedStringValue:s];
        }
        [cell drawWithFrame:r inView:self];
         */
        [cell setFont:[NSFont systemFontOfSize:8.0]];

        // check all instances on this hierarchy
        ienum = [[(DrawView *)[self clientView] filter] enumeratorOfEntityType:entityType
                                                       inContainer:container];
        while (instance = [ienum nextObject]) {
            r = [edesc rectForInstance:instance];

            r.origin.x = -1 + width * (level - 2/*@@@*/);
            r.origin.y -= offset;
            r.size.width = width+1;
            r.size.height += 1;
            if (1||/*@@@*/!NSIsEmptyRect(NSIntersectionRect(r, drawRect))) {
                [cell setStringValue:[instance description]];
                [cell drawWithFrame:r inView:self];

                [self drawInstance:instance
                            ofTree:tree
                            inRect:drawRect
                             level:level+1
                             width:width
                            offset:offset];
            }
        }
        [cell release];
    }
}

@end
