// HierarchyDescriptor
//
// contains the hierarchy and subhierarchies of entities, like:
// ()
//   (Node)
//         (Node,Thread)
//                      [ThreadState]
//                      [MutexThreadState]
//                      [Event]
//                      [UserThreadState]
//         [NodeLink]
//         [ActivityState]
//         (Node,Semaphore)
//                      [SemaphoreState]
//         (Node,Mutex)
//                      [MutexState]
//         (Node,User)
//                      [UserNodeState]
//   [Communication]
//   (GlobalUser)
//         [UserGlobalState]
//
// where (x) is -hierarchy, [x] is a EntityDescriptor and indentation
// means -subhierarchy.
//

#import <Foundation/Foundation.h>
#import "EntityDescriptor.h"

@interface HierarchyDescriptor : NSObject
{
    NSArray *hierarchy;
    NSMutableArray *subhierarchy;

    NSMutableDictionary *containerToRect;

    float offset;     // number of pixels to skip before this hierarchy
    float separation; // number of pixels between two of this
}

+ (HierarchyDescriptor *)descriptorWithHierarchy:(NSArray *)hier;
- (id)initWithHierarchy:(NSArray *)hier;
- (void)dealloc;

- (NSArray *)hierarchy;
- (NSEnumerator *)containerEnumerator;
- (NSEnumerator *)subhierarchyEnumerator;

- (HierarchyDescriptor *)descriptorForKey:(id)key;
- (EntityDescriptor *)descriptorForEntityType:(id)entityType;

- (HierarchyDescriptor *)descriptorForHierarchy:(NSArray *)newHierarchy;
- (void)addSubhierarchy:(NSArray *)newHierarchy;
- (void)addEntityDescriptor:(EntityDescriptor *)entityDescriptor;

- (void)setRect:(NSRect)rect forContainer:(id)container;
- (NSRect)rectInContainer:(id)container;

- (float)offset;
- (void)setOffset:(float)value;
- (float)separation;
- (void)setSeparation:(float)value;
@end
