
#import "HierarchyDescriptor.h"

@implementation HierarchyDescriptor

+ (HierarchyDescriptor *)descriptorWithHierarchy:(NSArray *)hier
{
    return [[[self alloc] initWithHierarchy:hier] autorelease];
}

- (id)initWithHierarchy:(NSArray *)hier
{
    self = [super init];
    if (self) {
        hierarchy = [hier retain];
        subhierarchy = [[NSMutableArray alloc] init];
        containerToRect = [[NSMutableDictionary alloc] init];
        [self setOffset:[[NSUserDefaults standardUserDefaults] floatForKey:[[[self hierarchy] description] stringByAppendingString:@" Offset"]]];
        [self setSeparation:[[NSUserDefaults standardUserDefaults] floatForKey:[[[self hierarchy] description] stringByAppendingString:@" Separation"]]];
    }
    return self;
}

- (void)dealloc
{
    [hierarchy release];
    [subhierarchy release];
    [super dealloc];
}

- (NSArray *)hierarchy { return hierarchy; }
- (NSEnumerator *)containerEnumerator { return [containerToRect keyEnumerator]; }
- (NSEnumerator *)subhierarchyEnumerator {
    return [subhierarchy objectEnumerator];
}

- (HierarchyDescriptor *)descriptorForKey:(id)key
{
    NSEnumerator *subEnum = [self subhierarchyEnumerator];
    HierarchyDescriptor *subdescriptor;
    while (subdescriptor = [subEnum nextObject]) {
        if ([subdescriptor respondsToSelector:@selector(hierarchy)]) {
            if ([[[subdescriptor hierarchy] lastObject] isEqual:key])
                return subdescriptor;
        }
    }
    return nil;
}

- (EntityDescriptor *)descriptorForEntityType:(id)entityType
{
    NSEnumerator *subEnum = [self subhierarchyEnumerator];
    EntityDescriptor *subdescriptor;
    while (subdescriptor = [subEnum nextObject]) {
        if ([subdescriptor respondsToSelector:@selector(entityType)]) {
            if ([[subdescriptor entityType] isEqual:entityType])
                return subdescriptor;
        }
    }
    return nil;
}

- (HierarchyDescriptor *)descriptorForHierarchy:(NSArray *)newHierarchy
{
    int count = [newHierarchy count];
    if (count > 0) {
        id key = [newHierarchy objectAtIndex:0];
        HierarchyDescriptor *subdescriptor = [self descriptorForKey:key];
        if (!subdescriptor) {
            return nil;
        }
        if (count > 1) {
            return [subdescriptor descriptorForHierarchy:[newHierarchy subarrayWithRange:NSMakeRange(1, count-1)]];
        }
        return subdescriptor;
    }
    return self;
}

- (void)addSubhierarchy:(NSArray *)newHierarchy
{
    int count = [newHierarchy count];
    if (count > 0) {
        id key = [newHierarchy objectAtIndex:0];
        HierarchyDescriptor *subdescriptor = [self descriptorForKey:key];
        if (!subdescriptor) {
            subdescriptor = [HierarchyDescriptor descriptorWithHierarchy:[[self hierarchy] arrayByAddingObject:key]];
            [subhierarchy addObject:subdescriptor];
        }
        if (count > 1) {
            [subdescriptor addSubhierarchy:[newHierarchy subarrayWithRange:NSMakeRange(1, count-1)]];
        }
    }
}

- (void)addEntityDescriptor:(EntityDescriptor *)entityDescriptor
{
//    NSLog(@"adding %@ in hierarchy %@", entityDescriptor, hierarchy);
    [subhierarchy addObject:entityDescriptor];
}

// this is not the best plkace for this, but...
- (void)setRect:(NSRect)rect forContainer:(id)container
{
    [containerToRect setObject:[NSValue valueWithRect:rect]
                        forKey:container];
}

- (NSRect)rectInContainer:(id)container
{
    NSValue *value = [containerToRect objectForKey:container];
    if (value)
        return [value rectValue];
    else
        return NSZeroRect;
}

- (float)offset { return offset; }
- (void)setOffset:(float)value
{
    offset = value;
    [[NSUserDefaults standardUserDefaults] setFloat:value forKey:[[[self hierarchy] description] stringByAppendingString:@" Offset"]];
}
- (float)separation { return separation; }
- (void)setSeparation:(float)value;
{
    separation = value;
    [[NSUserDefaults standardUserDefaults] setFloat:value forKey:[[[self hierarchy] description] stringByAppendingString:@" Separation"]];
}
@end
