#import "NodeDescriptor.h"

// 18.apr.1998 BS  creation

@implementation NodeDescriptor

- (id)initWithNode:(id)n
{
    self = [super init];
    if (self) {
        rect = NSZeroRect;
        node = [n retain];
        rects = [[NSMutableArray alloc] init];
        entityOffsets = [[NSMutableDictionary alloc] init];
        entitySlotHeights = [[NSMutableDictionary alloc] init];
        heights = [[NSMutableArray alloc] init];
        entityTypes = [[NSMutableArray alloc] init];
    }
    return self;
}

- (void)dealloc
{
    [node release];
    [rects release];
    [entityOffsets release];
    [entitySlotHeights release];
    [heights release];
    [entityTypes release];
    [super dealloc];
}

- (id)node                          { return node; }
- (void)setRect:(NSRect)r           { rect = r; }
- (NSRect)rect                      { return rect; }

- (void)setOffset:(float)offset forEntityType:(id)entityType
{
    [entityOffsets setObject:[NSNumber numberWithFloat:offset]
                      forKey:entityType];
}
- (float)offsetForEntityType:(id)entityType
{
    NSNumber *offset = [entityOffsets objectForKey:entityType];
    NSAssert2(offset, @"offset for unknown entity type %@ in node %@",
             entityType, node);
    return [offset floatValue];
}
- (void)setSlotHeight:(float)height forEntityType:(id)entityType
{
    [entitySlotHeights setObject:[NSNumber numberWithFloat:height]
                      forKey:entityType];
}
- (float)slotHeightForEntityType:(id)entityType
{
    NSNumber *height = [entitySlotHeights objectForKey:entityType];
    NSAssert2(height, @"height for unknown entity type %@ in node %@",
             entityType, node);
    return [height floatValue];
}

- (BOOL)hasEntitiesOfType:(id)entityType
{
    return ([entityOffsets objectForKey:entityType] != nil);
}

- (unsigned)count
{
    unsigned ct = [rects count];
    NSAssert3(((ct == [heights count]) && (ct == [entityTypes count])),
              @"Internal error: counts don't match in NodeDescriptor (rects=%d, heights=%d, entityTypes=%d)", ct, [heights count], [entityTypes count]);
    return ct;
}

- (void)addSubdivision
{
    [rects addObject:[NSValue valueWithRect:NSZeroRect]];
    [heights addObject:[NSNumber numberWithFloat:0]];
    [entityTypes addObject:[NSMutableArray array]];
}

- (float)offsetAtIndex:(unsigned)index
{
    return NSMinY([[rects objectAtIndex:index] rectValue]);
}

- (void)setRect:(NSRect)r atIndex:(unsigned)index
{
    [rects replaceObjectAtIndex:index
                     withObject:[NSValue valueWithRect:r]];
}
- (NSRect)rectAtIndex:(unsigned)index
{
    return [[rects objectAtIndex:index] rectValue];
}

- (void)setSlotHeight:(float)height atIndex:(unsigned)index
{
    [heights replaceObjectAtIndex:index
                       withObject:[NSNumber numberWithFloat:height]];
}

- (float)slotHeightAtIndex:(unsigned)index
{
    return [[heights objectAtIndex:index] floatValue];
}

- (void)addEntityType:(id)type atIndex:(unsigned)index
{
    [[entityTypes objectAtIndex:index] addObject:type];
}

- (NSArray *)entityTypesAtIndex:(unsigned)index
{
    return [entityTypes objectAtIndex:index];
}

- (NSString *)description
{
    return [node description];
}
@end