#import "STEntityDescriptor.h"
#import "../General.bproj/NSUserDefaults+Colors.h"
#import <AppKit/AppKit.h>
#if defined(GNUSTEP)
#import <AppKit/PSOperators.h>
#endif
#if defined(__APPLE__)
#import "PSOperators.h"
#endif
#if defined(NeXT)
#import <AppKit/psopsOpenStep.h>	// For PS and DPS function prototypes
#import "drawline.h" // for pswraps
#endif
#include <math.h>

// 18.apr.1998 BS  creation


NSDictionary *drawFunctionsDictionary;
NSDictionary *shapeFunctionsDictionary;

void emptydrawfunction(void) {}
void emptyshapefunction(float x, float y, float w, float h)
{
    NSRectFill(NSMakeRect(x, y, w, h));
}

void PSOut(float x, float y, float w, float h)
{
    PSsetlinewidth(2);
    PSmoveto(x+h, y);
    PSarc(x, y, h, 0, 360);
    PSmoveto(x, y);
    PSarc(x, y, 1, 0, 360);
}

void PSIn(float x, float y, float w, float h)
{
    PSsetlinewidth(2);
    PSmoveto(x+h, y);
    PSarc(x, y, h, 0, 360);
    PSmoveto(x, y - h/2);
    PSrlineto(0, h);
    PSmoveto(x - h/2, y);
    PSrlineto(h, 0);
}


#if defined(GNUSTEP) || defined(__APPLE__)
void PSRect(float x, float y, float w, float h)
{
//    PSnewpath();
    PSmoveto(x, y);
    PSrlineto(w, 0);
    PSrlineto(0, h);
    PSrlineto(-w, 0);
//    PSclosepath();
PSrlineto(0, -h);
}

void PSLine(float x, float y, float w, float h)
{
    PSmoveto(x, y);
    PSrlineto(w, h);
}

void PSTriangle(float x, float y, float w, float h)
{
//    PSnewpath();
    PSmoveto(x + w/2, y);
    PSrlineto(w/2, h);
    PSrlineto(-w, 0);
//    PSclosepath();
PSrlineto(w/2, -h);
}
//#endif

void PSArrow(float x, float y, float w, float h)
{
    float len;
    float ang;
    float bx, by;
    
    len = sqrt(w*w + h*h);
    if (len) {
        ang = atan2(h, w) * 180 / M_PI;
        PSnewpath();
        PSmoveto(x, y);
        PSrotate(ang);
        PScurrentpoint(&bx, &by);
        PSrmoveto(len - 7, 0);
        PSrlineto(0, 2.5);
        PSrlineto(7, -2.5);
        PSrlineto(-7, -2.5);
        PSclosepath();
        PSarc(bx, by, 1.5, 0, 360);
    }
}
void PSArrow2(float x, float y, float w, float h)
{
    float len;
    float ang;
    float bx, by;

    len = sqrt(w*w + h*h);
    if (len) {
        ang = atan2(h, w) * 180 / M_PI;
        PSmoveto(x, y);
        PSrotate(ang);
        PScurrentpoint(&bx, &by);
        PSrlineto(0, 2);
        PSrlineto(0, -4);
        PSrlineto(0, 2);	
        PSrlineto(len - 7, 0);
        PSrlineto(0, 2.5);
        PSrlineto(7, -2.5);
        PSrlineto(-7, -2.5);
        PSrlineto(0, 2.5);
    }
}
//#if defined(GNUSTEP) || defined(__APPLE__)

void PSPin(float x, float y, float w, float h)
{
    PSmoveto(x + w/2, y);
    PSarc(x + w/2, y + h - w/2, w/2, -90, 270);
}


void PSDashedStroke(void)
{
    float dash[6] = {6, 3, 3, 3, 1, 6};
    PSsetlinewidth(2);
    PSsetdash(dash, 6, 0);
    PSstroke();
}

void PSFillAndFrameBlack(void)
{
    PSgsave();
    PSfill();
    PSgrestore();
    PSsetgray(0);
    PSstroke();
}


void PSFillAndFrameWhite(void)
{
    PSgsave();
    PSfill();
    PSgrestore();
    PSsetgray(1);
    PSstroke();
}


void PSFrameWhite(void)
{
    PSsetgray(1);
    PSstroke();
}


void PSFillAndDashedStrokeBlack(void)
{
    float dash[6] = {6, 3, 3, 3, 1, 6};
    PSgsave();
    PSfill();
    PSgrestore();
    PSsetgray(0);
    PSsetlinewidth(2);
    PSsetdash(dash, 6, 0);
    PSstroke();
}


            
#endif


@implementation STEntityDescriptor
+ (void)initialize
{
    if (self == [STEntityDescriptor class]) {
        shapeFunctionsDictionary = [[NSDictionary alloc] initWithObjectsAndKeys:
            [NSValue valueWithPointer:PSTriangle], @"PSTriangle",
            [NSValue valueWithPointer:PSPin],      @"PSPin",
            [NSValue valueWithPointer:PSIn],      @"PSIn",
            [NSValue valueWithPointer:PSOut],      @"PSOut",
            [NSValue valueWithPointer:PSRect],     @"PSRect",
            [NSValue valueWithPointer:PSArrow],    @"PSArrow",
#if defined(GNUSTEP) || defined(__APPLE__)
            [NSValue valueWithPointer:PSArrow2],   @"PSArrow2",
//            [NSValue valueWithPointer:PSArrow3],   @"PSArrow3",
#endif
            [NSValue valueWithPointer:PSLine],     @"PSLine",
            nil];
        drawFunctionsDictionary = [[NSDictionary alloc] initWithObjectsAndKeys:
            [NSValue valueWithPointer:PSfill],              @"PSfill",
            [NSValue valueWithPointer:PSstroke],            @"PSstroke",
            [NSValue valueWithPointer:PSDashedStroke],      @"PSDashedStroke",
            [NSValue valueWithPointer:PSFillAndFrameBlack], @"PSFillAndFrameBlack",
            [NSValue valueWithPointer:PSFillAndFrameWhite], @"PSFillAndFrameWhite",
            [NSValue valueWithPointer:PSFrameWhite],        @"PSFrameWhite",
            [NSValue valueWithPointer:PSFillAndDashedStrokeBlack], @"PSFillAndDashedStrokeBlack",
            nil];
    }
}

+ (drawfunction *)drawFunctionForName:(NSString*)name;
{
    NSValue *v = nil;
    drawfunction *f;

    if (name != nil)
        v = [drawFunctionsDictionary objectForKey:name];
    if (v != nil)
        f = (drawfunction *)[v pointerValue];
    else {
//        NSLog(@"Draw function named %@ not found. Known are: %@",
//              name, drawFunctionsDictionary);
        f = emptydrawfunction;
    }

    return f;
}

+ (pathfunction *)shapeFunctionForName:(NSString*)name;
{
    NSValue *v = nil;
    pathfunction *f;

    if (name != nil)
        v = [shapeFunctionsDictionary objectForKey:name];
    if (v != nil)
        f = (pathfunction *)[v pointerValue];
    else {
//        NSLog(@"Shape function named %@ not found. Known are: %@",
//              name, shapeFunctionsDictionary);
        f = emptyshapefunction;
    }

    return f;
}

+ (NSString *)nameForDrawFunction:(drawfunction *)f;
{
    NSValue *v = [NSValue valueWithPointer:f];
    NSArray *names = [drawFunctionsDictionary allKeysForObject:v];
    if (names && ([names count] > 0))
        return [names objectAtIndex:0];
    return nil;
}

+ (NSString *)nameForShapeFunction:(pathfunction *)f;
{
    NSValue *v = [NSValue valueWithPointer:f];
    NSArray *names = [shapeFunctionsDictionary allKeysForObject:v];
    if (names && ([names count] > 0))
        return [names objectAtIndex:0];
    return nil;
}

+ (NSEnumerator *)shapeFunctionsValueEnumerator;
{
    return [shapeFunctionsDictionary objectEnumerator];
}

+ (NSEnumerator *)drawFunctionsValueEnumerator;
{
    return [drawFunctionsDictionary objectEnumerator];
}


+ (STEntityDescriptor *)descriptorWithEntityType:(PajeEntityType *)etype
                                     drawingType:(PajeDrawingType)dtype
{
    return [[[self alloc] initWithEntityType:etype
                                 drawingType:dtype] autorelease];
}

- (id)initWithEntityType:(PajeEntityType *)etype
             drawingType:(PajeDrawingType)dtype
{
    self = [super init];
    if (self) {
        id tmp;
        containerToRect = [[NSMutableDictionary alloc] init];
        instanceToRect = [[NSMutableDictionary alloc] init];
        entityType = [etype retain];
        drawingType = dtype;
        canDisplay = ![[NSUserDefaults standardUserDefaults] boolForKey:[[entityType description] stringByAppendingString:@" Do Not Show"]];
        drawingRect = [[NSUserDefaults standardUserDefaults] rectForKey:[[entityType description] stringByAppendingString:@" Drawing Rect"]];
        pathFunction = [[self class] shapeFunctionForName:[[NSUserDefaults standardUserDefaults] stringForKey:[[entityType description] stringByAppendingString:@" Path Function"]]];
        drawFunction = [[self class] drawFunctionForName:[[NSUserDefaults standardUserDefaults] stringForKey:[[entityType description] stringByAppendingString:@" Draw Function"]]];
        highlightFunction = [[self class] drawFunctionForName:[[NSUserDefaults standardUserDefaults] stringForKey:[[entityType description] stringByAppendingString:@" Highlight Function"]]];
        tmp = [[NSUserDefaults standardUserDefaults] stringForKey:[[entityType description] stringByAppendingString:@" Inset"]];
        insetAmount = tmp ? [tmp floatValue] : 2;
        switch (drawingType) {
            case PajeEventDrawingType:
                if (NSEqualRects(NSZeroRect, drawingRect))
                    drawingRect = NSMakeRect(-2, -4, 4, 6);
                if (!pathFunction || (pathFunction == emptyshapefunction))
                    pathFunction = [[self class] shapeFunctionForName:@"PSTriangle"];
                if (!drawFunction || (drawFunction == emptydrawfunction))
                    drawFunction = [[self class] drawFunctionForName:@"PSFillAndFrameBlack"];
                if (!highlightFunction || (highlightFunction == emptydrawfunction))
                    highlightFunction = [[self class] drawFunctionForName:@"PSFillAndFrameWhite"];
                break;
            case PajeStateDrawingType:
                if (NSEqualRects(NSZeroRect, drawingRect))
//[self setDrawingRect: NSMakeRect(0, 0, 0, 6)];
                    drawingRect = NSMakeRect(0, 0, 0, 6);
                if (!pathFunction || (pathFunction == emptyshapefunction))
                    pathFunction = [[self class] shapeFunctionForName:@"PSRect"];
                if (!drawFunction || (drawFunction == emptydrawfunction))
                    drawFunction = [[self class] drawFunctionForName:@"PSFillAndFrameBlack"];
                if (!highlightFunction || (highlightFunction == emptydrawfunction))
                    highlightFunction = [[self class] drawFunctionForName:@"PSFillAndFrameWhite"];
                break;
            case PajeVariableDrawingType:
                if (NSEqualRects(NSZeroRect, drawingRect))
//[self setDrawingRect: NSMakeRect(0, 0, 0, 40)];
                    drawingRect = NSMakeRect(0, 0, 0, 40);
                if (!pathFunction || (pathFunction == emptyshapefunction))
                    pathFunction = [[self class] shapeFunctionForName:@"PSRect"];
                if (!drawFunction || (drawFunction == emptydrawfunction))
                    drawFunction = [[self class] drawFunctionForName:@"PSFillAndFrameBlack"];
                if (!highlightFunction || (highlightFunction == emptydrawfunction))
                    highlightFunction = [[self class] drawFunctionForName:@"PSFillAndFrameWhite"];
                break;
            case PajeLinkDrawingType:
//                drawingRect = NSMakeRect(0, 0, 0, 0);
                if (!pathFunction || (pathFunction == emptyshapefunction))
                    pathFunction = [[self class] shapeFunctionForName:@"PSLine"];
                if (!drawFunction || (drawFunction == emptydrawfunction))
                    drawFunction = [[self class] drawFunctionForName:@"PSstroke"];
                if (!highlightFunction || (highlightFunction == emptydrawfunction))
                    highlightFunction = [[self class] drawFunctionForName:@"PSFillAndFrameWhite"];
                break;
            case PajeContainerDrawingType:
                //if (NSEqualRects(NSZeroRect, drawingRect))
                //    drawingRect = NSMakeRect(0, 0, 0, 6);
                if (!pathFunction || (pathFunction == emptyshapefunction))
                    pathFunction = [[self class] shapeFunctionForName:@"PSRect"];
                if (!drawFunction || (drawFunction == emptydrawfunction))
                    drawFunction = [[self class] drawFunctionForName:@"PSfill"];
                if (!highlightFunction || (highlightFunction == emptydrawfunction))
                    highlightFunction = [[self class] drawFunctionForName:@"PSFillAndFrameWhite"];
                break;
            default:
                NSAssert1(0, @"Invalid drawing type %d", drawingType);
        }
    }
    return self;
}

- (void)dealloc
{
    [entityType release];
    [containerToRect release];
    [instanceToRect release];
    [super dealloc];
}

// forget everything
- (void)reset
{
    [containerToRect removeAllObjects];
    [instanceToRect removeAllObjects];
}


// accessor methods
- (PajeEntityType *)entityType                  { return entityType; }
- (PajeDrawingType)drawingType                  { return drawingType; }
- (void)setDrawingRect:(NSRect)r
{    
    drawingRect = r;
    [[NSUserDefaults standardUserDefaults] setRect:r forKey:[[entityType description] stringByAppendingString:@" Drawing Rect"]];
}
- (NSRect)drawingRect                           { return drawingRect; }
- (void)setPathFunction:(pathfunction *)f
{
    pathFunction = f;
    [[NSUserDefaults standardUserDefaults] setObject:[[self class] nameForShapeFunction:f] forKey:[[entityType description] stringByAppendingString:@" Path Function"]];
}
- (void)setDrawFunction:(drawfunction *)f
{
    drawFunction = f;
    [[NSUserDefaults standardUserDefaults] setObject:[[self class] nameForDrawFunction:f] forKey:[[entityType description] stringByAppendingString:@" Draw Function"]];
}
- (void)setHighlightFunction:(drawfunction *)f
{
    highlightFunction = f;
    [[NSUserDefaults standardUserDefaults] setObject:[[self class] nameForDrawFunction:f] forKey:[[entityType description] stringByAppendingString:@" Highlight Function"]];
}
- (pathfunction *)pathFunction                  { return pathFunction; }
- (drawfunction *)drawFunction                  { return drawFunction; }
- (drawfunction *)highlightFunction             { return highlightFunction; }
- (BOOL)canDisplay                              { return canDisplay; }
- (void)setCanDisplay:(BOOL)flag
{
    canDisplay = flag;
    [[NSUserDefaults standardUserDefaults] setBool:!canDisplay forKey:[[entityType description] stringByAppendingString:@" Do Not Show"]];
}

- (float)insetAmount                            { return insetAmount; }


- (void)setRect:(NSRect)rect forContainer:(PajeContainer *)container
{
    [containerToRect setObject:[NSValue valueWithRect:rect]
                        forKey:container];
}

- (NSRect)rectInContainer:(PajeContainer *)container
{
    NSValue *value = [containerToRect objectForKey:container];
    if (value)
        return [value rectValue];
    else
        return NSZeroRect;
}

- (NSRect)rectForInstance:(id)entity
{
    NSValue *value = [instanceToRect objectForKey:entity];
    if (value)
        return [value rectValue];
    else
        return NSZeroRect;
}

- (void)setRect:(NSRect)rect forInstance:(id)entity
{
    [instanceToRect setObject:[NSValue valueWithRect:rect]
                        forKey:entity];
}

- (BOOL)isInstance:(id)entity inRect:(NSRect)rect
{
    return !NSIsEmptyRect(NSIntersectionRect(rect,
                                             [self rectForInstance:entity]));
}

- (BOOL)isContainer:(PajeContainer *)container inRect:(NSRect)rect
{
    return !NSIsEmptyRect(NSIntersectionRect(rect,
                                             [self rectInContainer:container]));
}

- (BOOL)isPoint:(NSPoint)point inContainer:(PajeContainer *)container
{
    return NSPointInRect(point, [self rectInContainer:container]);
}
- (BOOL)isPoint:(NSPoint)point inInstance:(id)entity
{
    return NSPointInRect(point, [self rectForInstance:entity]);
}

- (float)offset { return NSMinY(drawingRect); }
- (void)setOffset:(float)value
{
    drawingRect.origin.y = value;
    [self setDrawingRect:drawingRect];
}
- (float)separation { return NSHeight(drawingRect); }
- (void)setSeparation:(float)value
{
    drawingRect.size.height = value;
    [self setDrawingRect:drawingRect];
}

- (NSEnumerator *)instanceEnumerator
{
    return [instanceToRect keyEnumerator];
}

// from NSObject protocol

- (unsigned int)hash
{
    return [entityType hash];
}
- (BOOL)isEqual:(id)other
{
    if (other == self || other == entityType) return YES;
    return [[entityType description] isEqual:[other description]];
}
- (NSString *)description
{
    return [entityType description];
}
@end
