

#import "MultiContainer.h"
#import "../General.bproj/FoundationAdditions.h"
#import "../General.bproj/PajeType.h"

@implementation MultiContainer

+ (MultiContainer *)container
{
    return [[[self alloc] init] autorelease];
}

- (id)init
{
    self = [super init];
    if (self) {
        containers = [[NSMutableArray alloc] init];
    }
    return self;
}

- (void)dealloc
{
    [containers release];
    [super dealloc];
}

- (void)addContainer:(PajeContainer *)container
{
    [containers addObject:container];
}

- (NSEnumerator *)containerEnumerator
{
    return [containers objectEnumerator];
}

- (BOOL)conflictsWithContainer:(PajeContainer *)container
{
    NSEnumerator *compEnum = [self containerEnumerator];
    id internalContainer;
    while (internalContainer = [compEnum nextObject]) {
        if (!([[container endTime] earlierThanDate:[internalContainer startTime]]
              || [[internalContainer endTime] earlierThanDate:[container startTime]]))
            return YES;
    }
    return NO;
}


- (NSDate *)startTime
{
    if ([containers count] > 0)
        return [[containers objectAtIndex:0] startTime];
    else
        return [NSDate distantPast];
}

- (NSDate *)endTime
{
    int ct = [containers count];
    if (ct > 0)
        return [[containers objectAtIndex:ct-1] endTime];
    else
        return [NSDate distantFuture];
}

- (unsigned int)hash
{
    return [containers hash];
}

- (NSArray *)_containers
{
    return containers;
}

- (BOOL)isEqual:(id)other
{
    if ([other isKindOfClass:[MultiContainer class]])
        return [containers isEqual:[(MultiContainer *)other _containers]];
    return NO;
}

- (id)copyWithZone:(NSZone *)zone
{
    return [self retain];
}

- (NSString *)xdescription
{
    return [containers description];
}
- (NSString *)description
{
    int count, n;
    NSMutableString *s;
    count = [containers count];
    if (count == 0)
        return @"";
    s = [NSMutableString stringWithString:[[containers objectAtIndex:0] description]];
    for (n = 1; n < count; n++) {
        [s appendString:@","];
        [s appendString:[[containers objectAtIndex:n] description]];
    }
    return s;
}
- (PajeContainer *)container
{
    return [[containers objectAtIndex:0] container];
}

- (PajeEntityType *)entityType
{
    return [[containers objectAtIndex:0] entityType];
}
@end
