/* VTNode.m created by benhur on Thu 21-Aug-1997 */

#import "VTNode.h"

@implementation VTNode

+ (VTNode *)node
{
    return [[[self alloc] init] autorelease];
}

- (id)init
{
    [super init];

    serviceTable = [[TranslationTable translationTable] retain];
    services = [[NSMutableArray array] retain];
    threadCount = 0;
    filteringMode = REUSE_SLOTS;

    return self;
}

- (void)dealloc
{
    [serviceTable release];
    [services release];
    [super dealloc];
}

- (filterType)filteringMode
{
    return filteringMode;
}

- (void)setFilteringMode:(filterType)mode
{
    filteringMode = mode;
}


- (NSString *)description
{
    return [NSString stringWithFormat:@"serviceTable=%@; services=%@;", serviceTable, services];
}

- (void)startThread:(id)threadId
            service:(id)service
{
    int vep;

    vep = [serviceTable addValue:service];
    while (vep >= [services count])
        [services addObject:[VTService serviceWithFirstSlot:threadCount]];
    if ([[services objectAtIndex:vep] startThread:threadId]) {
        int count = [services count];
        threadCount++;
        for (vep = vep+1; vep < count; vep++)
            [[services objectAtIndex:vep] incrementFirstSlot];
    }
}

- (void)endThread:(id)threadId
          service:(id)service
{
    int vep = [serviceTable indexOfValue:service];
    if (vep != NSNotFound)
        [[services objectAtIndex:vep] endThread:threadId];
    else
        NSLog(@"%@: ending non started thread:%@", NSStringFromClass([self class]), threadId); //*** error: ending unstarted thread
}

- (int)slotForThread:(id)threadId
             service:(id)service
{
    int vep;

    if (filteringMode == NO_FILTERING)
        return [threadId intValue];
    if (filteringMode == COLLAPSE_NODE)
        return 0;
    vep = [serviceTable indexOfValue:service];
    if (vep != NSNotFound) {
        switch (filteringMode) {
            case REUSE_SLOTS:
                return [[services objectAtIndex:vep] slotForThread:threadId];
            case COLLAPSE_SERVICE:
                return vep;
            default:
                NSLog(@"%@: Unknown filtering mode %d", NSStringFromClass([self class]), filteringMode);
        }
    } else {
        NSLog(@"%@: %@ unexistant service %@", NSStringFromClass([self class]), NSStringFromSelector(_cmd), service);
    }
    return 0;
}

- (int)threadCount
{
    switch (filteringMode) {
        case REUSE_SLOTS:
            return threadCount;
            break;
        case COLLAPSE_SERVICE:
            return [serviceTable count];
            break;
        case COLLAPSE_NODE:
            return 1;
            break;
        default:
            NSLog(@"%@: Unknown filtering mode %d", NSStringFromClass([self class]), filteringMode);
    }
    return 1;
}

@end
