/* VTService.m created by benhur on Thu 21-Aug-1997 */

#import "VTService.h"

@implementation VTService

+ (VTService *)serviceWithFirstSlot:(int)value
{
    return [[[self alloc] initWithFirstSlot:value] autorelease];
}

- (id)initWithFirstSlot:(int)value
{
    [super init];

    threadCount = 0;
    firstSlot = value;
    slotsInUse = [[NSMutableData data] retain];
    threadToSlot = [[NSMutableDictionary dictionary] retain];

    return self;
}

- (void)dealloc
{
    [slotsInUse release];
    [threadToSlot release];
    [super dealloc];
}

- (NSString *)description
{
    return [NSString stringWithFormat:@"threadToSlot=%@", threadToSlot];
}

- (unsigned int)getNextSlot
{
    int i, m;
    int bitCtr=0;
    int byteCount = [slotsInUse length];
    unsigned char *bytePtr = [slotsInUse mutableBytes];

    for (i = 0; i < byteCount; i++) {
        for (m=1; m<(1<<8); m<<=1) {
            if (!(*bytePtr & m)) {
                *bytePtr |= m;
                return bitCtr;
            }
            bitCtr++;
        }
        bytePtr++;
    }

    // all bits are in use; alloc another byte
    [slotsInUse increaseLengthBy:1];
    bytePtr = [slotsInUse mutableBytes];
    *(bytePtr + byteCount) = 1;
    return bitCtr;
}

- (void)freeSlot:(unsigned int)slot
{
    unsigned byteCtr = slot / 8;
    unsigned bitMask = 1 << (slot % 8);
    unsigned char *bytePtr = [slotsInUse mutableBytes];

    if (byteCtr < [slotsInUse length]) {
        *(bytePtr + byteCtr) &= ~bitMask;
    } else {
        [NSException raise:@"PajeInternalInconsistency"
                    format:@"%@: invalid slot to free:%d",
                           NSStringFromClass([self class]), slot];
    }
}


- (BOOL)startThread:(id)threadId
{
    int slot;

//NSLog(@"starting thread %@", threadId);
    if ([threadToSlot objectForKey:threadId] == nil) {
        slot = [self getNextSlot];
//NSLog(@"thread %@ in slot %d", threadId, slot);
        [threadToSlot setObject:[NSNumber numberWithInt:slot] forKey:threadId];
        if (slot >= threadCount) {
            threadCount++;
            return YES;
        }
    }
    return NO;
}

- (void)endThread:(id)threadId
{
    NSNumber *virtual;

    virtual = [threadToSlot objectForKey:threadId];
    if (virtual) {
        [self freeSlot:[virtual intValue]];
//        NSLog(@"VirtualThread: ending thread %@ (%@)", threadId, virtual);
    } else {
        NSLog(@"VirtualThread: ending unstarted thread %@", threadId); //*** error!
    }
}

- (void)incrementFirstSlot
{
    firstSlot++;
}

- (void)setFirstSlot:(int)value
{
    firstSlot = value;
}

- (int)slotForThread:(id)threadId
{
    NSNumber *virtual;

    virtual = [threadToSlot objectForKey:threadId];
    if (virtual) {
        return firstSlot + [virtual intValue];
    } else {
        NSLog(@"VirtualThread: unknown thread:%@. known are:%@", threadId, threadToSlot);
        return firstSlot;
    }
}
@end
