  /*
   * VirtualThread.m
   *
   * Component that re-numbers the threads in a trace, reusing
   * the numbers as the threads die.
   *
   * 19960514 BS  creation
   */

#import "VirtualThread.h"
#import "../General.bproj/FoundationAdditions.h"
#import "MultiContainer.h"
#import "../General.bproj/MultiEnumerator.h"


@implementation VirtualThread

// methods for reutilisation of thread slots on the graph,
// by changing thread numbers.

- (id)init
{
    [super init];

    // load the bundle. it initializes view.
    if (![NSBundle loadNibNamed:@"VirtualThread" owner:self])
        NSRunAlertPanel(NSStringFromClass([self class]),
                        @"Couldn't load interface file",
                        nil, nil, nil);

    defaultFilteringMode = [filteringModeSelector selectedTag];

    // view is an NSBox. we need its contents.
    view = [view contentView];
#ifndef GNUSTEP
    [[view window] autorelease];
#endif
    [view removeFromSuperview];

    [[NSNotificationCenter defaultCenter] postNotificationName:@"PajeRegisterFilterNotification" object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
                        NSStringFromClass([self class]), @"Name",
                        view, @"View", nil]];

    return self;
}

- (void)dealloc
{
    [entityTypeToContainers release];
    [containerToNewContainer release];
    [super dealloc];
}

- (void)reset
{
    [nodeSelector removeAllItems];
    [nodeSelector addItemWithTitle:@"All"];
    [filteringModeSelector selectCellWithTag:defaultFilteringMode];
}

- (void)changeSelectedNode:(id)sender
{
//    id nodeTitle = [sender titleOfSelectedItem];
    id node = nil;//[nodes objectForKey:nodeTitle];
    filterType filteringMode;

    if (!node)
        node = nil;//[nodes objectForKey:[NSNumber numberWithInt:[nodeTitle intValue]]];

    filteringMode = [node filteringMode];
    [filteringModeSelector selectCellWithTag:filteringMode];
}

BOOL filterYN;

- (void)changeFilteringMode:(id)sender
{
    id nodeTitle = [nodeSelector titleOfSelectedItem];
    id node;
    filterType filteringMode = [sender selectedTag];

    filterYN = (filteringMode != 0);
    // should only release the ones that have changed..
    [entityTypeToContainers release];
    [containerToNewContainer release];
    entityTypeToContainers = containerToNewContainer = nil;
    
    if ([nodeTitle isEqual:@"All"]) {
        id nodeEnum = nil;//[nodes objectEnumerator];
        defaultFilteringMode = filteringMode;
        while (node = [nodeEnum nextObject])
            [node setFilteringMode:filteringMode];
    } else {
        node = nil;//[nodes objectForKey:nodeTitle];
        if (!node)
            node = nil;//[nodes objectForKey:[NSNumber numberWithInt:[nodeTitle intValue]]];
        [node setFilteringMode:filteringMode];
    }

    // inform our successor that the trace has changed
    [super hierarchyChanged];
}



- (void)hierarchyChanged
{
    [entityTypeToContainers release];
    [containerToNewContainer release];
    entityTypeToContainers = containerToNewContainer = nil;

    [super hierarchyChanged];
}

- (NSEnumerator *)enumeratorOfEntityType:(PajeEntityType *)entityType
                             inContainer:(PajeContainer *)container
                                fromTime:(NSDate *)start
                                  toTime:(NSDate *)end
{
    if ([container isKindOfClass:[MultiContainer class]]) {
        MultiEnumerator *multiEnum = [MultiEnumerator enumerator];
        NSEnumerator *compEnum = [(MultiContainer *)container containerEnumerator];
        id compContainer;

        while (compContainer = [compEnum nextObject]) {
            NSEnumerator *origEnum;
            origEnum = [inputComponent enumeratorOfEntityType:entityType
                                                  inContainer:compContainer
                                                     fromTime:start
                                                       toTime:end];
            [multiEnum addEnumerator:origEnum];
        }
        return multiEnum;
    }
    return [inputComponent enumeratorOfEntityType:entityType
                                      inContainer:container
                                         fromTime:start
                                           toTime:end];
}

- (NSEnumerator *)unfilteredEnumeratorOfEntityType:(PajeEntityType *)entityType
                                       inContainer:(PajeContainer *)container
{
    if ([container isKindOfClass:[MultiContainer class]]) {
        MultiEnumerator *multiEnum = [MultiEnumerator enumerator];
        NSEnumerator *containerEnum = [(MultiContainer *)container containerEnumerator];
        id instance;

        while (instance = [containerEnum nextObject]) {
            NSEnumerator *origEnum;
            origEnum = [inputComponent enumeratorOfEntityType:entityType
                                                  inContainer:instance];
            [multiEnum addEnumerator:origEnum];
        }
        return multiEnum;        
    } else
        return [inputComponent enumeratorOfEntityType:entityType
                                          inContainer:container];
}

- (NSArray *)calculateInstancesOfEntityType:(PajeEntityType *)entityType
                                inContainer:(PajeContainer *)container
{
    NSMutableArray *containers = [NSMutableArray array];
    NSEnumerator *enumerator;
    id instance;

    enumerator = [self unfilteredEnumeratorOfEntityType:entityType
                                            inContainer:container];

    while (instance = [enumerator nextObject]) {
        NSEnumerator *newEnum = [containers objectEnumerator];
        MultiContainer *newContainer;

        while (newContainer = [newEnum nextObject]) {
//            if ([entityType isEqual:@"Node"] && ([[instance nodeId] intValue]&1 == [[[[newContainer _containers] objectAtIndex:0] nodeId] intValue]&1))
//                break;
            if (![newContainer conflictsWithContainer:instance])
                break;
        }
        if (nil == newContainer) {
            newContainer = [MultiContainer container];
            [containers addObject:newContainer];
        }
        [newContainer addContainer:instance];

        if (!containerToNewContainer)
            containerToNewContainer = [[NSMutableDictionary alloc] init];
        [containerToNewContainer setObject:newContainer forKey:instance];
    }
    return containers;
}

- (BOOL)isFilteringEntityType:(PajeEntityType *)entityType
                  inContainer:(PajeContainer *)container
{
    return filterYN;
}

- (NSEnumerator *)enumeratorOfEntityType:(PajeEntityType *)entityType
                             inContainer:(PajeContainer *)container
{
    NSMutableDictionary *containerToContained;
    NSArray *instances;

    if ([[container description] isEqual:@"(0,3)"])
        NSLog(@"(0,3)");
    
    if (![self isFilteringEntityType:entityType
                         inContainer:container])
        return [self unfilteredEnumeratorOfEntityType:entityType
                                          inContainer:container];

    if (nil == entityTypeToContainers)
        entityTypeToContainers = [[NSMutableDictionary alloc] init];

    containerToContained = [entityTypeToContainers objectForKey:entityType];
    if (nil == containerToContained) {
        containerToContained = [NSMutableDictionary dictionary];
        [entityTypeToContainers setObject:containerToContained
                                   forKey:entityType];
    }

    instances = [containerToContained objectForKey:container];

    if (nil == instances) {
        instances = [self calculateInstancesOfEntityType:entityType
                                             inContainer:container];
        [containerToContained setObject:instances forKey:container];
    }

    return [instances objectEnumerator];
}

- (PajeContainer *)newContainerForContainer:(PajeContainer *)container;
{
    id newContainer = [containerToNewContainer objectForKey:container];
    if (nil != newContainer)
        return newContainer;
    else
        return container;
}

- (PajeContainer *)containerForEntity:(id<PajeEntity>)entity;
{
    return [self newContainerForContainer:[inputComponent containerForEntity:entity]];
}

- (PajeContainer *)sourceContainerForEntity:(id<PajeLink>)entity;
{
    return [self newContainerForContainer:[inputComponent sourceContainerForEntity:entity]];
}

- (PajeContainer *)destContainerForEntity:(id<PajeLink>)entity;
{
    return [self newContainerForContainer:[inputComponent destContainerForEntity:entity]];
}

@end
