  /*
   * VirtualThread.m
   *
   * Component that re-numbers the threads in a trace, reusing
   * the numbers as the threads die.
   * Eve component for the Scope visualising tool.
   *
   * 19960514 BS  creation
   */

#import "VirtualThread.h"

@implementation VirtualThread

// methods for reutilisation of thread slots on the graph,
// by changing thread numbers.

- (id)init
{
    [super init];

    [self addOutput:[[EveOutputPort alloc]
                        initPortValueType:[NSObject class]]];
    [self addInput:[[EveAsyncInputPort alloc]
                        initPortValueType:[NSObject class]]];
    [self setAsynchronousFlush:YES];

    nodes = [[NSMutableDictionary dictionary] retain];

    // load the bundle. it initializes view.
    if (![NSBundle loadNibNamed:@"VirtualThread" owner:self])
        NSRunAlertPanel(NSStringFromClass([self class]),
                        @"Couldn't load interface file",
                        nil, nil, nil);

    defaultFilteringMode = [filteringModeSelector selectedTag];

    // view is an NSBox. we need its contents.
    view = [view contentView];
    [[view window] autorelease];
    [view removeFromSuperview];

    [[NSNotificationCenter defaultCenter] postNotificationName:@"PajeNewFilterInspectorNotification" object:self userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
                        NSStringFromClass([self class]), @"Name",
                        view, @"View", nil]];

    return self;
}

- (void)dealloc
{
    [trace release];
    [nodes release];
    [lastStartTime release];
    [lastEndTime release];
    [super dealloc];
}

- (void)reset
{
    [trace release];
    trace = nil;
    [nodes removeAllObjects];
    [nodeSelector removeAllItems];
    [nodeSelector addItemWithTitle:@"All"];
    [filteringModeSelector selectCellWithTag:defaultFilteringMode];
}

- (void)changeSelectedNode:(id)sender
{
    id nodeTitle = [sender titleOfSelectedItem];
    id node = [nodes objectForKey:nodeTitle];
    filterType filteringMode;

    if (!node)
        node = [nodes objectForKey:[NSNumber numberWithInt:[nodeTitle intValue]]];

    filteringMode = [node filteringMode];
    [filteringModeSelector selectCellWithTag:filteringMode];
}

- (void)changeFilteringMode:(id)sender
{
    id nodeTitle = [nodeSelector titleOfSelectedItem];
    id node;
    filterType filteringMode = [sender selectedTag];

    if ([nodeTitle isEqual:@"All"]) {
        id nodeEnum = [nodes objectEnumerator];
        defaultFilteringMode = filteringMode;
        while (node = [nodeEnum nextObject])
            [node setFilteringMode:filteringMode];
    } else {
        node = [nodes objectForKey:nodeTitle];
        if (!node)
            node = [nodes objectForKey:[NSNumber numberWithInt:[nodeTitle intValue]]];
        [node setFilteringMode:filteringMode];
    }

    // inform our successor that the trace has changed
    [[self outputNum:0] setPortValue:self];
}

- (void)handleAsynchronousInput:(id)aPort
{
    Trace *newTrace = [aPort portValue];
    NSDate *startTime = [[newTrace startTime] retain];
    NSDate *endTime = [[newTrace endTime] retain];
    NSEnumerator *nodeEnum, *entityEnum;
    id nodeId;
    id entity;
    VTNode *node;
    BOOL starting;
    id threadId, service;
//    static int nv=0, x;
//    static NSMutableArray *a1, *a2;
//    if (!a1) {
//        a1 = [[NSMutableArray alloc] init]; a2 = [[NSMutableArray alloc] init];
//    }
//    x=0;
    
    // see what portion of the trace must be analysed
    //BUG colocar um startTime e um endTime por nodo pra que isso funcione!
    //Trace ta retornando endtime grande sem ter
    if (trace != newTrace) {
        [trace release];
        trace = [newTrace retain];
        // let's forget everything and recalculate
        [nodes removeAllObjects]; // this should be in a reset notification
        [lastStartTime release];
        [lastEndTime release];
        lastStartTime = [startTime retain];
        lastEndTime = [endTime retain];
    } else {
        if ([startTime earlierDate:lastStartTime] == startTime) {
            // let's forget everything and recalculate
            [nodes removeAllObjects];
            [lastStartTime release];
            [lastEndTime release];
            lastStartTime = [startTime retain];
            lastEndTime = [endTime retain];
        } else if ([endTime laterDate:lastEndTime] == endTime) {
            // let's see only what's new at the end of the trace
            [startTime release];
            startTime = lastEndTime;
            lastEndTime = [endTime retain];
        } else {
            // nothing to do. make startTime == endTime (will be tested below)
            [startTime release];
            startTime = [endTime retain];
        }
    }

//    NSLog(@"Trace times:%@-%@ looking %@-%@", [trace startTime], [trace endTime], startTime, endTime);
    if (![startTime isEqualToDate:endTime]) {
        nodeEnum = [trace nodeEnumerator];
        while (nodeId = [nodeEnum nextObject]) {

//            x++;
            node = [nodes objectForKey:nodeId];
            if (nil == node) {
                node = [VTNode node];
                [node setFilteringMode:defaultFilteringMode];
                [nodes setObject:node forKey:nodeId];
                [nodeSelector addItemWithTitle:[nodeId description]];
            } 

            entityEnum = [trace enumeratorOfEntities:PajeEvent
                                              inNode:nodeId
                                            fromTime:startTime
                                              toTime:endTime];
            while (entity = [entityEnum nextObject]) {
//                if(x==1) NSLog(@"%@", [entity name]);
                if ([entity isThreadStartingOrFinishing:&starting]) {
                    [trace getNode:NULL   thread:&threadId
                         startTime:NULL  endTime:NULL
                             color:NULL ofEntity:entity];
                    service = [entity service];
                    if (starting) {
//                        if (nv<=4 && x==1) [a1 addObject:[entity time]];
//                        NSLog(@"starting thread n%@s%@t%@\n",nodeId, service, threadId);
                        [node startThread:threadId service:service];
                    } else {
//                        NSLog(@"ending thread n%@s%@t%@\n",nodeId, service, threadId);
                        [node endThread:threadId service:service];
                    }
//                    NSLog(@"node%@ = %@", nodeId, node);
                }
            }

#if 0
            //if (x==1) NSLog(@"a1=%@", a1);
            if (/*nv==4 && */x==1) {
            entityEnum = [trace enumeratorOfEntities:PajeEvent
                                              inNode:nodeId
                                            fromTime:[trace startTime]
                                              toTime:[trace endTime]];
            while (entity = [entityEnum nextObject]) {
                if ([entity isThreadStartingOrFinishing:&starting]) {
                    if (starting) {
                        if (nv<=4 && x==1) [a2 addObject:[entity time]];
                    }}}
            NSLog(@"Comparacao:%d\na1=%@\na2=%@", [a1 isEqual:a2], a1, a2);
            [a2 removeAllObjects];
            if(nv==4)exit(1);
            }
#endif
        }
    }

    [startTime release];
    [endTime release];

    [[self outputNum:0] setPortValue:self];
}


- (int)slotForThread:(id)threadId
             service:(id)service
                node:(id)nodeId
{
    VTNode *node = [nodes objectForKey:nodeId];

    if (nil != node) {
        return [node slotForThread:threadId service:service];
    } else {
        NSLog(@"%@: slot for thread %@ in unknown node %@",
              NSStringFromClass([self class]), threadId, nodeId);
        return 0;
    }
}



- (int)threadCountForNode:(id)nodeId
{
    VTNode *node = [nodes objectForKey:nodeId];

    if (nil != node) {
        if ([node filteringMode] == NO_FILTERING)
            return [trace threadCountForNode:nodeId];
        else
            return [node threadCount];
    } else {
        NSLog(@"%@: threadCount for unknown node %@",
              NSStringFromClass([self class]), nodeId);
        return 0;
    }
}

- (void)getNode:(id *)node
         thread:(id *)thread
      startTime:(NSDate **)start
        endTime:(NSDate **)end
          color:(NSColor **)color
       ofEntity:(id<PajeState>)entity
{
    [trace getNode:node
            thread:thread
         startTime:start
           endTime:end
             color:color
          ofEntity:entity];
    if (thread) {
        *thread = [NSNumber numberWithInt:[self slotForThread:*thread
                                                      service:[entity service]
                                                         node:*node]];        
    }
}

- (void)getSourceNode:(id *)sourceNode
             destNode:(id *)destNode
         sourceThread:(id *)sourceThread
           destThread:(id *)destThread
            startTime:(NSDate **)start
              endTime:(NSDate **)end
                color:(NSColor **)color
             ofEntity:(id<PajeComm>)entity
{
    [trace getSourceNode:sourceNode
                destNode:destNode
            sourceThread:sourceThread
              destThread:destThread
               startTime:start
                 endTime:end
                   color:color
                ofEntity:entity];
    if (sourceThread) {
        *sourceThread = [NSNumber numberWithInt:
            [self slotForThread:*sourceThread
                        service:[entity sourceService]
                           node:*sourceNode]];
    }
    if (destThread) {
        *destThread = [NSNumber numberWithInt:
            [self slotForThread:*destThread
                        service:[entity destService]
                           node:*destNode]];        
    }
}


- (BOOL)respondsToSelector:(SEL)aSelector
{
    return ([super respondsToSelector:aSelector]
            || [trace respondsToSelector:aSelector]);
}

- (void)forwardInvocation:(NSInvocation *)invocation
{
    [invocation invokeWithTarget:trace];
}

- (NSMethodSignature *)methodSignatureForSelector:(SEL)sel {
    NSMethodSignature *result = [super methodSignatureForSelector:sel];
    if (nil == result)
        result = [trace methodSignatureForSelector:sel];
    return result;
}

@end
